//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.Interface;
using Microsoft.Practices.ObjectBuilder;
using OHE.LCP.Admin.CourseCopying.Resources;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.UI;
using OHE.LCP.Admin.CourseCopying;
using OHE.FoundationModules.Email.ServiceProxy.EmailService;
using OHE.LCP.Admin.CourseCopying.Properties;
using OHE.FoundationModules.Impersonation;

namespace OHE.LCP.Admin.CourseCopying
{
    public class CourseCopyingController
    {
        private ICourseCopyService _courseCopyService;
        private IEmailServiceProxy _emailProxy;


        public CourseCopyingController()
        {
        }

        [InjectionConstructor]
        public CourseCopyingController([ServiceDependency] ICourseCopyService courseCopyService)
        {
            _courseCopyService = courseCopyService;
            _emailProxy = new EmailServiceProxy();
        }

        #region Master Course Methods
        public IList<MasterCourse> GetAllMasterCourses(bool getBannerCourses)
        {
            return _courseCopyService.GetAllMasterCourses(getBannerCourses);
        }

        public IList<MasterCourse> GetAllActiveMasterCourses(bool getBannerCourses)
        {
            return _courseCopyService.GetAllActiveMasterCourses(getBannerCourses);
        }

        public MasterCourse GetMasterCourse(int masterCourseID)
        {
            return _courseCopyService.GetMasterCourse(masterCourseID);
        }

        public void InsertMasterCourse(MasterCourse masterCourse)
        {
            _courseCopyService.InsertMasterCourse(masterCourse);
        }

        public void UpdateMasterCourse(MasterCourse masterCourse)
        {
            _courseCopyService.UpdateMasterCourse(masterCourse);
        }
        #endregion

        #region Source Node Methods
        public IList<SourceNode> GetAllSourceNodes()
        {
            return _courseCopyService.GetAllSourceNodes();
        }
        #endregion

        #region ECollege Term Methods
        public IList<ECollegeTerm> GetAllECollegeTerms()
        {
            return _courseCopyService.GetAllECollegeTerms();
        }

        public IList<ECollegeTerm> GetAllActiveECollegeTerms()
        {
            return _courseCopyService.GetAllActiveECollegeTerms();
        }

        public ECollegeTerm GetECollegeTerm(int termID)
        {
            return _courseCopyService.GetECollegeTerm(termID);
        }

        public void InsertECollegeTerm(ECollegeTerm ecollegeTerm)
        {
            _courseCopyService.InsertECollegeTerm(ecollegeTerm);
        }

        public void UpdateECollegeTerm(ECollegeTerm ecollegeTerm)
        {
            _courseCopyService.UpdateECollegeTerm(ecollegeTerm);
        }
        #endregion

        #region Banner Course Methods

        public IList<Course> GetAllBannerCourses()
        {
            return _courseCopyService.GetAllBannerCourses();
        }

        public Course GetBannerCourse(string courseCode)
        {
            return _courseCopyService.GetBannerCourse(courseCode);
        }

        public void MapMasterCourseBannerCourse(int masterCourseID, string bannerCourseCode)
        {
            _courseCopyService.MapMasterCourseBannerCourse(masterCourseID, bannerCourseCode);
        }

        public IList<Course> GetBannerCoursesByTermSubTermSchool(string term, string subTerm, string schoolName)
        {
            return _courseCopyService.GetBannerCoursesByTermSubTermSchool(term, subTerm, schoolName);
        }
        #endregion

        #region Banner Term Methods
        public IList<Term> GetAllTerms()
        {
            return _courseCopyService.GetAllTerms();
        }
        public IList<SubTerm> GetSubTermsByTerm(string term)
        {
            return _courseCopyService.GetSubTermsByTerm(term);
        }
        #endregion

        public IList<School> GetSchools()
        {
            return _courseCopyService.GetSchools();
        }

        public string GetNextBatchNumber()
        {
            return _courseCopyService.GetNextBatchNumber();
        }

        public IList<Batch> GetBatches()
        {
            return _courseCopyService.GetBatches();
        }

        #region Copy History Methods
        public void InsertCopyHistory(CopyHistory copyHistory)
        {
            _courseCopyService.InsertCopyHistory(copyHistory);
        }
        public IList<CopyHistory> GetCopyHistoryByBatchNumber(string batchNumber)
        {
            return _courseCopyService.GetCopyHistoryByBatchNumber(batchNumber);
        }
        #endregion

        public List<Course> GetNewSections()
        {
            return _courseCopyService.GetNewSections();
        }

        public List<Course> GetCoursesToBeCopied()
        {
            //Get the list of courses out of session
            List<Course> _courses = (List<Course>)HttpContext.Current.Session[Constants.SessionKeys.Courses];
            //Create a new course filter to get only the courses that are to be copied
            CourseToBeCopiedFinder _toBeCopiedFinder = new CourseToBeCopiedFinder();
            //Apply the filter
            return _courses.FindAll(_toBeCopiedFinder.Test);
        }

        public void ProcessCourseCopy()
        {
            //Get the next batch number
            string _batchNum = this.GetNextBatchNumber();
            //Create and save the file to be sent to eCollege
            CreateECollegeFile(_batchNum);
            //Save the history items
            SaveHistoryItems(_batchNum);
            //Send the email to CDL
            SendEmail(_batchNum);
            //Clean up the session items
            HttpContext.Current.Session[Constants.SessionKeys.Courses] = null;
            HttpContext.Current.Session[Constants.SessionKeys.SelectedECollegeTerm] = null;
            HttpContext.Current.Session[Constants.SessionKeys.SelectedTerm] = null;
            HttpContext.Current.Session[Constants.SessionKeys.SelectedSubTerm] = null;
            HttpContext.Current.Session[Constants.SessionKeys.SelectedSchool] = null;
            //Go to the completed page
            HttpContext.Current.Response.Redirect(Constants.URLs.Completed + "?batchnum=" + _batchNum);
        }

        private void CreateECollegeFile(string batchNumber)
        {
            //Get the selected eCollege Term out of session
            ECollegeTerm _eCollegeTerm = (ECollegeTerm)HttpContext.Current.Session[Constants.SessionKeys.SelectedECollegeTerm];

            //Get the selected courses out of session
            List<Course> _courses = (List<Course>)HttpContext.Current.Session[Constants.SessionKeys.Courses];
            //Create the filter to just get the courses that were selected by the user
            CourseToBeCopiedFinder _toBeCopiedFinder = new CourseToBeCopiedFinder();
            //Get just the selected courses
            List<Course> _selectedCourses = _courses.FindAll(_toBeCopiedFinder.Test);
            //Create a list to hold the output items
            List<OutputItem> _items = new List<OutputItem>();
            //Initialize variables
            //Get only the sections for the selected term and subterm
            SectionByTermSubTermFinder _sectionFinder = new SectionByTermSubTermFinder(HttpContext.Current.Session[Constants.SessionKeys.SelectedTerm].ToString(), HttpContext.Current.Session[Constants.SessionKeys.SelectedSubTerm].ToString());
            foreach (Course _course in _selectedCourses)
            {
                //Filter out the unneeded sections
                _course.Sections = ((List<Section>)_course.Sections).FindAll(_sectionFinder.Test);
                foreach (Section _section in _course.Sections)
                {
                    this.ProcessSectionForFile(_section, _course, _eCollegeTerm.eCollegeID, Settings.Default.SchoolCode, _items);
                }
            }

            this.SaveFile(batchNumber, _items);
        }

        private void SaveHistoryItems(string batchNumber)
        {
            //Get the selected eCollege Term out of session
            ECollegeTerm _eCollegeTerm = (ECollegeTerm)HttpContext.Current.Session[Constants.SessionKeys.SelectedECollegeTerm];

            //Get the selected courses out of session
            List<Course> _courses = (List<Course>)HttpContext.Current.Session[Constants.SessionKeys.Courses];
            //Create the filter to just get the courses that were selected by the user
            CourseToBeCopiedFinder _toBeCopiedFinder = new CourseToBeCopiedFinder();
            //Get just the selected courses
            List<Course> _selectedCourses = _courses.FindAll(_toBeCopiedFinder.Test);
            SectionByTermSubTermFinder _sectionFinder = new SectionByTermSubTermFinder(HttpContext.Current.Session[Constants.SessionKeys.SelectedTerm].ToString(), HttpContext.Current.Session[Constants.SessionKeys.SelectedSubTerm].ToString());
            foreach (Course _course in _selectedCourses)
            {
                //Set the sections for this course to only the sections for the term and subterm
                _course.Sections = ((List<Section>)_course.Sections).FindAll(_sectionFinder.Test);
                foreach (Section _section in _course.Sections)
                {
                    //Create a copyHistory item and save it to the database
                    CopyHistory _copyHistory = new CopyHistory(_course.CourseCode, _course.CourseTitle, _section.CRN, _section.GroupCode, "", _section.SchoolName, _section.SectionID, _section.SectionSubTerm.SubTermName, _section.SectionSubTerm.SubTermDescription, _section.SectionTerm.TermName, _section.SectionTerm.TermDescription, HttpContext.Current.User.Identity.Name, 0, DateTime.Now, _eCollegeTerm.eCollegeTerm, _eCollegeTerm.eCollegeID, _eCollegeTerm.TermID, _course.SelectedMasterCourse.CourseDisplayCode, _course.SelectedMasterCourse.eCollegeID, _course.SelectedMasterCourse.MasterCourseID, _section.SectionNode.Node, _course.SelectedMasterCourse.CourseTitle, batchNumber, _section.InstructorFirstName, _section.InstructorMiddleInitial, _section.InstructorLastName, _section.SectionCode, _section.CourseStartDate, false);
                    this.InsertCopyHistory(_copyHistory);
                }
            }
        }

        public void SendEmail(string batchNumber)
        {
            //Build the URL to be embedded in the email
            string Port = HttpContext.Current.Request.ServerVariables["SERVER_PORT"];
            if (Port == null || Port == "80" || Port == "443")
            { Port = ""; }
            else
            { Port = ":" + Port; }
            string Protocol = HttpContext.Current.Request.ServerVariables["SERVER_PORT_SECURE"];
            if (Protocol == null || Protocol == "0")
            { Protocol = "http://"; }
            else
            { Protocol = "https://"; }
            string basePath = String.Format("{0}{1}{2}{3}", Protocol, HttpContext.Current.Request.ServerVariables["SERVER_NAME"], Port, HttpContext.Current.Request.ApplicationPath);
            string _url = String.Format("{0}/CourseCopying/History.aspx?batch={1}", basePath, batchNumber);

            //Add in send email code
            SendCourseCopyingConfirmationEmailRequest request = new SendCourseCopyingConfirmationEmailRequest();
            CourseCopyingConfirmationValues values = new CourseCopyingConfirmationValues();
            values.BatchNo = batchNumber;
            values.URL = _url;
            request.Values = values;
            request.Partner = "Walden";
            request.SystemID = 35;
            try
            {
                this._emailProxy.SendCourseCopyingConfirmationEmail(request);
            }
            catch
            {
                // log error //
            }
        }

        public void OnConfirmBack()
        {
            //Go back to the Course Copy page
            HttpContext.Current.Response.Redirect(Constants.URLs.CourseCopy);
        }

        public Course GetBannerCourse(string courseCode, string subTerm, string term)
        {
            return _courseCopyService.GetBannerCourse(courseCode, subTerm, term);
        }

        public void Redirect(string url)
        {
            HttpContext.Current.Response.Redirect(url);
        }

        public void ProcessSectionForFile(Section section, Course course, string eCollegeTermID, string schoolCode, List<OutputItem> items)
        {
            OutputItem item = null;
            OutputItem searchItem = null;

            if (String.IsNullOrEmpty(section.GroupCode))
            {
                //Not part of a cross listed course
                //Create a new output item and populate it
                item = new OutputItem();
                item.ColC = String.Format("{0}.", section.SectionNode.Node);
                item.ColE = course.SelectedMasterCourse.eCollegeID;
                item.ColF = schoolCode;
                item.ColG = eCollegeTermID;
                item.ColH = String.Format("{0}.", section.SectionNode.Node);
                item.ColI = String.Format("{0}-{1}", course.CourseCode, section.SectionCode);
                item.ColK = String.Format("{0}-{1}", section.CRN, section.SectionTerm.TermName);
                item.ColR = String.Format("{0}-{1} {2}", course.CourseCode, section.SectionCode, course.SelectedMasterCourse.CourseTitle);
                //Add it to the list
                items.Add(item);
            }
            else
            {
                //Is part of a cross listed course
                //First find if there is an item with this group code already
                OutputItemByGroupCodeFinder finder = new OutputItemByGroupCodeFinder(section.GroupCode);
                searchItem = items.Find(finder.Test);
                if (searchItem == null)
                {
                    //No item found with this group code, so add this one
                    item = new OutputItem();
                    item.GroupCode = section.GroupCode;
                    item.ColC = String.Format("{0}.", section.SectionNode.Node);
                    item.ColE = course.SelectedMasterCourse.eCollegeID;
                    item.ColF = schoolCode;
                    item.ColG = eCollegeTermID;
                    item.ColH = String.Format("{0}.", section.SectionNode.Node);
                    item.ColI = String.Format("{0}-{1}", course.CourseCode, section.SectionCode);
                    item.ColK = String.Format("{0}-{1}-{2}", section.CRN, section.SectionTerm.TermName, section.GroupCode);
                    item.ColR = String.Format("{0}-{1} {2}", course.CourseCode, section.SectionCode, course.SelectedMasterCourse.CourseTitle);
                    //Add it to the list
                    items.Add(item);
                }
                else
                {
                    //Item found with this group code, so modify the current
                    searchItem.ColI = String.Format("{0}/{1}-{2}", searchItem.ColI, course.CourseCode, section.SectionCode);
                    if (searchItem.ColL.Equals(""))
                    {
                        searchItem.ColL = String.Format("{0}-{1}-{2}", section.CRN, section.SectionTerm.TermName, section.GroupCode);
                    }
                    else
                    {
                        if (searchItem.ColN.Equals(""))
                        {
                            searchItem.ColN = String.Format("{0}-{1}-{2}", section.CRN, section.SectionTerm.TermName, section.GroupCode);
                        }
                        else
                        {
                            if (searchItem.ColO.Equals(""))
                            {
                                searchItem.ColO = String.Format("{0}-{1}-{2}", section.CRN, section.SectionTerm.TermName, section.GroupCode);
                            }
                            else
                            {
                                searchItem.ColP = String.Format("{0}-{1}-{2}", section.CRN, section.SectionTerm.TermName, section.GroupCode);
                            }
                        }
                    }
                    searchItem.ColR = String.Format("{0} {1}", searchItem.ColI, course.SelectedMasterCourse.CourseTitle);
                }
            }
        }

        public void SaveFile(string batchNumber, List<OutputItem> items)
        {
            //String to hold the file contents
            StringBuilder sb = new StringBuilder();
            string delimiter = "";
            switch (Settings.Default.EcollegeFileDelimiter.ToLower())
            {
                case "tab":
                    delimiter = "\t";
                    break;
                case "comma":
                    delimiter = ",";
                    break;
                case "pipe":
                    delimiter = "|";
                    break;
            }


            //Now go through all of the OutputItems and create the string to write to the file
            foreach (OutputItem oItem in items)
            {
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColA, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColB, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColC, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColD, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColE, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColF, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColG, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColH, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColI, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColJ, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColK, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColL, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColM, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColN, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColO, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColP, delimiter);
                sb.AppendFormat("{0}{1}{0}{2}", Settings.Default.EcollegeFileStringsWrapper, oItem.ColQ, delimiter);
                sb.AppendFormat("{0}{1}{0}\r\n", Settings.Default.EcollegeFileStringsWrapper, oItem.ColR);
            }

            sb.Append("\r\n");

            //Set the path using the web.config setting
            string path = String.Format("{0}{1}{2}.{3}",
                                         Settings.Default.CourseCopyingFileSharePath,
                                         Settings.Default.EcollegeFileNamePrefix,
                                         batchNumber,
                                         Settings.Default.EcollegeFileExtention);

            //Create and save the file
            string credentials = Settings.Default.ImpersonationCredentials;
            if (!String.IsNullOrEmpty(credentials))
            {
                string[] credentialParts = credentials.Split("|".ToCharArray(), 3);
                if (credentialParts.Length == 3)
                {
                    ImpersonationManager mgr = new ImpersonationManager();
                    if (mgr.DoImpersonation(credentialParts[0], credentialParts[1], credentialParts[2]))
                    {
                        StreamWriter sw = File.CreateText(path);
                        sw.Write(sb.ToString());
                        sw.Close();
                        mgr.UndoImpersonation();
                    }
                    else
                    {
                        throw new Exception("Security Violation.  Unable to write Course Copy File.");
                    }
                }
                else
                {
                    throw new Exception("Security Violation.  Credentials not supplied for Course Copy file output.");
                }
            }

        }
    }
}
