//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.CompositeWeb.Interfaces;
using Microsoft.Practices.CompositeWeb.Services;
using Microsoft.Practices.CompositeWeb.Configuration;
using Microsoft.Practices.CompositeWeb.EnterpriseLibrary.Services;
using OHE.LCP.Admin.CourseCopying.Interface;
using OHE.LCP.Admin.CourseCopying.Services;

namespace OHE.LCP.Admin.CourseCopying
{
    public class CourseCopyingModuleInitializer : ModuleInitializer
    {
        private const string AuthorizationSection = "compositeWeb/authorization";

        public override void Load(ICompositionContainer moduleContainer)
        {
            base.Load(moduleContainer);

            AddGlobalServices(moduleContainer.Parent.Services);
            AddModuleServices(moduleContainer.Services);
            RegisterSiteMapInformation(moduleContainer.Services.Get<ISiteMapBuilderService>(true));
        }

        protected virtual void AddGlobalServices(IServiceCollection globalServices)
        {
            // TODO: add a service that will be visible to any module
        }

        protected virtual void AddModuleServices(IServiceCollection moduleServices)
        {
            // TODO: add a service that will be visible to this module
            moduleServices.AddNew<CourseCopyService, ICourseCopyService>();
        }

        protected virtual void RegisterSiteMapInformation(ISiteMapBuilderService siteMapBuilderService)
        {
            SiteMapNodeInfo moduleNode = new SiteMapNodeInfo("CourseCopying", "~/CourseCopying/Default.aspx", "Course Copying");
            siteMapBuilderService.AddNode(moduleNode, Constants.Permissions.AllowCourseCopying);

            SiteMapNodeInfo item0Node = new SiteMapNodeInfo("CourseCopy", "~/CourseCopying/CourseCopy.aspx", "Course Copying");
            siteMapBuilderService.AddNode(item0Node, moduleNode, Constants.Permissions.AllowCourseCopying, 1);

            SiteMapNodeInfo item1Node = new SiteMapNodeInfo("MasterCourses", "~/CourseCopying/MasterCourses.aspx", "Manage Master Courses");
            siteMapBuilderService.AddNode(item1Node, moduleNode, Constants.Permissions.AllowCourseCopying, 2);

            SiteMapNodeInfo item2Node = new SiteMapNodeInfo("Terms", "~/CourseCopying/Terms.aspx", "Manage eCollege Terms");
            siteMapBuilderService.AddNode(item2Node, moduleNode, Constants.Permissions.AllowCourseCopying, 3);

            SiteMapNodeInfo item3Node = new SiteMapNodeInfo("DefaultMapping", "~/CourseCopying/DefaultMapping.aspx", "Default Mapping");
            siteMapBuilderService.AddNode(item3Node, moduleNode, Constants.Permissions.AllowCourseCopying, 4);

            SiteMapNodeInfo item4Node = new SiteMapNodeInfo("History", "~/CourseCopying/History.aspx", "History");
            siteMapBuilderService.AddNode(item4Node, moduleNode, Constants.Permissions.AllowCourseCopying, 5);

            SiteMapNodeInfo item5Node = new SiteMapNodeInfo("NewSections", "~/CourseCopying/NewSections.aspx", "New Sections");
            siteMapBuilderService.AddNode(item5Node, moduleNode, Constants.Permissions.AllowCourseCopying, 6);
        }

        public override void Configure(IServiceCollection services, System.Configuration.Configuration moduleConfiguration)
        {
            IAuthorizationRulesService authorizationRuleService = services.Get<IAuthorizationRulesService>();
            if (authorizationRuleService != null)
            {
                AuthorizationConfigurationSection authorizationSection = moduleConfiguration.GetSection(AuthorizationSection) as AuthorizationConfigurationSection;
                if (authorizationSection != null)
                {
                    foreach (AuthorizationRuleElement ruleElement in authorizationSection.ModuleRules)
                    {
                        authorizationRuleService.RegisterAuthorizationRule(ruleElement.AbsolutePath, ruleElement.RuleName);
                    }
                }
            }
        }
    }
}