using System;
using System.Collections.Generic;
using System.Text;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;

namespace OHE.LCP.Admin.CourseCopying.Resources
{

    public class CourseToBeCopiedFinder
    {
        public CourseToBeCopiedFinder()
        {
        }
        public bool Test(Course _course)
        {
            return (_course.ToBeCopied);
        }
    }

    public class SectionByTermSubTermFinder
    {
        private string _term = "";
        private string _subterm = "";
        public SectionByTermSubTermFinder(string term, string subterm)
        {
            this._term = term;
            this._subterm = subterm;
        }
        public bool Test(Section _section)
        {
            return (_section.SectionTerm.TermName == _term && _section.SectionSubTerm.SubTermName == _subterm);
        }
    }

    public class OutputItemByGroupCodeFinder
    {
        private string _groupCode;
        public OutputItemByGroupCodeFinder(string groupCode)
        {
            _groupCode = groupCode;
        }
        public bool Test(OutputItem _item)
        {
            return (_item.GroupCode == _groupCode);
        }
    }

    public class CourseByCourseCodeFinder
    {
        private string _bannerCourseCode;
        public CourseByCourseCodeFinder(string courseCode)
        {
            _bannerCourseCode = courseCode;
        }
        public bool Test(Course _course)
        {
            return (_course.CourseCode == _bannerCourseCode);
        }
    }

    public class MasterCourseByCourseCodeFinder
    {
        private string _bannerCourseCode;
        public MasterCourseByCourseCodeFinder(string courseCode)
        {
            _bannerCourseCode = courseCode;
        }
        public bool Test(MasterCourse _masterCourse)
        {
            if (_masterCourse.BannerCourses == null)
            { return false; }
            else
            {
                if (_masterCourse.BannerCourses.Count < 1)
                { return false; }
                else
                {
                    bool retval = false;
                    foreach (Course _course in _masterCourse.BannerCourses)
                    {
                        if (_course.CourseCode == this._bannerCourseCode)
                        {
                            retval = true;
                            break;
                        }
                    }
                    return retval;
                }
            }
        }
    }

    public class CopyHistoryBySectionIDFinder
    {
        string _sectionID = "";
        public CopyHistoryBySectionIDFinder(string sectionID)
        {
            _sectionID = sectionID;
        }
        public bool Test(CopyHistory copyHistory)
        {
            return (copyHistory.BannerSectionID == _sectionID);
        }
    }

    public class CopyHistoryByCourseTermSubTermFinder
    {
        string _courseCode = "";
        string _term = "";
        string _subTerm = "";
        public CopyHistoryByCourseTermSubTermFinder(string courseCode, string term, string subTerm)
        {
            _courseCode = courseCode;
            _term = term;
            _subTerm = subTerm;
        }
        public bool Test(CopyHistory copyHistory)
        {
            return (copyHistory.BannerCourseCode == _courseCode && copyHistory.BannerTerm == _term && copyHistory.BannerSubTerm == _subTerm);
        }
    }

    public class SectionByCourseCodeFinder
    {
        string _courseCode = "";
        public SectionByCourseCodeFinder(string courseCode)
        {
            _courseCode = courseCode;
        }
        public bool Test(Section section)
        {
            return (section.CourseCode == _courseCode);
        }
    }

    public class TermFinder
    {
        string _term = "";
        public TermFinder(string term)
        {
            _term = term;
        }
        public bool Test(Term term)
        {
            return (term.TermName == _term);
        }
    }

    public class SubTermFinder
    {
        string _subTerm = "";
        public SubTermFinder(string subterm)
        {
            _subTerm = subterm;
        }
        public bool Test(SubTerm subTerm)
        {
            return (subTerm.SubTermName == _subTerm);
        }
    }
}
