using System;
using System.Collections.Generic;
using System.Text;
using OHE.LCP.Admin.CourseCopying.Interface;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess;
using OHE.LCP.Admin.CourseCopying.Resources;

namespace OHE.LCP.Admin.CourseCopying.Services
{

    class CourseCopyService: ICourseCopyService
    {
        private MasterCourseRepository _masterCourseRepository = new MasterCourseRepository("CourseCopyConn");
        private SourceNodeRepository _sourceNodeRepository = new SourceNodeRepository("BannerConnection");
        private ECollegeTermRepository _ecollegeTermRepository = new ECollegeTermRepository("CourseCopyConn");
        private CourseRepository _courseRepository = new CourseRepository("BannerConnection");
        private BannerCourseStubRepository _bannerCourseStubRepository = new BannerCourseStubRepository("CourseCopyConn");
        private TermRepository _termRepository = new TermRepository("BannerConnection");
        private SubTermRepository _subTermRepository = new SubTermRepository("BannerConnection");
        private CopyHistoryRepository _copyHistoryRepository = new CopyHistoryRepository("CourseCopyConn");
        private SchoolRepository _schoolRepository = new SchoolRepository("BannerConnection");
        private SectionRepository _sectionRepository = new SectionRepository("BannerConnection");
        private BatchRepository _batchRepository = new BatchRepository("CourseCopyConn");

        #region ICourseCopyService Members

        #region Master Course Methods
        public IList<MasterCourse> GetAllMasterCourses(bool getBannerCourses)
        {
            //Get the list of all master courses
            IList<MasterCourse> _masterCourses = _masterCourseRepository.GetAllFromMasterCourse();
            if (getBannerCourses)
            {
                //Get the list of all banner courses
                List<Course> _bannerCourses = _courseRepository.GetAllFromCourse();
                //Create the list of stubs
                IList<BannerCourseStub> _bannerCourseStubs;
                //Iterate through all of the master courses
                foreach (MasterCourse _masterCourse in _masterCourses)
                {
                    //Get the list of BannerCourseStubs for each master course
                    _bannerCourseStubs = _bannerCourseStubRepository.GetMasterCourseBannerCourseByMasterCourseID(_masterCourse.MasterCourseID);
                    //Iterate through the stubs and find the correct Banner course from the complete list of banner courses and add it to the master courses list
                    foreach (BannerCourseStub _stub in _bannerCourseStubs)
                    {
                        CourseByCourseCodeFinder _courseFinder = new CourseByCourseCodeFinder(_stub.BannerCourseCode);
                        Course _course = _bannerCourses.Find(_courseFinder.Test);
                        if (_course != null)
                        {
                            _masterCourse.BannerCourses.Add(_course);
                        }
                    }
                }
            }
            return _masterCourses;
        }

        public IList<MasterCourse> GetAllActiveMasterCourses(bool getBannerCourses)
        {
            IList<MasterCourse> _masterCourses = _masterCourseRepository.GetAllActiveFromMasterCourse();
            if (getBannerCourses)
            {
                //Get the list of all banner courses
                List<Course> _bannerCourses = _courseRepository.GetAllFromCourse();
                IList<BannerCourseStub> _bannerCourseStubs;
                //Iterate through all of the master courses
                foreach (MasterCourse _masterCourse in _masterCourses)
                {
                    //Get the list of BannerCourseStubs for each master course
                    _bannerCourseStubs = _bannerCourseStubRepository.GetMasterCourseBannerCourseByMasterCourseID(_masterCourse.MasterCourseID);
                    //Iterate through the stubs and find the correct Banner course from the complete list of banner courses and add it to the master courses list
                    foreach (BannerCourseStub _stub in _bannerCourseStubs)
                    {
                        CourseByCourseCodeFinder _courseFinder = new CourseByCourseCodeFinder(_stub.BannerCourseCode);
                        Course _course = _bannerCourses.Find(_courseFinder.Test);
                        if (_course != null)
                        {
                            _masterCourse.BannerCourses.Add(_course);
                        }
                    }
                }
            }
            return _masterCourses;
        }

        public MasterCourse GetMasterCourse(int masterCourseID)
        {
            return _masterCourseRepository.GetMasterCourseByMasterCourseID(masterCourseID);
        }

        public void InsertMasterCourse(MasterCourse masterCourse)
        {
            _masterCourseRepository.Add(masterCourse);
        }

        public void UpdateMasterCourse(MasterCourse masterCourse)
        {
            _masterCourseRepository.Save(masterCourse);
        }
        #endregion

        #region Source Node Methods
        public IList<SourceNode> GetAllSourceNodes()
        {
            return _sourceNodeRepository.GetAllFromSourceNode();
        }
        #endregion

        #region ECollege Term Methods
        public IList<ECollegeTerm> GetAllECollegeTerms()
        {
            return _ecollegeTermRepository.GetAllFromECollegeTerm();
        }

        public IList<ECollegeTerm> GetAllActiveECollegeTerms()
        {
            return _ecollegeTermRepository.GetAllActiveECollegeTerm();
        }

        public ECollegeTerm GetECollegeTerm(int TermID)
        {
            return _ecollegeTermRepository.GetECollegeTermByTermID(TermID);
        }

        public void InsertECollegeTerm(ECollegeTerm ecollegeTerm)
        {
            _ecollegeTermRepository.Add(ecollegeTerm);
        }

        public void UpdateECollegeTerm(ECollegeTerm ecollegeTerm)
        {
            _ecollegeTermRepository.Save(ecollegeTerm);
        }
        #endregion

        #region Banner Course Methods
        public IList<Course> GetAllBannerCourses()
        {
            List<MasterCourse> _masterCourses = (List<MasterCourse>)this.GetAllActiveMasterCourses(true);
            IList<Course> _bannerCourses = _courseRepository.GetAllFromCourse();
            //Get all of the sections at once
            List<Section> _bannerSections = _sectionRepository.GetAllFromSection();
            //Get all of the terms
            List<Term> _bannerTerms = _termRepository.GetAllFromTerm();
            //Get all of the subterms
            List<SubTerm> _bannerSubTerms = _subTermRepository.GetAllFromSubTerm();
            foreach (Course _course in _bannerCourses)
            {
                //Add in the matching sections
                SectionByCourseCodeFinder _finder = new SectionByCourseCodeFinder(_course.CourseCode);
                List<Section> _sections = _bannerSections.FindAll(_finder.Test);
                _course.Sections = _sections;
                foreach (Section _section in _course.Sections)
                {
                    //Add in the term
                    TermFinder _termFinder = new TermFinder(_section.TermString);
                    _section.SectionTerm = _bannerTerms.Find(_termFinder.Test);
                    //Add in the subterm
                    SubTermFinder _subTermFinder = new SubTermFinder(_section.SubTermString);
                    _section.SectionSubTerm = _bannerSubTerms.Find(_subTermFinder.Test);
                }
                //Search to see if there is a Master Course that has this Course's Course Code
                MasterCourseByCourseCodeFinder _masterCourseFinder = new MasterCourseByCourseCodeFinder(_course.CourseCode);
                List<MasterCourse> _matchingMasterCourses = _masterCourses.FindAll(_masterCourseFinder.Test);

                if (_matchingMasterCourses.Count == 1)
                {
                    _course.Master_Course = _matchingMasterCourses[0];
                }
            }
            return _bannerCourses;
        }

        public Course GetBannerCourse(string courseCode)
        {
            Course _course = _courseRepository.GetCourseByCourseCode(courseCode);
            List<MasterCourse> _masterCourses = (List<MasterCourse>)this.GetAllActiveMasterCourses(true);

            //Search to see if there is a Master Course that has this Course's Course Code
            MasterCourseByCourseCodeFinder _masterCourseFinder = new MasterCourseByCourseCodeFinder(_course.CourseCode);
            List<MasterCourse> _matchingMasterCourses = _masterCourses.FindAll(_masterCourseFinder.Test);
            if (_matchingMasterCourses.Count == 1)
            {
                _course.Master_Course = _matchingMasterCourses[0];
            }

            return _course;
        }

        public Course GetBannerCourse(string courseCode, string subTerm, string term)
        {
            Course _course = _courseRepository.GetCourseByCourseCode(courseCode);
            List<MasterCourse> _masterCourses = (List<MasterCourse>)this.GetAllActiveMasterCourses(true);

            //Search to see if there is a Master Course that has this Course's Course Code
            MasterCourseByCourseCodeFinder _masterCourseFinder = new MasterCourseByCourseCodeFinder(_course.CourseCode);
            List<MasterCourse> _matchingMasterCourses = _masterCourses.FindAll(_masterCourseFinder.Test);
            if (_matchingMasterCourses.Count == 1)
            {
                _course.Master_Course = _matchingMasterCourses[0];
            }

            //Populate the list of CopyHistoryItems
            _course.CopyHistoryItems = _copyHistoryRepository.GetCopyHistoryByCourseTermSubTerm(_course.CourseCode, subTerm, term);

            return _course;
        }

        public void MapMasterCourseBannerCourse(int masterCourseID, string bannerCourseCode)
        {
            BannerCourseStub _newStub = new BannerCourseStub(bannerCourseCode, masterCourseID);
            _bannerCourseStubRepository.Add(_newStub);
        }

        public IList<Course> GetBannerCoursesByTermSubTermSchool(string term, string subTerm, string schoolName)
        {
            //First set the values to null if necessary.  The stored proc is expecting null values
            if (term == "")
            {
                term = null;
            }

            if (subTerm == "")
            {
                subTerm = null;
            }

            if (schoolName == "")
            {
                schoolName = null;
            }

            //Get the list of banner courses from the database
            IList<Course> _bannerCourses = _courseRepository.GetCourseByTermSubTermSchool(term, subTerm, schoolName);
            //Get a list of all of the sections
            List<Section> _bannerSections = _sectionRepository.GetAllFromSection();
            //Get all of the terms
            List<Term> _bannerTerms = _termRepository.GetAllFromTerm();
            //Get all of the subterms
            List<SubTerm> _bannerSubTerms = _subTermRepository.GetAllFromSubTerm();

            //Get a list of all active master courses
            List<MasterCourse> _masterCourses = (List<MasterCourse>)this.GetAllActiveMasterCourses(true);
            //Go through and set the Master_Course property for each course to the default Master course
            foreach (Course _course in _bannerCourses)
            {
                //Add in the matching sections
                SectionByCourseCodeFinder _finder = new SectionByCourseCodeFinder(_course.CourseCode);
                List<Section> _sections = _bannerSections.FindAll(_finder.Test);
                _course.Sections = _sections;

                foreach (Section _section in _course.Sections)
                {
                    //Add in the term
                    TermFinder _termFinder = new TermFinder(_section.TermString);
                    _section.SectionTerm = _bannerTerms.Find(_termFinder.Test);
                    //Add in the subterm
                    SubTermFinder _subTermFinder = new SubTermFinder(_section.SubTermString);
                    _section.SectionSubTerm = _bannerSubTerms.Find(_subTermFinder.Test);
                }

                //Search to see if there is a Master Course that has this Course's Course Code
                MasterCourseByCourseCodeFinder _masterCourseFinder = new MasterCourseByCourseCodeFinder(_course.CourseCode);
                List<MasterCourse> _matchingMasterCourses = _masterCourses.FindAll(_masterCourseFinder.Test);

                if (_matchingMasterCourses.Count == 1)
                {
                    _course.Master_Course = _matchingMasterCourses[0];
                }

                //Populate the list of CopyHistoryItems
                _course.CopyHistoryItems = _copyHistoryRepository.GetCopyHistoryByCourseTermSubTerm(_course.CourseCode, subTerm, term);

                //Now add in all of the sections for this course
                //_course.Sections = _sectionRepository.GetSectionByCourseCode(_course.CourseCode);
            }            

            return _bannerCourses;
        }
        #endregion

        #region Banner Term Methods
        public IList<Term> GetAllTerms()
        {
            return _termRepository.GetAllFromTerm();
        }
        public IList<SubTerm> GetSubTermsByTerm(string term)
        {
            return _subTermRepository.GetSubTermByTerm(term);
        }
        #endregion

        public IList<School> GetSchools()
        {
            return _schoolRepository.GetSchools();
        }

        private int StringToInt(string _inputString)
        {
            if (_inputString.Substring(0, 1) == "0")
            {
                //Leading zero
                return Convert.ToInt32(_inputString.Substring(1, 1));
            }
            else 
            {
                //Nonleading zero
                return Convert.ToInt32(_inputString);
            }
        }

        private string IntToString(int _inputInt)
        {
            if (_inputInt < 10)
            {
                return "0" + _inputInt.ToString();
            }
            else 
            {
                return _inputInt.ToString();
            }
        }

        public string GetNextBatchNumber()
        {
            string _nextBatchNumber = "";
            CopyHistory _copyHistItem = _copyHistoryRepository.GetLatestBatchNumber();
            if (_copyHistItem == null)
            {
                //There is no latest batch number.  So create one
                _nextBatchNumber = DateTime.Now.Year.ToString() + IntToString(DateTime.Now.Month) + IntToString(DateTime.Now.Day) + "01";
            }
            else 
            {
                //There is a latest batch number.  If the batch number is from today then increment by one.
                //Otherwise create a new one for today
                _nextBatchNumber = _copyHistItem.BatchNumber;
                if(_nextBatchNumber.Substring(0,4) == DateTime.Now.Year.ToString() && StringToInt(_nextBatchNumber.Substring(4,2)) == DateTime.Now.Month && StringToInt(_nextBatchNumber.Substring(6,2)) == DateTime.Now.Day)
                {
                    _nextBatchNumber = DateTime.Now.Year.ToString() + IntToString(DateTime.Now.Month) + IntToString(DateTime.Now.Day) + IntToString(StringToInt(_nextBatchNumber.Substring(8,2))+1);
                }
                else
                {
                    _nextBatchNumber = DateTime.Now.Year.ToString() + IntToString(DateTime.Now.Month) + IntToString(DateTime.Now.Day) + "01";
                }
            }
            return _nextBatchNumber;
        }

        public IList<Batch> GetBatches()
        {
            return _batchRepository.GetBatches();
        }

        #region Copy History Methods
        public void InsertCopyHistory(CopyHistory copyHistory)
        {
            _copyHistoryRepository.Add(copyHistory);
        }
        public IList<CopyHistory> GetCopyHistoryByBatchNumber(string batchNumber)
        {
            return _copyHistoryRepository.GetCopyHistoryByBatchNumber(batchNumber);
        }
        #endregion


        public List<Course> GetNewSections()
        {
            //This method gets all of the new sections that have been added to Banner after
            //the course has already been copied to eCollege.  It returns a list of course objects
            //that contain all of the new sections

            //First get all of the sections from Banner
            IList<Section> _allSections = _sectionRepository.GetAllFromSection();

            //Now get all of the Copy History items
            List<CopyHistory> _allCopyHistory = _copyHistoryRepository.GetAllFromCopyHistory();

            //Specifically we want to find only sections that have not been copied but who have
            //other sections from the same course (and same term) that have been copied

            //Set up all of the variables that we will need
            IList<Section> _uncopiedSections = new List<Section>();
            IList<Section> _newSections = new List<Section>();
            CopyHistoryBySectionIDFinder _copyHistoryBySectionFinder;
            CopyHistoryByCourseTermSubTermFinder _copyHistoryByCourseTermSubTermFinder;
            List<CopyHistory> _CopyHistorySearchResults;
            List<CopyHistory> _CopyHistoryUncopied;

            //The first step is to filter out all sections that have already been copied
            foreach (Section _section in _allSections)
            {
                //Find out if the section is in allCopyHistory
                _copyHistoryBySectionFinder = new CopyHistoryBySectionIDFinder(_section.SectionID);
                _CopyHistorySearchResults = _allCopyHistory.FindAll(_copyHistoryBySectionFinder.Test);
                if (_CopyHistorySearchResults.Count == 0)
                {
                    //No sections were found that match the same section id in copy history
                    //If the section is not in allCopyHistory then add it to the uncopiedSections list
                    _uncopiedSections.Add(_section);
                }
            }

            //Now we need to look through all of the unCopiedSections and find out which of these 
            //have other sections from the same course, term and subterm that have already been copied
            foreach (Section _section2 in _uncopiedSections)
            {
                //_filter2 = new SectionFilter2(_section2.SectionCourse.CourseCode, _section2.SectionTerm.TermName, _section2.SectionSubTerm.SubTermName);
                _copyHistoryByCourseTermSubTermFinder = new CopyHistoryByCourseTermSubTermFinder(_section2.CourseCode, _section2.TermString, _section2.SubTermString);
                _CopyHistoryUncopied = _allCopyHistory.FindAll(_copyHistoryByCourseTermSubTermFinder.Test);
                if (_CopyHistoryUncopied.Count > 0)
                {
                    //This section (which we know has not been copied) has other sections in the history that share the same
                    //course code, term and subterm - so it needs to be added to the list of new sections
                    _section2.HistoryECollegeTerm = _CopyHistoryUncopied[0].ECollegeTerm;
                    _section2.HistoryECollegeTermID = _CopyHistoryUncopied[0].ECollegeTermECollegeID;
                    _section2.HistoryECollegeID = Convert.ToInt32(_CopyHistoryUncopied[0].ECollegeTermID);
                    _newSections.Add(_section2);
                }
            }


            //Now we need to transform this list of sections in to a list of courses
            List<Course> _newCourses = new List<Course>();
            CourseByCourseCodeFinder _finder;
            Course _testCourse;

            if (_newSections.Count > 0)
            {

                //Need to add in all of the terms and sub terms for the sections

                //Get all of the terms
                List<Term> _bannerTerms = _termRepository.GetAllFromTerm();
                //Get all of the subterms
                List<SubTerm> _bannerSubTerms = _subTermRepository.GetAllFromSubTerm();

                foreach (Section _section in _newSections)
                {
                    //Add in the term
                    TermFinder _termFinder = new TermFinder(_section.TermString);
                    _section.SectionTerm = _bannerTerms.Find(_termFinder.Test);
                    //Add in the subterm
                    SubTermFinder _subTermFinder = new SubTermFinder(_section.SubTermString);
                    _section.SectionSubTerm = _bannerSubTerms.Find(_subTermFinder.Test);
                }

                List<MasterCourse> _masterCourses = (List<MasterCourse>)this.GetAllActiveMasterCourses(true);

                foreach (Section _section3 in _newSections)
                {
                    //First check and see if the proper course already exists in the list
                    _finder = new CourseByCourseCodeFinder(_section3.CourseCode);
                    _testCourse = _newCourses.Find(_finder.Test);
                    if (_testCourse != null)
                    {
                        //This course already exists in the list of new courses
                        //So we should add this section to the course
                        _testCourse.Sections.Add(_section3);
                    }
                    else
                    {
                        //This course does not exist in the list
                        //So we should add this course and then add the section
                        //Course _newCourse = new Course(_section3.SectionCourse.CourseCode, _section3.SectionCourse.CourseTitle);
                        Course _newCourse = this.GetBannerCourse(_section3.CourseCode);
                        //Now we need to add the default master course for this banner course
                        MasterCourseByCourseCodeFinder _masterCourseFinder = new MasterCourseByCourseCodeFinder(_newCourse.CourseCode);
                        List<MasterCourse> _matchingMasterCourses = _masterCourses.FindAll(_masterCourseFinder.Test);

                        if (_matchingMasterCourses.Count == 1)
                        {
                            _newCourse.Master_Course = _matchingMasterCourses[0];
                        }
                        //Add the new section
                        _newCourse.Sections.Add(_section3);
                        _newCourses.Add(_newCourse);
                    }
                }
            }

            return _newCourses;
        }
        #endregion
    }
}
