﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using OHE.LCP.Admin.CourseCopying.Resources;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying;

namespace OHE.LCP.Admin.CourseCopying.Views
{
    public class CourseCopyPresenter : Presenter<ICourseCopy>
    {

        private CourseCopyingController _controller;
        public CourseCopyPresenter([CreateNew] CourseCopyingController controller)
        {
            _controller = controller;
        }

        public override void OnViewLoaded()
        {
        }

        public override void OnViewInitialized()
        {
            List<ECollegeTerm> terms = (List<ECollegeTerm>)_controller.GetAllActiveECollegeTerms();
            terms.Sort(delegate(ECollegeTerm t1, ECollegeTerm t2) { return t2.CourseStartDate.CompareTo(t1.CourseStartDate); });

            this.View.ECollegeTerms = terms;
            //Get the schools
            this.View.Schools = _controller.GetSchools();
            //Get the terms
            this.View.Terms = _controller.GetAllTerms();

            string _selectedTerm = "";
            //Set the selected term if necessary
            if (HttpContext.Current.Session[Constants.SessionKeys.SelectedTerm] != null)
            { _selectedTerm = HttpContext.Current.Session[Constants.SessionKeys.SelectedTerm].ToString(); }
            //Get the sub terms
            this.View.SubTerms = _controller.GetSubTermsByTerm(_selectedTerm);
            if (HttpContext.Current.Session[Constants.SessionKeys.Courses] != null)
            {
                //Get the courses out of session
                IList<Course> _courses = (IList<Course>)HttpContext.Current.Session[Constants.SessionKeys.Courses];
                this.View.Courses = _courses;
            }

        }

        public void OnGetCourses(string term, string subTerm, string schoolName)
        {
            this.View.Courses = _controller.GetBannerCoursesByTermSubTermSchool(term, subTerm, schoolName);
        }

        public void OnGetTerms()
        {
            this.View.Terms = _controller.GetAllTerms();
        }

        public void OnGetSubTerms(string term)
        {
            this.View.SubTerms = _controller.GetSubTermsByTerm(term);
        }

        public void OnGetSchools()
        {
            this.View.Schools = _controller.GetSchools();
        }

        public void OnUpdateClick(string selectedTerm, string selectedSubTerm, string SelectedSchool)
        {
            //Save the selections to session
            HttpContext.Current.Session[Constants.SessionKeys.SelectedTerm] = selectedTerm;
            HttpContext.Current.Session[Constants.SessionKeys.SelectedSubTerm] = selectedSubTerm;
            HttpContext.Current.Session[Constants.SessionKeys.SelectedSchool] = SelectedSchool;
            //Get the courses
            this.View.Courses = _controller.GetBannerCoursesByTermSubTermSchool(selectedTerm, selectedSubTerm, SelectedSchool);
        }

        public void OnTermSelected(string selectedTerm)
        {
            //Clear the selected sub term
            HttpContext.Current.Session[Constants.SessionKeys.SelectedSubTerm] = null;
            //Get the sub terms based on the selected term
            this.View.SubTerms = _controller.GetSubTermsByTerm(selectedTerm);
        }

        public void OnCopyClick(string selectedECollegeTermValue, string selectedECollegeTermDescription)
        {

            HttpContext.Current.Session[Constants.SessionKeys.SelectedECollegeTerm] = new ECollegeTerm(selectedECollegeTermValue, selectedECollegeTermDescription, true, 0, DateTime.Now);

            //Mark all of the items that were checked, put the courses in to session and then go to the confirm page
            List<Course> _courses = (List<Course>)HttpContext.Current.Session[Constants.SessionKeys.Courses];
            Course _course = null;
            string _courseCode = string.Empty;
            string keyPart = string.Empty;

            Telerik.Web.UI.RadGrid _coursesGrid = this.View.CourseGrid;

            foreach (Telerik.Web.UI.GridDataItem _row in _coursesGrid.Items)
            {
                //Get the course code from the datakeys EX: "{CourseCode:\"EDUC-1234\"}" 
                keyPart = _row.KeyValues.Split(":".ToCharArray(), 2)[1];
                _courseCode = keyPart.Substring(1, keyPart.Length - 3);
                //Create a predicate to find the right course in the list of courses
                CourseByCourseCodeFinder _courseFinder = new CourseByCourseCodeFinder(_courseCode);
                //Find the course that has this course code
                _course = _courses.Find(_courseFinder.Test);
                //Set the ToBeCopied property
                _course.ToBeCopied = _row.Selected;
            }

            //Put the courses back in session
            HttpContext.Current.Session[Constants.SessionKeys.Courses] = _courses;
            //Go to the confirmation page
            HttpContext.Current.Response.Redirect(Constants.URLs.Confirm);
        }

        public void GetBannerCourseCopyHistory(string courseCode, string subTerm, string term)
        {
            this.View.History =_controller.GetBannerCourse(courseCode, subTerm, term).CopyHistoryItems;
        }
    }
}




