﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Web;
using System.Web.UI;

namespace OHE.LCP.Admin.CourseCopying.Views
{
    public class MasterCoursesPresenter : Presenter<IMasterCourses>
    {

        private CourseCopyingController _controller;

        public MasterCoursesPresenter([CreateNew] CourseCopyingController controller)
         {
         		_controller = controller;
         }

        public override void OnViewLoaded()
        {
            this.View.MasterCourses = _controller.GetAllMasterCourses(false);
        }

        public override void OnViewInitialized()
        {
        }

        public void OnGetMasterCourse(int masterCourseID)
        {
            this.View.masterCourse = _controller.GetMasterCourse(masterCourseID);
        }

        public void OnInsertMasterCourse(MasterCourse masterCourse)
        {
            _controller.InsertMasterCourse(masterCourse);
            _controller.Redirect(Constants.URLs.MasterCourses);
        }

        public void OnUpdateMasterCourse(MasterCourse masterCourse)
        {
            //Get another master course with the same ID.  This is because we need to see if this master course is affiliated with any banner courses
            MasterCourse _testMasterCourse = this.GetMasterCourse(masterCourse.MasterCourseID);

            bool _oldIsActive = (bool)HttpContext.Current.Session[Constants.SessionKeys.OldIsActive];

            //Check to see if the user has updated a master course that is mapped to one or more banner courses
            //and they have also deactivated it.
            if (masterCourse.IsActive == false && _oldIsActive == true && _testMasterCourse.BannerCourses != null && _testMasterCourse.BannerCourses.Count > 0)
            {
                //Set the warning information
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseDisplayCode] = masterCourse.CourseDisplayCode;
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseTitle] = masterCourse.CourseTitle;
            }
            else 
            { 
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseDisplayCode] = string.Empty;
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseTitle] = string.Empty;
            }

            _controller.UpdateMasterCourse(masterCourse);

            //Check to see if the master course display code has a value.  If so, then set the warning values
            if (HttpContext.Current.Session[Constants.SessionKeys.MasterCourseDisplayCode] != null && HttpContext.Current.Session[Constants.SessionKeys.MasterCourseDisplayCode].ToString() != "")
            {
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseWarningDisplayCode] = HttpContext.Current.Session[Constants.SessionKeys.MasterCourseDisplayCode];
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseWarningTitle] = HttpContext.Current.Session[Constants.SessionKeys.MasterCourseTitle];
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseDisplayCode] = string.Empty;
                HttpContext.Current.Session[Constants.SessionKeys.MasterCourseTitle] = string.Empty;
            }
            this._controller.Redirect(Constants.URLs.MasterCourses);
        }

        public MasterCourse GetMasterCourse(int masterCourseID)
        {
            return _controller.GetMasterCourse(masterCourseID);
        }
    }
}




