﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Web;
using OHE.LCP.Admin.CourseCopying.Resources;
using System.Web.UI.WebControls;
using System.IO;
using System.Configuration;

namespace OHE.LCP.Admin.CourseCopying.Views
{
    public class NewSectionsPresenter : Presenter<INewSections>
    {

        private CourseCopyingController _controller;
        public NewSectionsPresenter([CreateNew] CourseCopyingController controller)
        {
            _controller = controller;
        }

        public override void OnViewLoaded()
        {
            // TODO: Implement code that will be executed every time the view loads
        }

        public override void OnViewInitialized()
        {
            this.View.NewSections = _controller.GetNewSections();
        }

        public void OnCopyNewSections()
        {
            //Get the next batch number
            string _batchNum = _controller.GetNextBatchNumber();

            //Create and save the file to be sent to eCollege
            CreateFile(_batchNum);

            //Save the history items
            SaveHistoryItems(_batchNum);

            //Send the email to CDL
            this.SendEmail(_batchNum);

            //Clean up the session items
            HttpContext.Current.Session[Constants.SessionKeys.NewCourses] = null;

            //Go to the completed page
            string url = Constants.URLs.Completed + "?batchnum=" + _batchNum;
            this._controller.Redirect(url);
        }

        // NewSectionPresenter
        private void CreateFile(string batchNumber)
        {

            //Get the selected courses out of session
            List<Course> _selectedCourses = (List<Course>)HttpContext.Current.Session[Constants.SessionKeys.NewCourses];
            //Create a list to hold the output items
            List<OutputItem> _items = new List<OutputItem>();
            //Initialize variables
            bool _checkFlag = false;
            string dataKey = this.View.SectionsGrid.MasterTableView.DataKeyNames[0];
            //Iterate through the courses
            foreach (Course _course in _selectedCourses)
            {
                //Iterate through the sections
                foreach (Section _section in _course.Sections)
                {
                    _checkFlag = false;
                    //Only process sections that were NOT checked to handle manually
                    foreach (Telerik.Web.UI.GridDataItem _row in this.View.SectionsGrid.Items)
                    {
                        if (this.View.SectionsGrid.MasterTableView.DataKeyValues[_row.ItemIndex][dataKey].ToString() == _section.SectionID.ToString())
                        {
                            //This is the grid row for the current section
                            //Check to see if the checkbox is selected
                            CheckBox _chk = (CheckBox)_row.FindControl("chkManual");
                            if (_chk.Checked)
                            {
                                _checkFlag = true;
                            }
                            break;
                        }
                    }
                    //Only process items that are NOT checked
                    if (!_checkFlag)
                    {

                        _controller.ProcessSectionForFile(_section, _course, _section.HistoryECollegeTermID, String.Format("{0}.", _section.SectionNode.Node), _items); 
                    }
                }
            }

            _controller.SaveFile(batchNumber, _items);
        }

        private void SaveHistoryItems(string batchNumber)
        {
            bool _checkFlag = false;
            //Get the selected courses out of session
            List<Course> _selectedCourses = (List<Course>)HttpContext.Current.Session[Constants.SessionKeys.NewCourses];
            //Iterate through the courses
            string dataKey = this.View.SectionsGrid.MasterTableView.DataKeyNames[0];
            foreach (Course _course in _selectedCourses)
            {
                //Iterate through the sections
                foreach (Section _section in _course.Sections)
                {
                    _checkFlag = false;
                    //Need to check and see if this section was checked to be handled manually
                    foreach (Telerik.Web.UI.GridDataItem _row in this.View.SectionsGrid.Items)
                    {
                        if (this.View.SectionsGrid.MasterTableView.DataKeyValues[_row.ItemIndex][dataKey].ToString() == _section.SectionID.ToString())
                        {
                            //This is the grid row for the current section
                            //Check to see if the checkbox is selected
                            CheckBox _chk = (CheckBox)_row.FindControl("chkManual");
                            if (_chk.Checked)
                            {
                                _checkFlag = true;
                            }
                            break;
                        }
                    }

                    //Create a copyHistory item and save it to the database
                    CopyHistory _copyHistory = new CopyHistory(_course.CourseCode, _course.CourseTitle, _section.CRN, _section.GroupCode, "", _section.SchoolName, _section.SectionID, _section.SectionSubTerm.SubTermName, _section.SectionSubTerm.SubTermDescription, _section.SectionTerm.TermName, _section.SectionTerm.TermDescription, HttpContext.Current.User.Identity.Name, 0, DateTime.Now, _section.HistoryECollegeTerm, _section.HistoryECollegeTermID, _section.HistoryECollegeID, _course.SelectedMasterCourse.CourseDisplayCode, _course.SelectedMasterCourse.eCollegeID, _course.SelectedMasterCourse.MasterCourseID, _section.SectionNode.Node, _course.SelectedMasterCourse.CourseTitle, batchNumber, _section.InstructorFirstName, _section.InstructorMiddleInitial, _section.InstructorLastName, _section.SectionCode, _section.CourseStartDate, _checkFlag);
                    this.InsertCopyHistory(_copyHistory);
                }
            }
        }
        
        public void InsertCopyHistory(CopyHistory copyHistory)
        {
            _controller.InsertCopyHistory(copyHistory);
        }

        public void SendEmail(string batchNumber)
        {
            this._controller.SendEmail(batchNumber);
        }
    }
}




