namespace LCPCustomControls
{
    using System;
    using System.Collections.Specialized;
    using System.ComponentModel;
    using System.Globalization;
    using System.Threading;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    [ToolboxData("<{0}:DropDownDateTime runat=server></{0}:DropDownDateTime>")]
    [DefaultProperty("Value"), ValidationProperty("Value")]
    public class DropDownDateTime : WebControl, INamingContainer, IPostBackDataHandler
    {
        private bool _AllowBlankTime;
        private bool _AutoPostBack;
        private string _Culture;
        private string _DateFormat = Thread.CurrentThread.CurrentCulture.DateTimeFormat.ShortDatePattern;
        private bool _IsValidDateTime;
        private bool _SelectCurrentDate = true;
        private string _TimeFormat = (" " + Thread.CurrentThread.CurrentCulture.DateTimeFormat.ShortTimePattern);
        private int _YearsBack;
        private int _YearsForward;
        private CheckBox chkDate;
        private DropDownList ddlboxAMPM;
        private DropDownList ddlboxDay;
        private DropDownList ddlboxHour;
        private DropDownList ddlboxMinute;
        private DropDownList ddlboxMonth;
        private DropDownList ddlboxSecond;
        private DropDownList ddlboxYear;
        private static readonly object EventControlChanged = new object();
        private bool FullYear;
        private string MonthFormat;

        public event EventHandler Changed
        {
            add
            {
                base.Events.AddHandler(EventControlChanged, value);
            }
            remove
            {
                base.Events.RemoveHandler(EventControlChanged, value);
            }
        }

        public DropDownDateTime()
        {
            base.Load += new EventHandler(this.DropDownDateTime_Load);
            base.Init += new EventHandler(this.DropDownDateTime_Init);
        }

        private void BuildControl()
        {
            if (this.EnableSet)
            {
                this.chkDate.Checked = this.SetChecked;
                this.Controls.Add(this.chkDate);
                this.ddlboxYear.Enabled = this.SetChecked;
                this.ddlboxMonth.Enabled = this.SetChecked;
                this.ddlboxDay.Enabled = this.SetChecked;
                this.ddlboxSecond.Enabled = this.SetChecked;
                this.ddlboxMinute.Enabled = this.SetChecked;
                this.ddlboxHour.Enabled = this.SetChecked;
                this.ddlboxAMPM.Enabled = this.SetChecked;
            }
            DateTime now = DateTime.Now;
            if ((this.SelectCurrentDate && (this.SelectedDay == -1)) && (this.SelectedMonth == -1))
            {
                this.SelectedDay = now.Day;
                this.SelectedMonth = now.Month;
                this.SelectedYear = now.Year;
                this.SelectedHour = now.Hour;
                this.SelectedMinute = now.Minute;
                this.SelectedSecond = now.Second;
                this.SelectedAMPM = now.ToString("tt", DateTimeFormatInfo.InvariantInfo);
                DateTimeFormatInfo provider = new DateTimeFormatInfo();
                provider.FullDateTimePattern = this.DateFormat + this.TimeFormat;
                this.Value = now.ToString(this.DateFormat + this.TimeFormat, provider);
            }
            if (this.Page.Request.Browser.JavaScript)
            {
                this.ddlboxDay.Attributes.Add("onchange", "ChangeOptionDays('" + this.ClientID + "','" + this.DateFormat + "');");
                this.ddlboxMonth.Attributes.Add("onchange", "ChangeOptionDays('" + this.ClientID + "','" + this.DateFormat + "');");
                this.ddlboxYear.Attributes.Add("onchange", "ChangeOptionDays('" + this.ClientID + "','" + this.DateFormat + "');");
                this.chkDate.Attributes.Add("onclick", "ChangeOptionDays('" + this.ClientID + "','" + this.DateFormat + "');");
            }
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            if (this.Culture != null)
            {
                Thread.CurrentThread.CurrentCulture = new CultureInfo(this.Culture);
            }
            for (int i = 1; i <= 12; i++)
            {
                DateTime time2 = new DateTime(0x7d0, i, 1);
                this.ddlboxMonth.Items.Add(new ListItem(time2.ToString(this.MonthFormat), i.ToString()));
            }
            if (this.YearsBack >= 0)
            {
                for (int j = -this.YearsBack; j <= this.YearsForward; j++)
                {
                    int num3 = now.Year + j;
                    if (this.FullYear)
                    {
                        this.ddlboxYear.Items.Add(num3.ToString());
                    }
                    else
                    {
                        this.ddlboxYear.Items.Add(new ListItem(num3.ToString().Substring(2, 2), num3.ToString()));
                    }
                }
            }
            else
            {
                for (int k = this.YearsForward; k <= -this.YearsBack; k++)
                {
                    int num5 = now.Year - k;
                    if (this.FullYear)
                    {
                        this.ddlboxYear.Items.Add(num5.ToString());
                    }
                    else
                    {
                        this.ddlboxYear.Items.Add(new ListItem(num5.ToString().Substring(2, 2), num5.ToString()));
                    }
                }
            }
            if (this.SelectedYear > 0)
            {
                string str = this.SelectedYear.ToString(CultureInfo.CurrentCulture);
                if (this.ddlboxYear.Items.FindByValue(str) != null)
                {
                    this.ddlboxYear.ClearSelection();
                    this.ddlboxYear.Items.FindByValue(str).Selected = true;
                }
            }
            if (this.SelectedMonth > 0)
            {
                string str2 = this.SelectedMonth.ToString(CultureInfo.CurrentCulture);
                if (this.ddlboxMonth.Items.FindByValue(str2) != null)
                {
                    this.ddlboxMonth.ClearSelection();
                    this.ddlboxMonth.Items.FindByValue(str2).Selected = true;
                }
            }
            if (this.SelectedDay > 0)
            {
                string str3 = this.SelectedDay.ToString(CultureInfo.CurrentCulture);
                if (this.ddlboxDay.Items.FindByValue(str3) != null)
                {
                    this.ddlboxDay.ClearSelection();
                    this.ddlboxDay.Items.FindByValue(str3).Selected = true;
                }
            }
            if (this.SelectedHour >= 0)
            {
                int selectedHour = this.SelectedHour;
                if (this.ddlboxAMPM.Items.Count > 0)
                {
                    if ((this.SelectedAMPM == "PM") && (selectedHour > 12))
                    {
                        selectedHour -= 12;
                    }
                    if ((this.SelectedAMPM == "AM") && (selectedHour == 0))
                    {
                        selectedHour = 12;
                    }
                }
                if (this.ddlboxHour.Items.FindByValue(selectedHour.ToString()) != null)
                {
                    this.ddlboxHour.ClearSelection();
                    this.ddlboxHour.Items.FindByValue(selectedHour.ToString()).Selected = true;
                }
            }
            if ((this.SelectedMinute >= 0) && (this.ddlboxMinute.Items.FindByValue(this.SelectedMinute.ToString()) != null))
            {
                this.ddlboxMinute.ClearSelection();
                this.ddlboxMinute.Items.FindByValue(this.SelectedMinute.ToString()).Selected = true;
            }
            if ((this.SelectedSecond >= 0) && (this.ddlboxSecond.Items.FindByValue(this.SelectedSecond.ToString()) != null))
            {
                this.ddlboxSecond.ClearSelection();
                this.ddlboxSecond.Items.FindByValue(this.SelectedSecond.ToString()).Selected = true;
            }
            if ((this.SelectedAMPM.Length > 0) && (this.ddlboxAMPM.Items.FindByValue(this.SelectedAMPM.ToString()) != null))
            {
                this.ddlboxAMPM.ClearSelection();
                this.ddlboxAMPM.Items.FindByValue(this.SelectedAMPM.ToString()).Selected = true;
            }
            this.ddlboxYear.ForeColor = this.ForeColor;
            this.ddlboxDay.ForeColor = this.ForeColor;
            this.ddlboxMonth.ForeColor = this.ForeColor;
            this.ddlboxHour.ForeColor = this.ForeColor;
            this.ddlboxMinute.ForeColor = this.ForeColor;
            this.ddlboxSecond.ForeColor = this.ForeColor;
            this.ddlboxAMPM.ForeColor = this.ForeColor;
            this.chkDate.ForeColor = this.ForeColor;
            this.ddlboxYear.BackColor = this.BackColor;
            this.ddlboxDay.BackColor = this.BackColor;
            this.ddlboxMonth.BackColor = this.BackColor;
            this.ddlboxHour.BackColor = this.BackColor;
            this.ddlboxMinute.BackColor = this.BackColor;
            this.ddlboxSecond.BackColor = this.BackColor;
            this.ddlboxAMPM.BackColor = this.BackColor;
            this.chkDate.BackColor = this.BackColor;
            this.ddlboxYear.Font.Names = this.Font.Names;
            this.ddlboxDay.Font.Names = this.Font.Names;
            this.ddlboxMonth.Font.Names = this.Font.Names;
            this.ddlboxHour.Font.Names = this.Font.Names;
            this.ddlboxMinute.Font.Names = this.Font.Names;
            this.ddlboxSecond.Font.Names = this.Font.Names;
            this.ddlboxAMPM.Font.Names = this.Font.Names;
            this.chkDate.Font.Names = this.Font.Names;
            this.ddlboxYear.CssClass = this.CssClass;
            this.ddlboxDay.CssClass = this.CssClass;
            this.ddlboxMonth.CssClass = this.CssClass;
            this.ddlboxHour.CssClass = this.CssClass;
            this.ddlboxMinute.CssClass = this.CssClass;
            this.ddlboxSecond.CssClass = this.CssClass;
            this.ddlboxAMPM.CssClass = this.CssClass;
            this.chkDate.CssClass = this.CssClass;
            Thread.CurrentThread.CurrentCulture = currentCulture;
        }

        protected override void CreateChildControls()
        {
            this.ddlboxDay = new DropDownList();
            this.ddlboxDay.ID = "Day";
            this.ddlboxMonth = new DropDownList();
            this.ddlboxMonth.ID = "Month";
            this.ddlboxYear = new DropDownList();
            this.ddlboxYear.ID = "Year";
            this.ddlboxMinute = new DropDownList();
            this.ddlboxMinute.ID = "Minute";
            this.ddlboxSecond = new DropDownList();
            this.ddlboxSecond.ID = "Second";
            this.ddlboxHour = new DropDownList();
            this.ddlboxHour.ID = "Hour";
            this.ddlboxAMPM = new DropDownList();
            this.ddlboxAMPM.ID = "AMPM";
            bool flag = true;
            bool flag2 = true;
            bool flag3 = true;
            bool flag4 = true;
            bool flag5 = true;
            bool flag6 = true;
            bool flag7 = true;
            if (this.DisplayFormat.Length > 0)
            {
                for (int i = 0; i < this.DisplayFormat.Length; i++)
                {
                    if ((this.DisplayFormat.Substring(i, 1) == "H") || (this.DisplayFormat.Substring(i, 1) == "h"))
                    {
                        if (flag)
                        {
                            string text = "--";
                            bool flag8 = false;
                            int num2 = 12;
                            int num3 = 1;
                            if ((i < (this.DisplayFormat.Length - 1)) && ((this.DisplayFormat.Substring(i, 2) == "HH") || (this.DisplayFormat.Substring(i, 2) == "hh")))
                            {
                                flag8 = true;
                            }
                            if (this.AllowBlankTime)
                            {
                                this.ddlboxHour.Items.Add(new ListItem(text, "-1"));
                            }
                            if (this.DisplayFormat.Substring(i, 1) == "H")
                            {
                                num2 = 0x17;
                                num3 = 0;
                                flag4 = false;
                            }
                            for (int j = num3; j <= num2; j++)
                            {
                                string str2 = j.ToString();
                                if ((j < 10) && flag8)
                                {
                                    str2 = "0" + str2;
                                }
                                this.ddlboxHour.Items.Add(new ListItem(str2, j.ToString()));
                            }
                            this.Controls.Add(this.ddlboxHour);
                            flag = false;
                        }
                    }
                    else if (this.DisplayFormat.Substring(i, 1) == "m")
                    {
                        if (flag2)
                        {
                            bool flag9 = false;
                            if ((i < (this.DisplayFormat.Length - 1)) && (this.DisplayFormat.Substring(i, 2) == "mm"))
                            {
                                flag9 = true;
                            }
                            if (this.AllowBlankTime)
                            {
                                this.ddlboxMinute.Items.Add(new ListItem("--", "-1"));
                            }
                            for (int k = 0; k < 60; k++)
                            {
                                string str3 = k.ToString();
                                if ((k < 10) && flag9)
                                {
                                    str3 = "0" + str3;
                                }
                                this.ddlboxMinute.Items.Add(new ListItem(str3, k.ToString()));
                            }
                            this.Controls.Add(this.ddlboxMinute);
                            flag2 = false;
                        }
                    }
                    else if (this.DisplayFormat.Substring(i, 1) == "s")
                    {
                        if (flag3)
                        {
                            bool flag10 = false;
                            if ((i < (this.DisplayFormat.Length - 1)) && (this.DisplayFormat.Substring(i, 2) == "ss"))
                            {
                                flag10 = true;
                            }
                            if (this.AllowBlankTime)
                            {
                                this.ddlboxSecond.Items.Add(new ListItem("--", "-1"));
                            }
                            for (int m = 0; m < 60; m++)
                            {
                                string str4 = m.ToString();
                                if ((m < 10) && flag10)
                                {
                                    str4 = "0" + str4;
                                }
                                this.ddlboxSecond.Items.Add(new ListItem(str4, m.ToString()));
                            }
                            this.Controls.Add(this.ddlboxSecond);
                            flag3 = false;
                        }
                    }
                    else if (this.DisplayFormat.Substring(i, 1) == "t")
                    {
                        if (flag4)
                        {
                            if (this.AllowBlankTime)
                            {
                                this.ddlboxAMPM.Items.Add(new ListItem("--", "--"));
                            }
                            this.ddlboxAMPM.Items.Add("AM");
                            this.ddlboxAMPM.Items.Add("PM");
                            this.Controls.Add(this.ddlboxAMPM);
                            flag4 = false;
                        }
                    }
                    else if (this.DisplayFormat.Substring(i, 1) == "y")
                    {
                        if (flag7)
                        {
                            if ((i < (this.DisplayFormat.Length - 3)) && (this.DisplayFormat.Substring(i, 4) == "yyyy"))
                            {
                                this.FullYear = true;
                            }
                            this.Controls.Add(this.ddlboxYear);
                            flag7 = false;
                        }
                    }
                    else if (this.DisplayFormat.Substring(i, 1) == "M")
                    {
                        if (flag6)
                        {
                            if ((i < (this.DisplayFormat.Length - 3)) && (this.DisplayFormat.Substring(i, 4) == "MMMM"))
                            {
                                this.MonthFormat = "MMMM";
                            }
                            else if ((i < (this.DisplayFormat.Length - 2)) && (this.DisplayFormat.Substring(i, 3) == "MMM"))
                            {
                                this.MonthFormat = "MMM";
                            }
                            else if ((i < (this.DisplayFormat.Length - 1)) && (this.DisplayFormat.Substring(i, 2) == "MM"))
                            {
                                this.MonthFormat = "MM";
                            }
                            else if ((i < this.DisplayFormat.Length) && (this.DisplayFormat.Substring(i, 1) == "M"))
                            {
                                this.MonthFormat = "MM";
                            }
                            this.Controls.Add(this.ddlboxMonth);
                            flag6 = false;
                        }
                    }
                    else if (this.DisplayFormat.Substring(i, 1) == "d")
                    {
                        if (flag5)
                        {
                            bool flag11 = false;
                            if ((i < (this.DisplayFormat.Length - 1)) && (this.DisplayFormat.Substring(i, 2) == "dd"))
                            {
                                flag11 = true;
                            }
                            for (int n = 1; n < 0x20; n++)
                            {
                                string str5 = n.ToString();
                                if ((n < 10) && flag11)
                                {
                                    str5 = "0" + str5;
                                }
                                this.ddlboxDay.Items.Add(new ListItem(str5, n.ToString()));
                            }
                            this.Controls.Add(this.ddlboxDay);
                            flag5 = false;
                        }
                    }
                    else
                    {
                        this.Controls.Add(new LiteralControl(this.DisplayFormat.Substring(i, 1)));
                    }
                }
            }
            if (flag && flag4)
            {
                this.SelectedHour = 12;
            }
            if (flag3)
            {
                this.SelectedSecond = 0;
            }
            if (flag2)
            {
                this.SelectedMinute = 0;
            }
            this.chkDate = new CheckBox();
            this.chkDate.ID = "Checkbox";
            if (this.AutoPostBack)
            {
                this.ddlboxDay.AutoPostBack = true;
                this.ddlboxYear.AutoPostBack = true;
                this.ddlboxMonth.AutoPostBack = true;
                this.chkDate.AutoPostBack = true;
                this.ddlboxSecond.AutoPostBack = true;
                this.ddlboxMinute.AutoPostBack = true;
                this.ddlboxHour.AutoPostBack = true;
                this.ddlboxAMPM.AutoPostBack = true;
            }
        }

        private void DropDownDateTime_Init(object sender, EventArgs e)
        {
            if (!this.Page.ClientScript.IsClientScriptBlockRegistered("DropDownDateTimeScript"))
            {
                string script = "\r\nfunction nDM(wM, wY){var nDM=31;if (wM==4||wM==6||wM==9||wM==11) nDM=30;if (wM==2 && (wY/4) !=Math.floor(wY/4)) nDM=28;if (wM==2 && (wY/4)==Math.floor(wY/4)) nDM=29;return nDM;};function ChangeOptionDays(w, DateFormat){if (document.getElementById){oH=document.getElementById(w+'_Hour');oMi=document.getElementById(w+'_Minute');oS=document.getElementById(w+'_Second');oAP=document.getElementById(w+'_AMPM');oD=document.getElementById(w+'_Day');oM=document.getElementById(w+'_Month');oY=document.getElementById(w+'_Year');CheckBoxObject=document.getElementById(w+'_Checkbox');} else if (document.layers){oH=document.layers[w+'_Hour'];oMi=document.layers[w+'_Minute'];oS=document.layers[w+'_Second'];oAP=document.layers[w+'_AMPM'];oD=document.layers[w+'_Day'];oM=document.layers[w+'_Month'];oY=document.layers[w+'_Year'];CheckBoxObject=document.layers[w+'_Checkbox'];} else if (document.all){oH=document.all[w+'_Hour'];oMi=document.all[w+'_Minute'];oS=document.all[w+'_Second'];oAP=document.all[w+'_AMPM'];oD=document.all[w+'_Day'];oM=document.all[w+'_Month'];oY=document.all[w+'_Year'];CheckBoxObject=document.all[w+'_Checkbox'];} var D=oD[oD.selectedIndex].value;var M=oM[oM.selectedIndex].value;var Y=oY[oY.selectedIndex].value;dfts=nDM(M, Y);cdis=oD.length; \tif (cdis>dfts){if (oD.selectedIndex>=(cdis-1)) oD.selectedIndex=(dfts-1);for (i=0;i<(cdis-dfts);i++){/*oD.options[oD.options.length - 1]=null;*/}}; if (dfts>cdis){for (i=0;i<(dfts-cdis);i++){NewOption=new Option(oD.options.length+1);oD.add(NewOption);}};if (oD.selectedIndex<0) oD.selectedIndex==0; if (CheckBoxObject)\t{oD.disabled=!CheckBoxObject.checked;oM.disabled=!CheckBoxObject.checked;oY.disabled=!CheckBoxObject.checked;if (oH) oH.disabled=!CheckBoxObject.checked;if (oMi) oMi.disabled=!CheckBoxObject.checked;if (oS) oS.disabled=!CheckBoxObject.checked;if (oAP) oAP.disabled=!CheckBoxObject.checked;}}";
                this.Page.ClientScript.RegisterClientScriptBlock(typeof(DropDownDateTime), "DropDownDateTimeControl", script, true);
            }
        }

        private void DropDownDateTime_Load(object sender, EventArgs e)
        {
            this.EnsureChildControls();
        }

        public virtual bool LoadPostData(string postDataKey, NameValueCollection postCollection)
        {
            bool flag = false;
            int num = 0;
            int num2 = 0;
            int num3 = 0;
            int num4 = 0;
            int num5 = 0;
            int num6 = 0;
            int parseHour = 0;
            parseHour = this.SelectedHour;
            if (postCollection[this.UniqueID.ToString() + "$Checkbox"] != "")
            {
                if (postCollection[this.UniqueID.ToString() + "$Checkbox"] == "on")
                {
                    if (!this.SetChecked)
                    {
                        flag = true;
                    }
                    this.SetChecked = true;
                }
                else
                {
                    if (this.SetChecked)
                    {
                        flag = true;
                    }
                    this.SetChecked = false;
                }
            }
            if (this.SetChecked || !this.EnableSet)
            {
                string s = postCollection[this.UniqueID.ToString() + "$Second"];
                if ((s != "") && (s != null))
                {
                    num = int.Parse(s);
                    if (num != this.SelectedSecond)
                    {
                        this.SelectedSecond = num;
                        flag = true;
                    }
                }
                string str2 = postCollection[this.UniqueID.ToString() + "$Minute"];
                if ((str2 != "") && (str2 != null))
                {
                    num2 = int.Parse(str2);
                    if (num2 != this.SelectedMinute)
                    {
                        this.SelectedMinute = num2;
                        flag = true;
                    }
                }
                string str3 = postCollection[this.UniqueID.ToString() + "$Hour"];
                if ((str3 != "") && (str3 != null))
                {
                    num3 = int.Parse(str3);
                    if (num3 != this.SelectedHour)
                    {
                        if ((this.SelectedAMPM == "") || ((this.SelectedHour != (num3 + 12)) && (num3 != (this.SelectedHour + 12))))
                        {
                            flag = true;
                        }
                        this.SelectedHour = num3;
                    }
                }
                parseHour = this.SelectedHour;
                string str4 = postCollection[this.UniqueID.ToString() + "$Day"];
                if ((str4 != "") && (str4 != null))
                {
                    num4 = int.Parse(str4);
                    if (num4 != this.SelectedDay)
                    {
                        this.SelectedDay = num4;
                        flag = true;
                    }
                }
                string str5 = postCollection[this.UniqueID.ToString() + "$Month"];
                if ((str5 != "") && (str5 != null))
                {
                    num5 = int.Parse(str5);
                    if (num5 != this.SelectedMonth)
                    {
                        this.SelectedMonth = num5;
                        flag = true;
                    }
                }
                string str6 = postCollection[this.UniqueID.ToString() + "$Year"];
                if ((str6 != "") && (str6 != null))
                {
                    num6 = int.Parse(str6);
                    if (num6 != this.SelectedYear)
                    {
                        this.SelectedYear = num6;
                        flag = true;
                    }
                }
                string str7 = postCollection[this.UniqueID.ToString() + "$AMPM"];
                if ((str7 != "") && (str7 != null))
                {
                    if (str7 != this.SelectedAMPM)
                    {
                        this.SelectedAMPM = str7;
                        flag = true;
                    }
                    if ((str7 == "PM") && (parseHour < 12))
                    {
                        parseHour += 12;
                    }
                    if ((str7 == "AM") && (parseHour == 12))
                    {
                        parseHour = 0;
                    }
                }
            }
            try
            {
                this.Value = this.SelectedDateString(parseHour);
            }
            catch (Exception exception)
            {
                HttpContext.Current.Response.Write(string.Concat(new object[] { "Postback Error:", exception.Message, " Year:", this.SelectedYear, " Month:", this.SelectedMonth, " EnableSet:", this.EnableSet.ToString(), "<BR/>" }));
            }
            return flag;
        }

        protected virtual void OnChange(EventArgs e)
        {
            EventHandler handler = (EventHandler) base.Events[EventControlChanged];
            if (handler != null)
            {
                handler(this, e);
            }
        }

        protected virtual void OnChanged(EventArgs e)
        {
            EventHandler handler = (EventHandler) base.Events[EventControlChanged];
            if (handler != null)
            {
                handler(this, e);
            }
        }

        public virtual void RaisePostDataChangedEvent()
        {
            this.OnChange(EventArgs.Empty);
        }

        protected override void Render(HtmlTextWriter output)
        {
            this.BuildControl();
            this.EnsureChildControls();
            output.AddAttribute(HtmlTextWriterAttribute.Type, "hidden");
            if (this.EnableSet && !this.SetChecked)
            {
                output.AddAttribute(HtmlTextWriterAttribute.Value, "");
            }
            else
            {
                output.AddAttribute(HtmlTextWriterAttribute.Value, this.Value);
            }
            output.AddAttribute(HtmlTextWriterAttribute.Name, this.UniqueID);
            output.AddAttribute(HtmlTextWriterAttribute.Id, this.ClientID);
            output.RenderBeginTag(HtmlTextWriterTag.Input);
            output.RenderEndTag();
            base.Render(output);
        }

        private string SelectedDateString(int ParseHour)
        {
            if ((this.EnableSet && (this.SelectedYear == -1)) && ((this.SelectedMonth == -1) && (this.SelectedDay == -1)))
            {
                return null;
            }
            try
            {
                DateTime time;
                DateTimeFormatInfo provider = new DateTimeFormatInfo();
                provider.FullDateTimePattern = this.DateFormat;
                if (((this.SelectedHour == -1) || (this.SelectedMinute == -1)) || ((this.SelectedSecond == -1) || (this.SelectedAMPM == "--")))
                {
                    time = new DateTime(this.SelectedYear, this.SelectedMonth, this.SelectedDay);
                    return time.ToString(this.DateFormat, provider);
                }
                time = new DateTime(this.SelectedYear, this.SelectedMonth, this.SelectedDay, ParseHour, this.SelectedMinute, this.SelectedSecond);
                return time.ToString(this.DateFormat + this.TimeFormat, provider);
            }
            catch
            {
                return "Invalid";
            }
        }

        private DateTime SelectedDateTime()
        {
            DateTimeFormatInfo info = new DateTimeFormatInfo();
            info.FullDateTimePattern = this.DateFormat;
            if (((this.SelectedHour == -1) || (this.SelectedMinute == -1)) || ((this.SelectedSecond == -1) || (this.SelectedAMPM == "--")))
            {
                return new DateTime(this.SelectedYear, this.SelectedMonth, this.SelectedDay);
            }
            return new DateTime(this.SelectedYear, this.SelectedMonth, this.SelectedDay, this.SelectedHour, this.SelectedMinute, this.SelectedSecond);
        }

        private void SetSelected(string StoredValue)
        {
            DateTimeFormatInfo provider = new DateTimeFormatInfo();
            provider.FullDateTimePattern = this.DateFormat + this.TimeFormat;
            try
            {
                if (this.EnableSet && (StoredValue == null))
                {
                    if (this.SelectCurrentDate)
                    {
                        DateTime now = DateTime.Now;
                        this.SelectedDay = now.Day;
                        this.SelectedMonth = now.Month;
                        this.SelectedYear = now.Year;
                        this.SelectedHour = now.Hour;
                        this.SelectedMinute = now.Minute;
                        this.SelectedSecond = now.Second;
                        this.SelectedAMPM = now.ToString("tt", DateTimeFormatInfo.InvariantInfo);
                    }
                }
                else
                {
                    DateTime time2 = new DateTime();
                    time2 = DateTime.ParseExact(StoredValue, this.DateFormat + this.TimeFormat, provider);
                    this.SelectedDay = time2.Day;
                    this.SelectedMonth = time2.Month;
                    this.SelectedYear = time2.Year;
                    this.SelectedAMPM = time2.ToString("tt", DateTimeFormatInfo.InvariantInfo);
                    this.SelectedHour = time2.Hour;
                    this.SelectedMinute = time2.Minute;
                    this.SelectedSecond = time2.Second;
                }
                this.IsValidDateTime = true;
            }
            catch
            {
                try
                {
                    DateTime time3 = new DateTime();
                    time3 = DateTime.ParseExact(StoredValue, this.DateFormat, provider);
                    this.SelectedDay = time3.Day;
                    this.SelectedMonth = time3.Month;
                    this.SelectedYear = time3.Year;
                    this.IsValidDateTime = true;
                }
                catch
                {
                    this.IsValidDateTime = false;
                }
            }
        }

        [DefaultValue(-1), Bindable(true), Description("Whether the control allows a blank time"), Category("Behavior")]
        public bool AllowBlankTime
        {
            get
            {
                return this._AllowBlankTime;
            }
            set
            {
                this._AllowBlankTime = value;
            }
        }

        [Description("AutoPostBack"), Category("Behavior"), DefaultValue(false), Bindable(true)]
        public bool AutoPostBack
        {
            get
            {
                return this._AutoPostBack;
            }
            set
            {
                this._AutoPostBack = value;
            }
        }

        [Description("Specifies the date language, example en-GB for english, fr-FR for french etc.. If this is not set the server culture is taken."), DefaultValue(-1), Category("Appearance"), Bindable(true)]
        public string Culture
        {
            get
            {
                return this._Culture;
            }
            set
            {
                if (value.ToLower() == "auto")
                {
                    this._Culture = HttpContext.Current.Request.ServerVariables["HTTP_ACCEPT_LANGUAGE"];
                }
                else
                {
                    this._Culture = value;
                }
            }
        }

        [Category("Output"), DefaultValue(-1), Bindable(true), Description("Specifies the time format output dd/MM/yyyy MM-dd-yyyy etc.. If this is not set then the server date format is taken. Please note MMM ddd MMMM dddd are not valid!!")]
        public string DateFormat
        {
            get
            {
                string str = (string) this.ViewState["DateFormat"];
                if (str == null)
                {
                    return this._DateFormat;
                }
                return str;
            }
            set
            {
                this._DateFormat = value;
                this.ViewState["DateFormat"] = value;
            }
        }

        [Description("Specifies the Display format, If this is not set then the server date format is taken."), DefaultValue(-1), Category("Appearance"), Bindable(true)]
        public string DisplayFormat
        {
            get
            {
                string str = (string) this.ViewState["DisplayFormat"];
                if (str == null)
                {
                    return this._DateFormat;
                }
                return str;
            }
            set
            {
                this.ViewState["DisplayFormat"] = value;
            }
        }

        [Bindable(true), DefaultValue(false), Description("Enables a checkbox to activate the date dropdownlists, if the checkbox is not set then the date is blank!!!"), Category("Behavior")]
        public bool EnableSet
        {
            get
            {
                if (this.ViewState["EnableSet"] == null)
                {
                    return false;
                }
                return (bool) this.ViewState["EnableSet"];
            }
            set
            {
                this.ViewState["EnableSet"] = value;
            }
        }

        public bool IsValidDateTime
        {
            get
            {
                return this._IsValidDateTime;
            }
            set
            {
                this._IsValidDateTime = value;
            }
        }

        [Bindable(true), Description("The control use the current date when not value is given"), Category("Behavior"), DefaultValue(-1)]
        public bool SelectCurrentDate
        {
            get
            {
                return this._SelectCurrentDate;
            }
            set
            {
                this._SelectCurrentDate = value;
            }
        }

        private string SelectedAMPM
        {
            get
            {
                string str = (string) this.ViewState["SelectedAMPM"];
                if (str == null)
                {
                    return string.Empty;
                }
                return str;
            }
            set
            {
                this.ViewState["SelectedAMPM"] = value;
            }
        }

        [Description("The DateTime value of the control."), Bindable(true), Category("Output"), DefaultValue(-1)]
        public DateTime SelectedDate
        {
            get
            {
                return this.SelectedDateTime();
            }
        }

        private int SelectedDay
        {
            get
            {
                if (this.ViewState["SelectedDay"] == null)
                {
                    return -1;
                }
                return (int) this.ViewState["SelectedDay"];
            }
            set
            {
                this.ViewState["SelectedDay"] = value;
            }
        }

        private int SelectedHour
        {
            get
            {
                if (this.ViewState["SelectedHour"] == null)
                {
                    return -1;
                }
                return (int) this.ViewState["SelectedHour"];
            }
            set
            {
                this.ViewState["SelectedHour"] = value;
            }
        }

        private int SelectedMinute
        {
            get
            {
                if (this.ViewState["SelectedMinute"] == null)
                {
                    return -1;
                }
                return (int) this.ViewState["SelectedMinute"];
            }
            set
            {
                this.ViewState["SelectedMinute"] = value;
            }
        }

        private int SelectedMonth
        {
            get
            {
                if (this.ViewState["SelectedMonth"] == null)
                {
                    return -1;
                }
                return (int) this.ViewState["SelectedMonth"];
            }
            set
            {
                this.ViewState["SelectedMonth"] = value;
            }
        }

        private int SelectedSecond
        {
            get
            {
                if (this.ViewState["SelectedSecond"] == null)
                {
                    return -1;
                }
                return (int) this.ViewState["SelectedSecond"];
            }
            set
            {
                this.ViewState["SelectedSecond"] = value;
            }
        }

        private string SelectedTime
        {
            get
            {
                string str = (string) this.ViewState["SelectedTime"];
                if (str == null)
                {
                    return string.Empty;
                }
                return str;
            }
            set
            {
                this.ViewState["SelectedTime"] = value;
            }
        }

        private int SelectedYear
        {
            get
            {
                if (this.ViewState["SelectedYear"] == null)
                {
                    return -1;
                }
                return (int) this.ViewState["SelectedYear"];
            }
            set
            {
                this.ViewState["SelectedYear"] = value;
            }
        }

        [DefaultValue(false), Category("Behavior"), Description("Sets checkbox when its enabled"), Bindable(true)]
        public bool SetChecked
        {
            get
            {
                if (this.ViewState["SetChecked"] == null)
                {
                    return false;
                }
                return (bool) this.ViewState["SetChecked"];
            }
            set
            {
                this.ViewState["SetChecked"] = value;
            }
        }

        [Description("Specifies the time format, If this is not set then the server date format is taken."), Bindable(true), Category("Output"), DefaultValue(-1)]
        public string TimeFormat
        {
            get
            {
                string str = (string) this.ViewState["TimeFormat"];
                if (str == null)
                {
                    return this._TimeFormat;
                }
                return str;
            }
            set
            {
                this._TimeFormat = value;
                this.ViewState["TimeFormat"] = value;
            }
        }

        [DefaultValue(-1), Bindable(true), Description("The output value of the control."), Category("Output")]
        public string Value
        {
            get
            {
                string str = (string) this.ViewState["Value"];
                if ((str != null) && (this.SetChecked || !this.EnableSet))
                {
                    return str;
                }
                return string.Empty;
            }
            set
            {
                this.SetSelected(value);
                this.ViewState["Value"] = value;
            }
        }

        [Description("Specifies a number of years from the current year. Value can also be negative"), Category("Appearance"), Bindable(true), DefaultValue(0)]
        public int YearsBack
        {
            get
            {
                int num = Convert.ToInt16(this.ViewState["YearsBack"]);
                if (num == 0)
                {
                    return this._YearsBack;
                }
                return num;
            }
            set
            {
                this._YearsBack = value;
                this.ViewState["YearsBack"] = value;
            }
        }

        [DefaultValue(0), Category("Appearance"), Bindable(true), Description("Specifies a number of years from the current year. Value can also be negative")]
        public int YearsForward
        {
            get
            {
                int num = Convert.ToInt16(this.ViewState["YearsForward"]);
                if (num == 0)
                {
                    return this._YearsForward;
                }
                return num;
            }
            set
            {
                this._YearsForward = value;
                this.ViewState["YearsForward"] = value;
            }
        }
    }
}

