//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.Interface;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb.Web;
using OHE.FoundationModules.Base.Business;
using OHE.FoundationModules.ProviderServices.BusinessEntities;
//using OHE.LCP.Base.Business;
//using OHE.LCP.Common;


namespace OHE.LCP.Admin.Messages
{
    /// <summary>
    /// This class is the controller for the Messages module.
    /// </summary>
    public class MessagesController
    {
        //Private variable for the Message service
        private IMessageService _messageService;
        
        /// <summary>
        /// Parameterless constructor
        /// </summary>
        public MessagesController()
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="messageService">Reference to the message service class</param>
        /// <param name="businessEntityCacheManager">Reference to the business entity cache manager</param>
        [InjectionConstructor]
        public MessagesController(
            [ServiceDependency] IMessageService messageService)
        {
            _messageService = messageService;
        }

        /// <summary>
        /// Gets all of the messages in the system.
        /// </summary>
        /// <returns></returns>
        public IList<Message> GetAllMessages()
        {
            return _messageService.GetAllMessages();
        }

        /// <summary>
        /// Gets the messages for the LCP.
        /// </summary>
        /// <param name="RoleIDs">A comma delimited list of RoleIDs for the user</param>
        /// <param name="SchoolIDs">A comma delimited list of SchoolIDs for the user</param>
        /// <param name="ProgramIDs">A comma delimited list of ProgramIDs for the user</param>
        /// <param name="ConcentrationIDs">A comma delimited list of ConcentrationIDs for the user</param>
        /// <param name="CourseIDs">A comma delimited list of CourseIDs for the user</param>
        /// <param name="SectionIDs">A comma delimited list of SectionIDs for the user</param>
        /// <param name="PersonID">The ID of the LCP user</param>
        /// <returns></returns>
        public List<Message> GetMessagesForLCP(string RoleIDs, string SchoolIDs, string ProgramIDs, string ConcentrationIDs, string CourseIDs, string SectionIDs, string PersonID)
        {
            return _messageService.GetMessagesForLCP(RoleIDs, SchoolIDs, ProgramIDs, ConcentrationIDs, CourseIDs, SectionIDs, PersonID);
        }

        /// <summary>
        /// Gets an individual message
        /// </summary>
        /// <param name="messageID">The ID of the message</param>
        /// <returns></returns>
        public Message GetMessage(int messageID)
        {
            return _messageService.GetMessage(messageID);
        }

        /// <summary>
        /// Inserts a new message in to the system
        /// </summary>
        /// <param name="message"></param>
        public void InsertMessage(Message message)
        {
            _messageService.InsertMessage(message);
        }

        /// <summary>
        /// Updates a message in the system
        /// </summary>
        /// <param name="message"></param>
        public void UpdateMessage(Message message)
        {
            _messageService.UpdateMessage(message);
        }

        /// <summary>
        /// Deletes a message from the system
        /// </summary>
        /// <param name="messageID"></param>
        public void DeleteMessage(int messageID)
        {
            _messageService.DeleteMessage(messageID);
        }

        /// <summary>
        /// Gets the business entities for Colleges, Programs and Concentrations.
        /// </summary>
        /// <param name="structureType">The structure type for the business entities</param>
        /// <returns></returns>
        public Dictionary<String, List<IBusinessEntity>> GetCollegeProgramConcentrations(Enums.BusinessEntityStructureType structureType)
        {
            return _messageService.GetCollegeProgramConcentrations(structureType);
        }
        
    }
}
