﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.LCP.Admin.Messages.BusinessEntities;

namespace OHE.LCP.Admin.Messages.Views
{
    /// <summary>
    /// The presenter for the message detail page.
    /// </summary>
    public class MessageDetailPresenter : Presenter<IMessageDetail>
    {
        private MessagesController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller.
        /// </summary>
        /// <param name="controller">The controller for the presenter</param>
        public MessageDetailPresenter([CreateNew] MessagesController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// Sets the Schools/Programs/Concentrations property.
        /// </summary>
        public override void OnViewInitialized()
        {
            base.OnViewInitialized();
            this.View.SchoolsProgramsConcentrations = _controller.GetCollegeProgramConcentrations(View.StructureType);
        }

        /// <summary>
        /// Sets the message property
        /// </summary>
        /// <param name="messageID">The ID of the message</param>
        public void OnGetMessage(int messageID)
        {
            this.View.message = _controller.GetMessage(messageID);
        }

        /// <summary>
        /// Inserts a new message
        /// </summary>
        /// <param name="message">The message object to be inserted</param>
        public void OnMessageInserted(Message message)
        {
            _controller.InsertMessage(message);
        }

        /// <summary>
        /// Updates a message
        /// </summary>
        /// <param name="message">The message object to be updated</param>
        public void OnMessageUpdated(Message message)
        {
            _controller.UpdateMessage(message);
        }

    }
}




