﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.LCP.Admin.Messages.BusinessEntities;
using Telerik.Web.UI;
using System.Web.UI.HtmlControls;

namespace OHE.LCP.Admin.Messages.Views
{
    /// <summary>
    /// The presenter class for the Message Listing page.
    /// </summary>
    public class MessageListPresenter : Presenter<IMessageList>
    {
        private MessagesController _controller;
        private List<Message> _messages;

        /// <summary>
        /// Constructor.  Sets the reference to the controller
        /// </summary>
        /// <param name="controller">The controller</param>
        public MessageListPresenter([CreateNew] MessagesController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// Sets the Messages property to the list of messages from the controller when the view is loaded.
        /// </summary>
        public override void OnViewLoaded()
        {
            base.OnViewLoaded();
            this.View.MessageGrid.SelectedIndexChanged += new EventHandler(MessageGrid_SelectedIndexChanged);
            this.View.MessageGrid.ItemDataBound += new GridItemEventHandler(MessageGrid_ItemDataBound);
            _messages = (List<Message>)_controller.GetAllMessages();
            this.View.Messages = _messages;
            GetCurrentMessagePage();
        }

        /// <summary>
        /// Deletes the selected message.
        /// </summary>
        /// <param name="messageID">The ID of the message to be deleted</param>
        public void OnMessageDeleted(int messageID)
        {
            _controller.DeleteMessage(messageID);
        }

        /// <summary>
        /// Get the page of the message last edited
        /// </summary>
        private void GetCurrentMessagePage()
        {
            if (this.View.SelectedMessageID != null)
            {
                Message m = _messages.Find(new Predicate<Message>(IsMessage));
                int index = _messages.IndexOf(m);
                this.View.MessageGrid.MasterTableView.CurrentPageIndex = Convert.ToInt32(System.Math.Floor((Double)((index / this.View.MessageGrid.MasterTableView.PageSize))));
                this.View.MessageBody.InnerHtml = m.Body;
            }
        }

        /// <summary>
        /// Predicate to find last edited message
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        private bool IsMessage(Message message)
        {
            if (message.MessageID == this.View.SelectedMessageID)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Displays the body of the message
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void MessageGrid_SelectedIndexChanged(object sender, EventArgs e)
        {
            GridDataItem dataItem = (GridDataItem)this.View.MessageGrid.SelectedItems[0];
            this.View.MessageBody.InnerHtml = dataItem["Body"].Text;
        }

        /// <summary>
        /// Selects the last edited message
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void MessageGrid_ItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.RowIndex >= 0 && e.Item.RowIndex % 2 == 0)
            {
                e.Item.Cells[3].Text = "<a href=\"MessageDetail.aspx?mode=edit&amp;id=" + e.Item.Cells[4].Text + "\">Edit</a>";

                if (this.View.SelectedMessageID != null)
                {
                    if (e.Item.Cells[4].Text == this.View.SelectedMessageID.ToString())
                        e.Item.Selected = true;
                }
            }
        }
    }
}




