//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using OHE.Admin.Security.Module.Services;
using OHE.Security.DataTypes;
using System.Threading;

namespace OHE.Admin.Security.Module
{
    public class SecurityController
    {
        private IAdministrationServiceAgent _administrationServiceAgent;
        private IRoleServiceAgent _roleServiceAgent;
        private IAuthenticationServiceAgent _authenticationServiceAgent;
        
        public SecurityController([ServiceDependency] IAdministrationServiceAgent administrationServiceAgent,
                                  [ServiceDependency] IRoleServiceAgent roleServiceAgent,
                                 [ServiceDependency] IAuthenticationServiceAgent authenticationServiceAgent)
        {
            _administrationServiceAgent = administrationServiceAgent;
            _roleServiceAgent = roleServiceAgent;
            _authenticationServiceAgent = authenticationServiceAgent;
            
        }

        public virtual List<SecurityContext> SecurityContexts
        {
            get
            {
                return _administrationServiceAgent.GetAllSecurityContexts();
            }
        }

        public virtual List<Institution> Institutions
        {
            get
            {
                return _administrationServiceAgent.GetAllInstitutions();
            }
        }


        public virtual void CreateSecurityContext(SecurityContext context)
        {
            context.CreateOperator = Thread.CurrentPrincipal.Identity.Name;
            _administrationServiceAgent.CreateSecurityContext(context);
            this.Redirect(Constants.URLS.SecurityContextList);
        }

        public virtual void DeleteSecurityContext(SecurityContext context)
        {
            _administrationServiceAgent.DeleteSecurityContext(context);
        }

        public virtual void UpdateSecurityContext(SecurityContext context)
        {
            context.UpdateOperator = Thread.CurrentPrincipal.Identity.Name;
            _administrationServiceAgent.UpdateSecurityContext(context);
            this.Redirect(Constants.URLS.SecurityContextList);
        }

        public virtual List<SecurityContext> GetAllSecurityContexts()
        {
            return _administrationServiceAgent.GetAllSecurityContexts();
        }

        public virtual SecurityContext GetSecurityContext(int ID)
        {
            return _administrationServiceAgent.GetSecurityContext(ID);
        }

        public virtual void CreateInstitution(Institution user)
        {
            user.CreateOperator = Thread.CurrentPrincipal.Identity.Name;
            _administrationServiceAgent.CreateInstitution(user);
            this.Redirect(Constants.URLS.InstitutionList);
        }

        public virtual void DeleteInstitution(Institution user)
        {
            _administrationServiceAgent.DeleteInstitution(user);
        }

        public virtual void UpdateInstitution(Institution user)
        {
            user.UpdateOperator = Thread.CurrentPrincipal.Identity.Name;
            _administrationServiceAgent.UpdateInstitution(user);
            this.Redirect(Constants.URLS.InstitutionList);
        }

        public virtual List<Institution> GetAllInstitutions()
        {
            return _administrationServiceAgent.GetAllInstitutions();
        }

        public virtual Institution GetInstitution(Int32 ID)
        {
            return _administrationServiceAgent.GetInstitution(ID);
        }

        public virtual List<Action> GetAllActions(Int32 securityContextID)
        {
            return _administrationServiceAgent.GetAllActions(securityContextID);
        }

        public virtual void DeleteAction(Action action)
        {
            _administrationServiceAgent.DeleteAction(action);
        }

        public virtual Action GetAction(Int32 ID)
        {
            return _administrationServiceAgent.GetAction(ID);
        }

        public virtual void CreateAction(Action action)
        {
            action.CreateOperator = Thread.CurrentPrincipal.Identity.Name;
            _administrationServiceAgent.CreateAction(action);
            this.Redirect(String.Format(Constants.URLS.ActionList, action.SecurityContextID));
        }

        public virtual void UpdateAction(Action action)
        {
            action.UpdateOperator = Thread.CurrentPrincipal.Identity.Name;
            _administrationServiceAgent.UpdateAction(action);
            this.Redirect(String.Format(Constants.URLS.ActionList, action.SecurityContextID));
        }

        public virtual List<Role> GetAllRoles(Int32 securityContextID)
        {
            return _roleServiceAgent.GetAllRoles(securityContextID);
        }

        public virtual void DeleteRole(Role role)
        {
            _roleServiceAgent.DeleteRole(role);
        }

        public virtual Role GetRole(Int32 ID)
        {
            return _roleServiceAgent.GetRole(ID);
        }

        public virtual void CreateRole(Role role)
        {
            role.CreateOperator = Thread.CurrentPrincipal.Identity.Name;
            _roleServiceAgent.CreateRole(role);
            this.Redirect(String.Format(Constants.URLS.RoleList, role.SecurityContextID));
        }

        public virtual void UpdateRole(Role role)
        {
            role.UpdateOperator = Thread.CurrentPrincipal.Identity.Name;
            _roleServiceAgent.UpdateRole(role);
            this.Redirect(String.Format(Constants.URLS.RoleList, role.SecurityContextID));
        }

        public virtual List<User> FindUsersBySeachCriteria(UserSearchCriteria criteria, int startRowIndex, int maximumRows, out int totalRecords)
        {
            return _administrationServiceAgent.FindUsersBySearchCriteria(criteria, startRowIndex, maximumRows, out totalRecords);
        }

        public virtual User GetUser(int ID)
        {
            return _authenticationServiceAgent.GetUser(ID);
        }

        public virtual User GetUserWithAllIdentities(int ID)
        {
            return _authenticationServiceAgent.GetUserWithAllIdentities(ID);
        }

        public virtual void Redirect(String url)
        {
            System.Web.HttpContext.Current.Response.Redirect(url);
        }

        public virtual bool ResetPassword(OHE.Security.DataTypes.Institution institution, string userName, int systemID)
        {
            return _authenticationServiceAgent.ResetPassword(institution, userName, systemID);
        }

        public bool UpdateSecurityIdentity(Institution institution, int userID, int systemUserId, int systemID, bool active, string hint,
                                           string hintAnswer, bool isLockedOut, string userName, string updateOperator)
        {
            Identity identity = new Identity();
            identity.ActiveStatus = active;
            identity.Institution = institution;
            identity.SystemUserID = systemUserId;
            identity.Hint = hint;
            identity.HintAnswer = hintAnswer;
            identity.UpdateOperator = updateOperator;
            identity.IsLockedOut = isLockedOut;
            identity.SecurityContextId = systemID;
            identity.UserName = userName;
            return _authenticationServiceAgent.UpdateIdentity(userID, identity);
        }

        public List<string> CreateUser(User user)
        {
            user.CreateOperator = Thread.CurrentPrincipal.Identity.Name;
            return _authenticationServiceAgent.CreateUser(user);            
        }

        public bool UpdatePassword(string userName, string password, string oldPassword, int systemUserId)
        {
            Identity identity = new Identity();
            identity.UserName = userName;
            identity.Password = password;
            identity.SystemUserID = systemUserId;

            return _authenticationServiceAgent.UpdatePassword(identity, oldPassword);

        }
    }
}
