//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.CompositeWeb.Interfaces;
using Microsoft.Practices.CompositeWeb.Services;
using Microsoft.Practices.CompositeWeb.Configuration;
using Microsoft.Practices.CompositeWeb.EnterpriseLibrary.Services;
using OHE.FoundationModules.Authentication.ServiceProxy.EntityTranslators;
using OHE.FoundationModules.ProviderServices.Base;
using OHE.Admin.Security.Module.Services;

namespace OHE.Admin.Security.Module
{
    public class SecurityModuleInitializer : ModuleInitializer
    {
        private const string AuthorizationSection = "compositeWeb/authorization";

        public override void Load(ICompositionContainer moduleContainer)
        {
            base.Load(moduleContainer);

            AddGlobalServices(moduleContainer.Parent.Services);
            AddModuleServices(moduleContainer.Services);
            RegisterSiteMapInformation(moduleContainer.Services.Get<ISiteMapBuilderService>(true));
            RegisterTranslators(moduleContainer.Services.Get<IEntityTranslatorService>(true));
        }

        protected virtual void AddGlobalServices(IServiceCollection globalServices)
        {
            // TODO: add a service that will be visible to any module
        }

        protected virtual void AddModuleServices(IServiceCollection moduleServices)
        {
            moduleServices.AddNew<AdministrationServiceAgent, IAdministrationServiceAgent>();
            moduleServices.AddNew<RoleServiceAgent, IRoleServiceAgent>();
            moduleServices.AddNew<AuthenticationServiceAgent, IAuthenticationServiceAgent>();
        }

        protected virtual void RegisterSiteMapInformation(ISiteMapBuilderService siteMapBuilderService)
        {
            SiteMapNodeInfo moduleNode = new SiteMapNodeInfo("Security", "~/Security/Default.aspx", "Security");
            SiteMapNodeInfo InstitutionListNode = new SiteMapNodeInfo("InstitutionView", "~/Security/InstitutionList.aspx", "Institutions");
            SiteMapNodeInfo SecurityContextListNode = new SiteMapNodeInfo("SecurityContextView", "~/Security/SecurityContextList.aspx", "SecurityContexts");
            SiteMapNodeInfo UsersListNode = new SiteMapNodeInfo("UserView", "~/Security/UserSearch.aspx", "Users");

            siteMapBuilderService.AddNode(moduleNode, Constants.Permissions.AllowSecurityMaintenance, 3);
            siteMapBuilderService.AddNode(InstitutionListNode, moduleNode, Constants.Permissions.AllowInstitutionMaintenance);
            siteMapBuilderService.AddNode(SecurityContextListNode, moduleNode, Constants.Permissions.AllowSecurityContextMaintenance);
            siteMapBuilderService.AddNode(UsersListNode, moduleNode, Constants.Permissions.AllowUserMaintenance);

            // TODO: register other site map nodes that Security module might provide            
        }

        public override void Configure(IServiceCollection services, System.Configuration.Configuration moduleConfiguration)
        {
            IAuthorizationRulesService authorizationRuleService = services.Get<IAuthorizationRulesService>();
            if (authorizationRuleService != null)
            {
                AuthorizationConfigurationSection authorizationSection = moduleConfiguration.GetSection(AuthorizationSection) as AuthorizationConfigurationSection;
                if (authorizationSection != null)
                {
                    foreach (AuthorizationRuleElement ruleElement in authorizationSection.ModuleRules)
                    {
                        authorizationRuleService.RegisterAuthorizationRule(ruleElement.AbsolutePath, ruleElement.RuleName);
                    }
                }
            }
        }

        protected virtual void RegisterTranslators(IEntityTranslatorService entityTranslatorService)
        {
            entityTranslatorService.RegisterEntityTranslator(new SecurityContextTranslator());
            entityTranslatorService.RegisterEntityTranslator(new InstitutionTranslator());
            entityTranslatorService.RegisterEntityTranslator(new ActionTranslator());
            entityTranslatorService.RegisterEntityTranslator(new RoleTranslator());
            entityTranslatorService.RegisterEntityTranslator(new UserSearchCriteriaTranslator());
            entityTranslatorService.RegisterEntityTranslator(new UserTranslator());
            entityTranslatorService.RegisterEntityTranslator(new IdentityTranslator());
        }
    }
}