using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.CompositeWeb.Services;
using Microsoft.Practices.ObjectBuilder;
using service = OHE.FoundationModules.Authentication.ServiceProxy;
using client = OHE.Security.DataTypes;
using OHE.FoundationModules.ProviderServices.Base;

namespace OHE.Admin.Security.Module.Services
{
    public class AdministrationServiceAgent : IAdministrationServiceAgent
    {
        #region Private Members

        private service.IAdministrationServiceProxy _proxy = null;
        private IEntityTranslatorService _translator = null;
 
        #endregion

        #region Constructors
        [InjectionConstructor]
        public AdministrationServiceAgent
            (
                [ServiceDependency] IEntityTranslatorService translator
            )
        {
            _translator = translator;
            _proxy = new service.AdministrationSerivceProxy();
        }

        public AdministrationServiceAgent()
        {
        }

        #endregion


        #region IAuthenticationServiceAgent Members

        public void CreateSecurityContext(client.SecurityContext securityContext)
        {
            service.CreateSecurityContextRequest request = new service.CreateSecurityContextRequest();
            service.CreateSecurityContextResponse response = new service.CreateSecurityContextResponse();

            request.SecurityContext = _translator.Translate<service.SecurityContext>(securityContext);

            response = _proxy.CreateSecurityContext(request);

            securityContext = _translator.Translate<client.SecurityContext>(response.SecurityContext);
        }

        public List<client.SecurityContext> GetAllSecurityContexts()
        {
            service.GetAllSecurityContextsRequest request = new service.GetAllSecurityContextsRequest();
            service.GetAllSecurityContextsResponse response = new service.GetAllSecurityContextsResponse();
            List<client.SecurityContext> retValue = new List<client.SecurityContext>();

            response = _proxy.GetAllSecurityContexts(request);

            foreach (service.SecurityContext sc in response.SecurityContexts)
            {
                retValue.Add(_translator.Translate<client.SecurityContext>(sc));
            }

            return retValue;
        }

        public void DeleteSecurityContext(client.SecurityContext securityContext)
        {
            service.DeleteSecurityContextRequest request = new service.DeleteSecurityContextRequest();
            service.DeleteSecurityContextResponse response = new service.DeleteSecurityContextResponse();

            request.SecurityContext = _translator.Translate<service.SecurityContext>(securityContext);

            response = _proxy.DeleteSecurityContext(request);
        }

        public void UpdateSecurityContext(client.SecurityContext securityContext)
        {
            service.UpdateSecurityContextRequest request = new service.UpdateSecurityContextRequest();
            service.UpdateSecurityContextResponse response = new service.UpdateSecurityContextResponse();

            request.SecurityContext = _translator.Translate<service.SecurityContext>(securityContext);

            response = _proxy.UpdateSecurityContext(request);
        }

        public void CreateInstitution(client.Institution institution)
        {
            service.CreateInstitutionRequest request = new service.CreateInstitutionRequest();
            service.CreateInstitutionResponse response = new service.CreateInstitutionResponse();

            request.Institution = _translator.Translate<service.Institution>(institution);

            response = _proxy.CreateInstitution(request);

            institution = _translator.Translate<client.Institution>(response.Institution);
        }

        public List<client.Institution> GetAllInstitutions()
        {
            service.GetAllInstitutionsRequest request = new service.GetAllInstitutionsRequest();
            service.GetAllInstitutionsResponse response = new service.GetAllInstitutionsResponse();
            List<client.Institution> retValue = new List<client.Institution>();

            response = _proxy.GetAllInstitutions(request);

            foreach (service.Institution institution in response.Institutions)
            {
                retValue.Add(_translator.Translate<client.Institution>(institution));
            }

            return retValue;
        }

        public void DeleteInstitution(client.Institution institution)
        {
            service.DeleteInstitutionRequest request = new service.DeleteInstitutionRequest();
            service.DeleteInstitutionResponse response = new service.DeleteInstitutionResponse();

            request.Institution = _translator.Translate<service.Institution>(institution);

            response = _proxy.DeleteInstitution(request);

        }

        public void UpdateInstitution(client.Institution institution)
        {
            service.UpdateInstitutionRequest request = new service.UpdateInstitutionRequest();
            service.UpdateInstitutionResponse response = new service.UpdateInstitutionResponse();

            request.Institution = _translator.Translate<service.Institution>(institution);

            response = _proxy.UpdateInstitution(request);
        }

        public client.Institution GetInstitution(int ID)
        {
            service.GetInstitutionRequest request = new service.GetInstitutionRequest();
            service.GetInstitutionResponse response = new service.GetInstitutionResponse();

            request.ID = ID;

            response = _proxy.GetInstitution(request);

            return _translator.Translate<client.Institution>(response.Institution);
        }

        public OHE.Security.DataTypes.SecurityContext GetSecurityContext(int ID)
        {
            service.GetSecurityContextRequest request = new service.GetSecurityContextRequest();
            service.GetSecurityContextResponse response = new service.GetSecurityContextResponse();

            request.ID = ID;

            response = _proxy.GetSecurityContext(request);

            return _translator.Translate<client.SecurityContext>(response.SecurityContext);
        }

        public List<OHE.Security.DataTypes.Action> GetAllActions(int securityContextID)
        {
            service.GetActionsRequest request = new service.GetActionsRequest();
            service.GetActionsResponse response = new service.GetActionsResponse();
            List<client.Action> retValue = new List<client.Action>();

            request.SecurityContextID = securityContextID;

            response = _proxy.GetActions(request);

            foreach (service.Action action in response.Actions)
            {
                retValue.Add(_translator.Translate<client.Action>(action));
            }

            return retValue;
        }

        public void DeleteAction(OHE.Security.DataTypes.Action action)
        {
            service.DeleteActionRequest request = new service.DeleteActionRequest();
            service.DeleteActionResponse response = new service.DeleteActionResponse();

            request.Action = _translator.Translate<service.Action>(action);

            response = _proxy.DeleteAction(request);
        }

        public void CreateAction(OHE.Security.DataTypes.Action action)
        {
            service.CreateActionRequest request = new service.CreateActionRequest();
            service.CreateActionResponse response = new service.CreateActionResponse();

            request.Action = _translator.Translate<service.Action>(action);

            response = _proxy.CreateAction(request);

            action = _translator.Translate<client.Action>(response.Action);
        }

        public void UpdateAction(OHE.Security.DataTypes.Action action)
        {
            service.UpdateActionRequest request = new service.UpdateActionRequest();
            service.UpdateActionResponse response = new service.UpdateActionResponse();

            request.Action = _translator.Translate<service.Action>(action);

            _proxy.UpdateAction(request);
        }

        public OHE.Security.DataTypes.Action GetAction(int ID)
        {
            service.GetActionRequest request = new service.GetActionRequest();
            service.GetActionResponse response = new service.GetActionResponse();

            request.ID = ID;

            response = _proxy.GetAction(request);

            return _translator.Translate<client.Action>(response.Action);
        }

        public List<client.User> FindUsersBySearchCriteria(client.UserSearchCriteria criteria, int startRowIndex, int maximumRows, out int totalRecords)
        {
            service.FindUsersBySearchCriteriaRequest request = new service.FindUsersBySearchCriteriaRequest();
            service.FindUsersBySearchCriteriaResponse response = new service.FindUsersBySearchCriteriaResponse();

            request.Criteria = _translator.Translate<service.UserSearchCriteria>(criteria);
            request.StartRowIndex = startRowIndex;
            request.MaximumRows = maximumRows;
            List<client.User> retValue = new List<client.User>();

            try
            {
                response = _proxy.FindUsersBySearchCriteria(request);                

                foreach (service.User user in response.Users)
                {
                    retValue.Add(_translator.Translate<client.User>(user));
                }

                totalRecords = response.TotalRecords;
            }
            catch
            {
                totalRecords = 0;                
            }

            return retValue;
        }

        #endregion
    }
}