using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.CompositeWeb.Services;
using Microsoft.Practices.ObjectBuilder;
using service = OHE.FoundationModules.Authentication.ServiceProxy;
using client = OHE.Security.DataTypes;
using OHE.FoundationModules.ProviderServices.Base;
using OHE.FoundationModules.Authentication.ServiceProxy;

namespace OHE.Admin.Security.Module.Services
{

    public class AuthenticationServiceAgent : IAuthenticationServiceAgent
    {
        #region Private Members

        private service.IAuthenticationServiceProxy _proxy = null;
        private IEntityTranslatorService _translator = null;
 
        #endregion

        #region Constructors
        [InjectionConstructor]
        public AuthenticationServiceAgent
            (
                [ServiceDependency] IEntityTranslatorService translator
            )
        {
            _translator = translator;
            _proxy = new service.AuthenticationServiceProxy();
        }

        public AuthenticationServiceAgent()
        {
        }

        #endregion




        #region IAuthenticationServiceAgent Members

        public List<string> CreateUser(OHE.Security.DataTypes.User user)
        {
            List<string> results = new List<string>();
            service.CreateUserRequest request = new service.CreateUserRequest();
            service.CreateUserResponse response = new service.CreateUserResponse();

            request.Email = user.Email;
            request.FirstName = user.FirstName;
            request.LastName = user.LastName;
                                             
            for(int i=0;i<user.Identities.Count;i++)
            {
                request.Password = user.Identities[i].Password;
                request.SystemID = user.Identities[i].SecurityContextId;
                request.SystemUserId = user.Identities[i].SystemUserID;
                request.UserName = user.Identities[i].UserName;

                response = _proxy.CreateUser(request);
                                                
                results.Add(response.Status.ToString());
            }
            

            return results;
        }

        public OHE.Security.DataTypes.User GetUser(int ID)
        {
            service.GetUserRequest request = new service.GetUserRequest();
            service.GetUserResponse respsone = new service.GetUserResponse();

            request.UserID = ID;

            respsone = _proxy.GetUser(request);

            return _translator.Translate<client.User>(respsone.User);
        }

        public OHE.Security.DataTypes.User GetUserWithAllIdentities(int ID)
        {
            service.GetUserWithAllIdentitiesRequest request = new service.GetUserWithAllIdentitiesRequest();
            service.GetUserWithAllIdentitiesResponse respsone = new service.GetUserWithAllIdentitiesResponse();
            
            request.UserID = ID;

            respsone = _proxy.GetUserWithAllIdentities(request);
            
            return _translator.Translate<client.User>(respsone.User);
        }

        public bool ResetPassword(OHE.Security.DataTypes.Institution institution, string userName, int systemID)
        {
            Institutions i = (Institutions)Enum.Parse(typeof(Institutions), institution.ID.ToString());
            service.ResetPasswordRequest request = new service.ResetPasswordRequest(i, userName, systemID);
            return _proxy.ResetPassword(request).Result;
            
        }

        public bool UpdateIdentity(int userID, OHE.Security.DataTypes.Identity identity)
        {
            service.UpdateIdentityRequest request = new UpdateIdentityRequest(userID, identity);
            try
            {
                return _proxy.UpdateIdentity(request).Result;
            }
            catch
            {
                return false;
            }
            
        }

        public bool UpdatePassword(OHE.Security.DataTypes.Identity identity, string oldPassword)
        {                      
            service.ChangePasswordRequest request = new ChangePasswordRequest(); 
            request.NewPassword = identity.Password;
            request.OldPassword = oldPassword;
            request.SystemID = identity.SystemUserID;
            request.UserName = identity.UserName;
            
            return _proxy.ChangePassword(request).Result;

            
        }        
        #endregion
    }
}