using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.CompositeWeb;
using Microsoft.Practices.CompositeWeb.Services;
using Microsoft.Practices.ObjectBuilder;
using service = OHE.FoundationModules.Authentication.ServiceProxy;
using client = OHE.Security.DataTypes;
using OHE.FoundationModules.ProviderServices.Base;

namespace OHE.Admin.Security.Module.Services
{
    public class RoleServiceAgent : IRoleServiceAgent
    {
        #region Private Members

        private service.IRoleServiceProxy _proxy;
        private IEntityTranslatorService _translator = null;
 
        #endregion

        #region Constructors
        [InjectionConstructor]
        public RoleServiceAgent
            (
                [ServiceDependency] IEntityTranslatorService translator
            )
        {
            _translator = translator;
            _proxy = new service.RoleSerivceProxy();
        }

        public RoleServiceAgent()
        {
        }

        #endregion


        #region IRoleServiceAgent Members


        public List<OHE.Security.DataTypes.Role> GetAllRoles(int securityContextID)
        {
            service.GetRolesRequest request = new service.GetRolesRequest();
            service.GetRolesResponse response = new service.GetRolesResponse();
            List<client.Role> retValue = new List<client.Role>();

            request.SecurityContextID = securityContextID;

            response = _proxy.GetRoles(request);

            foreach (service.Role role in response.Roles)
            {
                retValue.Add(_translator.Translate<client.Role>(role));
            }

            return retValue;
        }

        public void DeleteRole(OHE.Security.DataTypes.Role role)
        {
            service.DeleteRoleRequest request = new service.DeleteRoleRequest();
            service.DeleteRoleResponse response = new service.DeleteRoleResponse();

            request.Role = _translator.Translate<service.Role>(role);

            response = _proxy.DeleteRole(request);
        }

        public void CreateRole(OHE.Security.DataTypes.Role role)
        {
            service.CreateRoleRequest request = new service.CreateRoleRequest();
            service.CreateRoleResponse response = new service.CreateRoleResponse();

            request.Role = _translator.Translate<service.Role>(role);

            response = _proxy.CreateRole(request);

            role = _translator.Translate<client.Role>(response.Role);
        }

        public void UpdateRole(OHE.Security.DataTypes.Role role)
        {
            service.UpdateRoleRequest request = new service.UpdateRoleRequest();
            service.UpdateRoleResponse response = new service.UpdateRoleResponse();

            request.Role = _translator.Translate<service.Role>(role);

            _proxy.UpdateRole(request);
        }

        public OHE.Security.DataTypes.Role GetRole(int ID)
        {
            service.GetRoleRequest request = new service.GetRoleRequest();
            service.GetRoleResponse response = new service.GetRoleResponse();

            request.ID = ID;

            response = _proxy.GetRole(request);

            return _translator.Translate<client.Role>(response.Role);
        }

        #endregion
    }
}