﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Admin.Security.Module;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter for the action detail page.
    /// </summary>
    public class ActionDetailsPresenter : Presenter<IActionDetailsView>
    {
        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller.
        /// </summary>
        /// <param name="controller">The controller for the presenter</param>
        public ActionDetailsPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// 
        /// </summary>
        public override void OnViewInitialized()
        {
            base.OnViewInitialized();
        }

        /// <summary>
        /// Sets the Action property
        /// </summary>
        /// <param name="ID">The ID of the action</param>
        public void OnGetAction(int ID)
        {
            this.View.Action = _controller.GetAction(ID);
        }

        /// <summary>
        /// Inserts a new action
        /// </summary>
        /// <param name="action">The action object to be inserted</param>
        public void OnActionInserted(Action action)
        {
            action.SecurityContextID = this.View.SecurityContextID;
            _controller.CreateAction(action);
        }

        /// <summary>
        /// Updates an action
        /// </summary>
        /// <param name="action">The action object to be updated</param>
        public void OnActionUpdated(Action action)
        {
            _controller.UpdateAction(action);
        }

        /// <summary>
        /// Cancels an action
        /// </summary>
        public void OnCancel()
        {
            _controller.Redirect(String.Format(Constants.URLS.ActionList, this.View.SecurityContextID));
        }

    }
}




