﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter class for the Action Listing page.
    /// </summary>
    public class ActionListPresenter : Presenter<IActionListView>
    {

        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller
        /// </summary>
        /// <param name="controller">The controller</param>
        public ActionListPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// Sets the Actions property to the list of actions from the controller when the view is loaded.
        /// </summary>
        public override void OnViewLoaded()
        {
            this.View.Actions = _controller.GetAllActions(this.View.SecurityContextID);
        }

        /// <summary>
        /// Deletes the selected action.
        /// </summary>
        /// <param name="ID">The action to be deleted</param>
        public void OnActionDeleted(Action action)
        {
            _controller.DeleteAction(action);
        }
    }
}




