﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Admin.Security.Module;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter for the institution detail page.
    /// </summary>
    public class InstitutionDetailsPresenter : Presenter<IInstitutionDetailsView>
    {
        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller.
        /// </summary>
        /// <param name="controller">The controller for the presenter</param>
        public InstitutionDetailsPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// 
        /// </summary>
        public override void OnViewInitialized()
        {
            base.OnViewInitialized();
        }

        /// <summary>
        /// Sets the Institution property
        /// </summary>
        /// <param name="ID">The ID of the instituton</param>
        public void OnGetInstitution(int ID)
        {
            this.View.Institution = _controller.GetInstitution(ID);
        }

        /// <summary>
        /// Inserts a new user
        /// </summary>
        /// <param name="user">The institution object to be inserted</param>
        public void OnInstitutionInserted(Institution institution)
        {
            _controller.CreateInstitution(institution);
        }

        /// <summary>
        /// Updates an institution
        /// </summary>
        /// <param name="user">The institution object to be updated</param>
        public void OnInstitutionUpdated(Institution institution)
        {
            _controller.UpdateInstitution(institution);
        }

        /// <summary>
        /// Cancels an action
        /// </summary>
        public void OnCancel()
        {
            _controller.Redirect(Constants.URLS.InstitutionList);
        }

    }
}




