﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Admin.Security.Module;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter for the role detail page.
    /// </summary>
    public class RoleDetailsPresenter : Presenter<IRoleDetailsView>
    {
        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller.
        /// </summary>
        /// <param name="controller">The controller for the presenter</param>
        public RoleDetailsPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// 
        /// </summary>
        public override void OnViewInitialized()
        {
            base.OnViewInitialized();
        }

        /// <summary>
        /// Sets the Role property
        /// </summary>
        /// <param name="ID">The ID of the role</param>
        public void OnGetRole(int ID)
        {
            this.View.Role = _controller.GetRole(ID);
        }

        /// <summary>
        /// Inserts a new role
        /// </summary>
        /// <param name="role">The role object to be inserted</param>
        public void OnRoleInserted(Role role)
        {
            role.SecurityContextID = this.View.SecurityContextID;
            _controller.CreateRole(role);
        }

        /// <summary>
        /// Updates an role
        /// </summary>
        /// <param name="role">The role object to be updated</param>
        public void OnRoleUpdated(Role role)
        {
            _controller.UpdateRole(role);
        }

        /// <summary>
        /// Cancels an action
        /// </summary>
        public void OnCancel()
        {
            _controller.Redirect(String.Format(Constants.URLS.RoleList, this.View.SecurityContextID));
        }

    }
}




