﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter class for the Role Listing page.
    /// </summary>
    public class RoleListPresenter : Presenter<IRoleListView>
    {

        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller
        /// </summary>
        /// <param name="controller">The controller</param>
        public RoleListPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// Sets the Roles property to the list of roles from the controller when the view is loaded.
        /// </summary>
        public override void OnViewLoaded()
        {
            this.View.Roles = _controller.GetAllRoles(this.View.SecurityContextID);
        }

        /// <summary>
        /// Deletes the selected role.
        /// </summary>
        /// <param name="ID">The role to be deleted</param>
        public void OnRoleDeleted(Role role)
        {
            _controller.DeleteRole(role);
        }
    }
}




