﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Admin.Security.Module;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter for the securitycontext detail page.
    /// </summary>
    public class SecurityContextDetailsPresenter : Presenter<ISecurityContextDetailsView>
    {
        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller.
        /// </summary>
        /// <param name="controller">The controller for the presenter</param>
        public SecurityContextDetailsPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// 
        /// </summary>
        public override void OnViewInitialized()
        {
            base.OnViewInitialized();
        }

        /// <summary>
        /// Sets the SecurityContext property
        /// </summary>
        /// <param name="ID">The ID of the securitycontext</param>
        public void OnGetSecurityContext(int ID)
        {
            this.View.SecurityContext = _controller.GetSecurityContext(ID);
        }

        /// <summary>
        /// Inserts a new securitycontext
        /// </summary>
        /// <param name="securityContext">The securitycontext object to be inserted</param>
        public void OnSecurityContextInserted(SecurityContext securityContext)
        {
            _controller.CreateSecurityContext(securityContext);
        }

        /// <summary>
        /// Updates an securitycontext
        /// </summary>
        /// <param name="securityContext">The securitycontext object to be updated</param>
        public void OnSecurityContextUpdated(SecurityContext securityContext)
        {
            _controller.UpdateSecurityContext(securityContext);
        }

        /// <summary>
        /// Cancels an action
        /// </summary>
        public void OnCancel()
        {
            _controller.Redirect(Constants.URLS.SecurityContextList);
        }

    }
}




