﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Security.DataTypes;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter class for the SecurityContext Listing page.
    /// </summary>
    public class SecurityContextListPresenter : Presenter<ISecurityContextListView>
    {

        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller
        /// </summary>
        /// <param name="controller">The controller</param>
        public SecurityContextListPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        /// <summary>
        /// Sets the SecurityContexts property to the list of securitycontexts from the controller when the view is loaded.
        /// </summary>
        public override void OnViewLoaded()
        {
            this.View.SecurityContexts = _controller.GetAllSecurityContexts();
        }

        /// <summary>
        /// Deletes the selected securitycontext.
        /// </summary>
        /// <param name="securityContext">The securitycontext to be deleted</param>
        public void OnSecurityContextDeleted(SecurityContext securityContext)
        {
            _controller.DeleteSecurityContext(securityContext);
        }
    }
}




