﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeWeb;
using OHE.Admin.Security.Module;
using OHE.Security.DataTypes;
using OHE.Web.Security;

namespace OHE.Admin.Security.Module.Views
{
    /// <summary>
    /// The presenter for the action detail page.
    /// </summary>
    public class UserSearchPresenter : Presenter<IUserSearchView>
    {
        private SecurityController _controller;

        /// <summary>
        /// Constructor.  Sets the reference to the controller.
        /// </summary>
        /// <param name="controller">The controller for the presenter</param>
        public UserSearchPresenter([CreateNew] SecurityController controller)
        {
        	_controller = controller;
        }

        public void OnSelecting(UserSearchCriteria criteria, int pageIndex, int pageSize)
        {
            int totalRecords = 0;
            this.View.Users = _controller.FindUsersBySeachCriteria(criteria, pageIndex, pageSize, out totalRecords);
            this.View.TotalUsersCount = totalRecords;
        }

        public List<User> FindUsersBySearchCriteria(UserSearchCriteria criteria, int pageIndex, int pagesize) 
        {
            int totalRecords = 0;
            return _controller.FindUsersBySeachCriteria(criteria, pageIndex, pagesize, out totalRecords);
        }

        public void OnDetailSelecting(int userId)
        {            
            this.View.Identities = _controller.GetUserWithAllIdentities(userId).Identities;
        }

        public bool ResetPassword(OHE.Security.DataTypes.Institution institution, string userName, int systemID)
        {
            return _controller.ResetPassword(institution, userName, systemID);
        }

        public bool UpdateSecurityIdentity(Institution institution, int userID, int systemUserId, int systemID, bool active, string hint, string hintAnswer, bool isLockedOut, string userName, string updateOperator)
        {
            return _controller.UpdateSecurityIdentity(institution, userID, systemUserId, systemID, active, hint, hintAnswer, isLockedOut, userName, updateOperator);
        }

        public List<string> CreateUser(User user)
        {
            return _controller.CreateUser(user);
        }

        public List<SecurityContext> GetAllSecurityContexts()
        {
            return _controller.GetAllSecurityContexts();
        }

        public List<Institution> GetAllInstitutions()
        {
            return _controller.GetAllInstitutions();
        }

        public bool UpdatePassword(string userName, string password, string oldPassword, int systemUserId)
        {
            return _controller.UpdatePassword(userName, password, oldPassword, systemUserId);
        }
    }
}




