﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// A class that converts a BannerCourseStub
    /// into a DbCommand that will delete
    /// that BannerCourseStub.
    /// </summary>
    internal class BannerCourseStubDeleteFactory : IDeleteFactory<DeleteMasterCourseBannerCourseIdentity>
    {
        /// <summary>
        /// Creates the BannerCourseStubDeleteFactory to build a delete statement for
        /// the given DeleteMasterCourseBannerCourseIdentity.
        /// </summary>
        /// <param name="DeleteMasterCourseBannerCourseIdentity">DeleteMasterCourseBannerCourseIdentity to delete from the database.</param>    
        public BannerCourseStubDeleteFactory()
        {
        }

        #region IDeleteFactory<DeleteMasterCourseBannerCourseIdentity> Members
        public DbCommand ConstructDeleteCommand(Database db, DeleteMasterCourseBannerCourseIdentity deleteMasterCourseBannerCourseIdentity)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.DeleteMasterCourseBannerCourse");
            db.AddInParameter(command, "bannerCourseCode", DbType.String, deleteMasterCourseBannerCourseIdentity.BannerCourseCode);
            db.AddInParameter(command, "masterCourseID", DbType.Int32, deleteMasterCourseBannerCourseIdentity.MasterCourseID);
            return command;
        }
        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "bannerCourseCode":
                    return "bannerCourseCode";
                case "masterCourseID":
                    return "masterCourseID";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }
}

