﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a BannerCourseStub
    /// and generating the corresponding SQL to insert that
    /// BannerCourseStub into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// BannerCourseStub object with it.
    /// </summary>
    internal class BannerCourseStubInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<BannerCourseStub>
    {
        /// <summary>
        /// Creates the BannerCourseStubInsertFactory to build an insert statement for
        /// the given BannerCourseStub object.
        /// </summary>
        /// <param name="BannerCourseStub">New BannerCourseStub to insert into the database.</param>
        public BannerCourseStubInsertFactory()
        {
        }

        #region IInsertFactory<BannerCourseStub> Members

        public DbCommand ConstructInsertCommand(Database db, BannerCourseStub bannerCourseStub)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.InsertMasterCourseBannerCourse");

            db.AddInParameter(command, "masterCourseID", DbType.Int32, bannerCourseStub.MasterCourseID);
            if (bannerCourseStub.BannerCourseCode != null)
            {
                db.AddInParameter(command, "bannerCourseCode", DbType.String, bannerCourseStub.BannerCourseCode);
            }
            return command;
        }

        public void SetNewID(Database db, DbCommand command, BannerCourseStub bannerCourseStub)
        {
            //TODO: Provide set mapping
        }

        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "masterCourseID":
                    return "MasterCourseID";
                case "bannerCourseCode":
                    return "BannerCourseCode";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }

}

