﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find BannerCourseStub in the
    /// CRM database.
    /// </summary>
    public class BannerCourseStubRepository : Repository<BannerCourseStub>
    {
        private string databaseName;

        public BannerCourseStubRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }


        public List<BannerCourseStub> GetMasterCourseBannerCourseByMasterCourseID(System.Int32 masterCourseID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetMasterCourseBannerCourseByMasterCourseIDSelectionFactory();

            try
            {
                return base.Find(selectionFactory, new GetMasterCourseBannerCourseByMasterCourseIDFactory(), masterCourseID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<BannerCourseStub>();
        }

        public void Add(BannerCourseStub bannerCourseStub)
        {
            BannerCourseStubInsertFactory insertFactory = new BannerCourseStubInsertFactory();
            try
            {
                base.Add(insertFactory, bannerCourseStub);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }

        public void Remove(System.String bannerCourseCode, System.Int32 masterCourseID)
        {
            IDeleteFactory<DeleteMasterCourseBannerCourseIdentity> deleteFactory = new BannerCourseStubDeleteFactory();

            try
            {
                DeleteMasterCourseBannerCourseIdentity deleteMasterCourseBannerCourseIdentity = new DeleteMasterCourseBannerCourseIdentity(bannerCourseCode, masterCourseID);
                base.Remove(deleteFactory, deleteMasterCourseBannerCourseIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, deleteFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

