﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a CopyHistory
    /// and generating the corresponding SQL to insert that
    /// CopyHistory into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// CopyHistory object with it.
    /// </summary>
    internal class CopyHistoryInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<CopyHistory>
    {
        /// <summary>
        /// Creates the CopyHistoryInsertFactory to build an insert statement for
        /// the given CopyHistory object.
        /// </summary>
        /// <param name="CopyHistory">New CopyHistory to insert into the database.</param>
        public CopyHistoryInsertFactory()
        {
        }

        #region IInsertFactory<CopyHistory> Members

        public DbCommand ConstructInsertCommand(Database db, CopyHistory copyHistory)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.InsertCopyHistory");
            db.AddInParameter(command, "handledManually", DbType.Boolean, copyHistory.HandledManually);
            db.AddInParameter(command, "bannerCourseStartDate", DbType.DateTime, copyHistory.BannerCourseStartDate);
            if (copyHistory.BannerSectionCode != null)
            {
                db.AddInParameter(command, "bannerSectionCode", DbType.String, copyHistory.BannerSectionCode);
            }
            if (copyHistory.InstructorFirstName != null)
            {
                db.AddInParameter(command, "instructorFirstName", DbType.String, copyHistory.InstructorFirstName);
            }
            if (copyHistory.InstructorMiddleInitial != null)
            {
                db.AddInParameter(command, "instructorMiddleInitial", DbType.String, copyHistory.InstructorMiddleInitial);
            }
            if (copyHistory.InstructorLastName != null)
            {
                db.AddInParameter(command, "instructorLastName", DbType.String, copyHistory.InstructorLastName);
            }
            if (copyHistory.BatchNumber != null)
            {
                db.AddInParameter(command, "batchNumber", DbType.String, copyHistory.BatchNumber);
            }
            if (copyHistory.BannerCourseCode != null)
            {
                db.AddInParameter(command, "bannerCourseCode", DbType.String, copyHistory.BannerCourseCode);
            }
            if (copyHistory.BannerCourseTitle != null)
            {
                db.AddInParameter(command, "bannerCourseTitle", DbType.String, copyHistory.BannerCourseTitle);
            }
            if (copyHistory.BannerCRN != null)
            {
                db.AddInParameter(command, "bannerCRN", DbType.String, copyHistory.BannerCRN);
            }
            if (copyHistory.BannerGroupCode != null)
            {
                db.AddInParameter(command, "bannerGroupCode", DbType.String, copyHistory.BannerGroupCode);
            }
            if (copyHistory.BannerNode != null)
            {
                db.AddInParameter(command, "bannerNode", DbType.String, copyHistory.BannerNode);
            }
            if (copyHistory.BannerSchoolName != null)
            {
                db.AddInParameter(command, "bannerSchoolName", DbType.String, copyHistory.BannerSchoolName);
            }
            if (copyHistory.BannerSectionID != null)
            {
                db.AddInParameter(command, "bannerSectionID", DbType.String, copyHistory.BannerSectionID);
            }
            if (copyHistory.BannerSubTerm != null)
            {
                db.AddInParameter(command, "bannerSubTerm", DbType.String, copyHistory.BannerSubTerm);
            }
            if (copyHistory.BannerSubTermDescription != null)
            {
                db.AddInParameter(command, "bannerSubTermDescription", DbType.String, copyHistory.BannerSubTermDescription);
            }
            if (copyHistory.BannerTerm != null)
            {
                db.AddInParameter(command, "bannerTerm", DbType.String, copyHistory.BannerTerm);
            }
            if (copyHistory.BannerTermDescription != null)
            {
                db.AddInParameter(command, "bannerTermDescription", DbType.String, copyHistory.BannerTermDescription);
            }
            if (copyHistory.CopierID != null)
            {
                db.AddInParameter(command, "copierID", DbType.String, copyHistory.CopierID);
            }
            if (copyHistory.DateCopied != null)
            {
                db.AddInParameter(command, "dateCopied", DbType.DateTime, copyHistory.DateCopied);
            }
            if (copyHistory.ECollegeTerm != null)
            {
                db.AddInParameter(command, "eCollegeTerm", DbType.String, copyHistory.ECollegeTerm);
            }
            if (copyHistory.ECollegeTermECollegeID != null)
            {
                db.AddInParameter(command, "eCollegeTermECollegeID", DbType.String, copyHistory.ECollegeTermECollegeID);
            }
            if (copyHistory.ECollegeTermID != null)
            {
                db.AddInParameter(command, "eCollegeTermID", DbType.Int32, copyHistory.ECollegeTermID);
            }
            if (copyHistory.MasterCourseDisplayCode != null)
            {
                db.AddInParameter(command, "masterCourseDisplayCode", DbType.String, copyHistory.MasterCourseDisplayCode);
            }
            if (copyHistory.MasterCourseECollegeID != null)
            {
                db.AddInParameter(command, "masterCourseECollegeID", DbType.String, copyHistory.MasterCourseECollegeID);
            }
            if (copyHistory.MasterCourseID != null)
            {
                db.AddInParameter(command, "masterCourseID", DbType.Int32, copyHistory.MasterCourseID);
            }
            if (copyHistory.MasterCourseSourceNode != null)
            {
                db.AddInParameter(command, "masterCourseSourceNode", DbType.String, copyHistory.MasterCourseSourceNode);
            }
            if (copyHistory.MasterCourseTitle != null)
            {
                db.AddInParameter(command, "masterCourseTitle", DbType.String, copyHistory.MasterCourseTitle);
            }
            db.AddOutParameter(command, "copyHistoryID", DbType.Int32, 4);
            return command;
        }

        public void SetNewID(Database db, DbCommand command, CopyHistory copyHistory)
        {
            System.Int32 id1 = (System.Int32)(db.GetParameterValue(command, "copyHistoryID"));
            copyHistory.CopyHistoryID = id1;

        }

        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "bannerCourseCode":
                    return "BannerCourseCode";
                case "bannerCourseTitle":
                    return "BannerCourseTitle";
                case "bannerCRN":
                    return "BannerCRN";
                case "bannerGroupCode":
                    return "BannerGroupCode";
                case "bannerNode":
                    return "BannerNode";
                case "bannerSchoolName":
                    return "BannerSchoolName";
                case "bannerSectionID":
                    return "BannerSectionID";
                case "bannerSubTerm":
                    return "BannerSubTerm";
                case "bannerSubTermDescription":
                    return "BannerSubTermDescription";
                case "bannerTerm":
                    return "BannerTerm";
                case "bannerTermDescription":
                    return "BannerTermDescription";
                case "batchNumber":
                    return "BatchNumber";
                case "copierID":
                    return "CopierID";
                case "dateCopied":
                    return "DateCopied";
                case "eCollegeTerm":
                    return "ECollegeTerm";
                case "eCollegeTermECollegeID":
                    return "ECollegeTermECollegeID";
                case "eCollegeTermID":
                    return "ECollegeTermID";
                case "masterCourseDisplayCode":
                    return "MasterCourseDisplayCode";
                case "masterCourseECollegeID":
                    return "MasterCourseECollegeID";
                case "masterCourseID":
                    return "MasterCourseID";
                case "masterCourseSourceNode":
                    return "MasterCourseSourceNode";
                case "masterCourseTitle":
                    return "MasterCourseTitle";
                case "instructorFirstName":
                    return "InstructorFirstName";
                case "instructorMiddleInitial":
                    return "InstructorMiddleInitial";
                case "instructorLastName":
                    return "InstructorLastName";
                case "bannerSectionCode":
                    return "BannerSectionCode";
                case "bannerCourseStartDate":
                    return "BannerCourseStartDate";
                case "handledManually":
                    return "HandledManually";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }

}

