﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find CopyHistory in the
    /// CRM database.
    /// </summary>
    public class CopyHistoryRepository : Repository<CopyHistory>
    {
        private string databaseName;

        public CopyHistoryRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }

        public CopyHistory GetLatestBatchNumber()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetLatestBatchNumberSelectionFactory();
            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.FindOne(selectionFactory, new GetLatestBatchNumberFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            return new CopyHistory();
        }

        public List<CopyHistory> GetAllFromCopyHistory()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromCopyHistorySelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllFromCopyHistoryFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<CopyHistory>();
        }

        public CopyHistory GetCopyHistoryByCopyHistoryID(System.Int32 copyHistoryID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetCopyHistoryByCopyHistoryIDSelectionFactory();

            try
            {
                return base.FindOne(selectionFactory, new GetCopyHistoryByCopyHistoryIDFactory(), copyHistoryID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new CopyHistory();
        }

        public List<CopyHistory> GetCopyHistoryByCourseTermSubTerm(System.String courseCode, System.String subTerm, System.String term)
        {
            ISelectionFactory<GetCopyHistoryByCourseTermSubTermIdentity> selectionFactory = new GetCopyHistoryByCourseTermSubTermSelectionFactory();

            try
            {

                GetCopyHistoryByCourseTermSubTermIdentity getCopyHistoryByCourseTermSubTermIdentity = new GetCopyHistoryByCourseTermSubTermIdentity(courseCode, subTerm, term);
                return base.Find(selectionFactory, new GetCopyHistoryByCourseTermSubTermFactory(), getCopyHistoryByCourseTermSubTermIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<CopyHistory>();
        }

        public List<CopyHistory> GetCopyHistoryByBatchNumber(System.String batchNumber)
        {
            ISelectionFactory<System.String> selectionFactory = new GetCopyHistoryByBatchNumberSelectionFactory();

            try
            {
                return base.Find(selectionFactory, new GetCopyHistoryByBatchNumberFactory(), batchNumber);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<CopyHistory>();
        }


        public void Add(CopyHistory copyHistory)
        {
            CopyHistoryInsertFactory insertFactory = new CopyHistoryInsertFactory();
            try
            {
                base.Add(insertFactory, copyHistory);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }

        public void Remove(System.Int32 copyHistoryID)
        {
            IDeleteFactory<System.Int32> deleteFactory = new CopyHistoryDeleteFactory();

            try
            {
                base.Remove(deleteFactory, copyHistoryID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, deleteFactory);
            }
        }


        public void Save(CopyHistory copyHistory)
        {
            CopyHistoryUpdateFactory updateFactory = new CopyHistoryUpdateFactory();
            try
            {
                base.Save(updateFactory, copyHistory);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

