﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find Course in the
    /// CRM database.
    /// </summary>
    public class CourseRepository : Repository<Course>
    {
        private string databaseName;

        public CourseRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }


        public List<Course> GetAllFromCourse()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromCourseSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllFromCourseFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<Course>();
        }

        public Course GetCourseByCourseCode(System.String courseCode)
        {
            ISelectionFactory<System.String> selectionFactory = new GetCourseByCourseCodeSelectionFactory();

            try
            {
                return base.FindOne(selectionFactory, new GetCourseByCourseCodeFactory(), courseCode);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new Course();
        }

        public List<Course> GetCourseByTermSubTermSchool(System.String term, System.String subTerm, System.String schoolName)
        {
            ISelectionFactory<GetCourseByTermSubTermSchoolIdentity> selectionFactory = new GetCourseByTermSubTermSchoolSelectionFactory();
            try
            {
                GetCourseByTermSubTermSchoolIdentity getCourseByTermSubTermIdentity = new GetCourseByTermSubTermSchoolIdentity(term, subTerm, schoolName);
                return base.Find(selectionFactory, new GetCourseByTermSubTermSchoolFactory(), getCourseByTermSubTermIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            return new List<Course>();
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

