﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a ECollegeTerm
    /// and generating the corresponding SQL to insert that
    /// ECollegeTerm into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// ECollegeTerm object with it.
    /// </summary>
    internal class ECollegeTermInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<ECollegeTerm>
    {
        /// <summary>
        /// Creates the ECollegeTermInsertFactory to build an insert statement for
        /// the given ECollegeTerm object.
        /// </summary>
        /// <param name="ECollegeTerm">New ECollegeTerm to insert into the database.</param>
        public ECollegeTermInsertFactory()
        {
        }

        #region IInsertFactory<ECollegeTerm> Members

        public DbCommand ConstructInsertCommand(Database db, ECollegeTerm eCollegeTerm)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.InsertECollegeTerm");

            db.AddInParameter(command, "isActive", DbType.Boolean, eCollegeTerm.IsActive);
            if (eCollegeTerm.CourseStartDate != null)
            {
                db.AddInParameter(command, "courseStartDate", DbType.DateTime, eCollegeTerm.CourseStartDate);
            }
            if (eCollegeTerm.eCollegeID != null)
            {
                db.AddInParameter(command, "eCollegeID", DbType.String, eCollegeTerm.eCollegeID);
            }
            if (eCollegeTerm.eCollegeTerm != null)
            {
                db.AddInParameter(command, "eCollegeTerm", DbType.String, eCollegeTerm.eCollegeTerm);
            }
            db.AddOutParameter(command, "termID", DbType.Int32, 4);
            return command;
        }

        public void SetNewID(Database db, DbCommand command, ECollegeTerm eCollegeTerm)
        {
            System.Int32 id1 = (System.Int32)(db.GetParameterValue(command, "termID"));
            eCollegeTerm.TermID = id1;

        }

        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "isActive":
                    return "IsActive";
                case "eCollegeID":
                    return "eCollegeID";
                case "eCollegeTerm":
                    return "eCollegeTerm";
                case "courseStartDate":
                    return "CourseStartDate";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }

}

