﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find ECollegeTerm in the
    /// CRM database.
    /// </summary>
    public class ECollegeTermRepository : Repository<ECollegeTerm>
    {
        private string databaseName;

        public ECollegeTermRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }


        public List<ECollegeTerm> GetAllFromECollegeTerm()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromECollegeTermSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllFromECollegeTermFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<ECollegeTerm>();
        }

        public List<ECollegeTerm> GetAllActiveECollegeTerm()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllActiveECollegeTermSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllActiveECollegeTermFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<ECollegeTerm>();
        }

        public ECollegeTerm GetECollegeTermByTermID(System.Int32 termID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetECollegeTermByTermIDSelectionFactory();

            try
            {
                return base.FindOne(selectionFactory, new GetECollegeTermByTermIDFactory(), termID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new ECollegeTerm();
        }

        public void Add(ECollegeTerm eCollegeTerm)
        {
            ECollegeTermInsertFactory insertFactory = new ECollegeTermInsertFactory();
            try
            {
                base.Add(insertFactory, eCollegeTerm);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }


        public void Save(ECollegeTerm eCollegeTerm)
        {
            ECollegeTermUpdateFactory updateFactory = new ECollegeTermUpdateFactory();
            try
            {
                base.Save(updateFactory, eCollegeTerm);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

