﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// This factory class takes a ECollegeTerm object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class ECollegeTermUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<ECollegeTerm>
    {
        /// <summary>
        /// Creates the ECollegeTermUpdateFactory to build an update statement for
        /// the given ECollegeTerm object.
        /// </summary>
        /// <param name="ECollegeTerm">ECollegeTerm to update into the database.</param>
        public ECollegeTermUpdateFactory()
        {
        }

        #region IUpdateFactory<ECollegeTerm> Members

        public DbCommand ConstructUpdateCommand(Database db, ECollegeTerm eCollegeTerm)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.UpdateECollegeTerm");

            db.AddInParameter(command, "isActive", DbType.Boolean, eCollegeTerm.IsActive);
            db.AddInParameter(command, "termID", DbType.Int32, eCollegeTerm.TermID);
            if (eCollegeTerm.CourseStartDate != null)
            {
                db.AddInParameter(command, "courseStartDate", DbType.DateTime, eCollegeTerm.CourseStartDate);
            }
            if (eCollegeTerm.eCollegeTerm != null)
            {
                db.AddInParameter(command, "eCollegeTerm", DbType.String, eCollegeTerm.eCollegeTerm);
            }
            if (eCollegeTerm.eCollegeID != null)
            {
                db.AddInParameter(command, "eCollegeID", DbType.String, eCollegeTerm.eCollegeID);
            }
            return command;
        }
        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "isActive":
                    return "IsActive";
                case "termID":
                    return "TermID";
                case "eCollegeTerm":
                    return "eCollegeTerm";
                case "eCollegeID":
                    return "eCollegeID";
                case "courseStartDate":
                    return "CourseStartDate";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }
}

