﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;
using System.Data.OracleClient;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// This class is responsible for turning a selection criteria
    /// into a select statement and database query parameters.
    /// </summary>
    internal class GetSubTermBySubTermSelectionFactory : ISelectionFactory<System.String>
    {
        /// <summary>
        /// Creates the GetSubTermBySubTermSelectionFactory to build a select statement for
        /// the given System.String.
        /// </summary>
        /// <param name="System.String">System.String to select from the database.</param>      
        public GetSubTermBySubTermSelectionFactory()
        {
        }

		#region ISelectionFactory<System.String> Members
		public DbCommand ConstructSelectCommand(Database db, System.String subTerm)
		{
 		
            //DbCommand command = db.GetStoredProcCommand("dbo.GetSubTermBySubTerm");
            //db.AddInParameter(command, "subTerm", DbType.String, subTerm);
            //return command;

            DbCommand command = db.GetStoredProcCommand("LAUREATE.laur_course_copy.GetSubTermBySubTerm");
            db.AddInParameter(command, "str_subterm", DbType.String, subTerm);
            OracleParameter refCursor1 = new OracleParameter();
            refCursor1.OracleType = OracleType.Cursor;
            refCursor1.ParameterName = "subterm_ref";
            refCursor1.Direction = ParameterDirection.Output;
            command.Parameters.Add(refCursor1);
            return command;

        }
        #endregion
        
        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "subTerm":
					return "subTerm";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion  
    }
}

