﻿
----------------------------------------------------------------
-- [dbo].[MasterCourse] Table
--
IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'InsertMasterCourse')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[InsertMasterCourse] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[InsertMasterCourse]
    @courseDisplayCode nvarchar(50),
	@courseTitle nvarchar(100),
	@eCollegeID nvarchar(7),
	@isActive bit,
	@masterCourseID int OUT,
	@sourceNode nvarchar(50)
AS
BEGIN
	SET NOCOUNT ON
	
    INSERT INTO [dbo].[MasterCourse] ([CourseDisplayCode], [CourseTitle], [eCollegeID], [IsActive], [SourceNode])
	VALUES (@courseDisplayCode, @courseTitle, @eCollegeID, @isActive, @sourceNode)
    SET @masterCourseID = SCOPE_IDENTITY()
    
    SET NOCOUNT OFF
END    

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'UpdateMasterCourse')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[UpdateMasterCourse] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[UpdateMasterCourse]
    @courseDisplayCode nvarchar(50),
	@courseTitle nvarchar(100),
	@eCollegeID nvarchar(7),
	@isActive bit,
	@masterCourseID int,
	@sourceNode nvarchar(50)
AS
BEGIN

	--The [dbo].[MasterCourse] table doesn't have a timestamp column. Optimistic concurrency logic cannot be generated
	SET NOCOUNT ON

	UPDATE [dbo].[MasterCourse] 
	SET [CourseDisplayCode] = @courseDisplayCode, [CourseTitle] = @courseTitle, [eCollegeID] = @eCollegeID, [IsActive] = @isActive, [SourceNode] = @sourceNode
	WHERE [MasterCourseID]=@masterCourseID

	SET NOCOUNT OFF
END

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'GetAllFromMasterCourse')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[GetAllFromMasterCourse] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[GetAllFromMasterCourse]    
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT
	[masterCourse].[CourseDisplayCode] AS 'CourseDisplayCode',
	[masterCourse].[CourseTitle] AS 'CourseTitle',
	[masterCourse].[eCollegeID] AS 'eCollegeID',
	[masterCourse].[IsActive] AS 'IsActive',
	[masterCourse].[MasterCourseID] AS 'MasterCourseID',
	[masterCourse].[SourceNode] AS 'SourceNode'
FROM [dbo].[MasterCourse] [masterCourse]

	SET NOCOUNT OFF
END

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'GetMasterCourseByMasterCourseID')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[GetMasterCourseByMasterCourseID] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[GetMasterCourseByMasterCourseID] 
	@masterCourseID int
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT
	[masterCourse].[CourseDisplayCode] AS 'CourseDisplayCode',
	[masterCourse].[CourseTitle] AS 'CourseTitle',
	[masterCourse].[eCollegeID] AS 'eCollegeID',
	[masterCourse].[IsActive] AS 'IsActive',
	[masterCourse].[MasterCourseID] AS 'MasterCourseID',
	[masterCourse].[SourceNode] AS 'SourceNode'
	FROM [dbo].[MasterCourse] [masterCourse]
	WHERE [MasterCourseID]=@masterCourseID

	SET NOCOUNT OFF
END

GO

----------------------------------------------------------------
-- [dbo].[MasterCourseBannerCourse] Table
--
IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'InsertMasterCourseBannerCourse')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[InsertMasterCourseBannerCourse] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[InsertMasterCourseBannerCourse]
    @bannerCourseCode nvarchar(255),
	@masterCourseID int
AS
BEGIN
	SET NOCOUNT ON
	
    INSERT INTO [dbo].[MasterCourseBannerCourse] ([BannerCourseCode], [MasterCourseID])
	VALUES (@bannerCourseCode, @masterCourseID)
    
    SET NOCOUNT OFF
END    

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'DeleteMasterCourseBannerCourse')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[DeleteMasterCourseBannerCourse] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[DeleteMasterCourseBannerCourse]
	 @bannerCourseCode nvarchar(255),
	@masterCourseID int
AS
BEGIN
	SET NOCOUNT ON
	
    DELETE FROM [dbo].[MasterCourseBannerCourse]
	WHERE [BannerCourseCode]=@bannerCourseCode AND [MasterCourseID]=@masterCourseID
    
    SET NOCOUNT OFF
END

GO


