﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a MasterCourse
    /// and generating the corresponding SQL to insert that
    /// MasterCourse into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// MasterCourse object with it.
    /// </summary>
    internal class MasterCourseInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<MasterCourse>
    {
        /// <summary>
        /// Creates the MasterCourseInsertFactory to build an insert statement for
        /// the given MasterCourse object.
        /// </summary>
        /// <param name="MasterCourse">New MasterCourse to insert into the database.</param>
        public MasterCourseInsertFactory()
        {
        }

        #region IInsertFactory<MasterCourse> Members

        public DbCommand ConstructInsertCommand(Database db, MasterCourse masterCourse)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.InsertMasterCourse");

            if (masterCourse.CourseDisplayCode != null)
            {
                db.AddInParameter(command, "courseDisplayCode", DbType.String, masterCourse.CourseDisplayCode);
            }
            if (masterCourse.CourseTitle != null)
            {
                db.AddInParameter(command, "courseTitle", DbType.String, masterCourse.CourseTitle);
            }
            db.AddInParameter(command, "isActive", DbType.Boolean, masterCourse.IsActive);
            if (masterCourse.eCollegeID != null)
            {
                db.AddInParameter(command, "eCollegeID", DbType.String, masterCourse.eCollegeID);
            }
            //db.AddInParameter(command, "SourceNode", DbType.String, masterCourse.MasterCourseSourceNode.Node);
            db.AddOutParameter(command, "masterCourseID", DbType.Int32, 4);
            return command;
        }

        public void SetNewID(Database db, DbCommand command, MasterCourse masterCourse)
        {
            System.Int32 id1 = (System.Int32)(db.GetParameterValue(command, "masterCourseID"));
            masterCourse.MasterCourseID = id1;

        }

        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "courseDisplayCode":
                    return "CourseDisplayCode";
                case "courseTitle":
                    return "CourseTitle";
                case "isActive":
                    return "IsActive";
                case "eCollegeID":
                    return "eCollegeID";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }

}

