﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find MasterCourse in the
    /// CRM database.
    /// </summary>
    public class MasterCourseRepository : Repository<MasterCourse>
    {
        private string databaseName;

        public MasterCourseRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }


        public List<MasterCourse> GetAllFromMasterCourse()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromMasterCourseSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllFromMasterCourseFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<MasterCourse>();
        }

        public List<MasterCourse> GetAllActiveFromMasterCourse()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllActiveFromMasterCourseSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllActiveFromMasterCourseFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<MasterCourse>();
        }

        public MasterCourse GetMasterCourseByMasterCourseID(System.Int32 masterCourseID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetMasterCourseByMasterCourseIDSelectionFactory();

            try
            {
                return base.FindOne(selectionFactory, new GetMasterCourseByMasterCourseIDFactory(), masterCourseID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new MasterCourse();
        }

        public void Add(MasterCourse masterCourse)
        {
            MasterCourseInsertFactory insertFactory = new MasterCourseInsertFactory();
            try
            {
                base.Add(insertFactory, masterCourse);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }


        public void Save(MasterCourse masterCourse)
        {
            MasterCourseUpdateFactory updateFactory = new MasterCourseUpdateFactory();
            try
            {
                base.Save(updateFactory, masterCourse);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

