﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// This factory class takes a MasterCourse object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class MasterCourseUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<MasterCourse>
    {
        /// <summary>
        /// Creates the MasterCourseUpdateFactory to build an update statement for
        /// the given MasterCourse object.
        /// </summary>
        /// <param name="MasterCourse">MasterCourse to update into the database.</param>
        public MasterCourseUpdateFactory()
        {
        }

        #region IUpdateFactory<MasterCourse> Members

        public DbCommand ConstructUpdateCommand(Database db, MasterCourse masterCourse)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.UpdateMasterCourse");

            if (masterCourse.CourseDisplayCode != null)
            {
                db.AddInParameter(command, "courseDisplayCode", DbType.String, masterCourse.CourseDisplayCode);
            }
            if (masterCourse.CourseTitle != null)
            {
                db.AddInParameter(command, "courseTitle", DbType.String, masterCourse.CourseTitle);
            }
            db.AddInParameter(command, "isActive", DbType.Boolean, masterCourse.IsActive);
            db.AddInParameter(command, "masterCourseID", DbType.Int32, masterCourse.MasterCourseID);
            if (masterCourse.eCollegeID != null)
            {
                db.AddInParameter(command, "eCollegeID", DbType.String, masterCourse.eCollegeID);
            }

            //Add in source node update
            //db.AddInParameter(command, "sourceNode", DbType.String, masterCourse.MasterCourseSourceNode.Node);
            return command;
        }
        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "courseDisplayCode":
                    return "CourseDisplayCode";
                case "courseTitle":
                    return "CourseTitle";
                case "isActive":
                    return "IsActive";
                case "masterCourseID":
                    return "MasterCourseID";
                case "eCollegeID":
                    return "eCollegeID";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }
}

