﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find Section in the
    /// CRM database.
    /// </summary>
    public class SectionRepository : Repository<Section>
    {
        private string databaseName;

        public SectionRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }


        public List<Section> GetAllFromSection()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromSectionSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllFromSectionFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<Section>();
        }

        public List<Section> GetSectionByCourseCode(System.String courseCode)
        {
            ISelectionFactory<System.String> selectionFactory = new GetSectionByCourseCodeSelectionFactory();

            try
            {
                return base.Find(selectionFactory, new GetSectionByCourseCodeFactory(), courseCode);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<Section>();
        }

        public Section GetSectionBySectionID(System.String sectionID)
        {
            ISelectionFactory<System.String> selectionFactory = new GetSectionBySectionIDSelectionFactory();

            try
            {
                return base.FindOne(selectionFactory, new GetSectionBySectionIDFactory(), sectionID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new Section();
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

