﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find SourceNode in the
    /// CRM database.
    /// </summary>
    public class SourceNodeRepository : Repository<SourceNode>
    {
        private string databaseName;

        public SourceNodeRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }


        public List<SourceNode> GetAllFromSourceNode()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromSourceNodeSelectionFactory();

            try
            {
                NullableIdentity nullableIdentity = new NullableIdentity();
                return base.Find(selectionFactory, new GetAllFromSourceNodeFactory(), nullableIdentity);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new List<SourceNode>();
        }

        public SourceNode GetSourceNodeByNode(System.String node)
        {
            ISelectionFactory<System.String> selectionFactory = new GetSourceNodeByNodeSelectionFactory();

            try
            {
                return base.FindOne(selectionFactory, new GetSourceNodeByNodeFactory(), node);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }

            return new SourceNode();
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if (ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch (ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

