﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.CourseCopying.DataAccess.SQLServer;

namespace OHE.LCP.Admin.CourseCopying.DataAccess
{
    /// <summary>
    /// Respository that lets you find Term in the
    /// CRM database.
    /// </summary>
    public class TermRepository : Repository<Term>
    {
        private string databaseName;

        public TermRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

		public List<Term> GetAllFromTerm()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromTermSelectionFactory();
            
            try
            {
				NullableIdentity nullableIdentity = new NullableIdentity();
				return base.Find(selectionFactory, new GetAllFromTermFactory(), nullableIdentity);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<Term>();
        }

		public Term GetTermByTerm(System.String term)
        {
            ISelectionFactory<System.String> selectionFactory = new GetTermByTermSelectionFactory();
            
            try
            {
				return base.FindOne(selectionFactory, new GetTermByTermFactory(), term);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
			return new Term();
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

