﻿using System;
using System.Collections.Generic;
using System.Text;

namespace OHE.LCP.Admin.Messages.BusinessEntities
{
    /// <summary>
    /// The Message class is a business entity that represents an LCP message.
    /// </summary>
    [Serializable]
    public partial class Message
    {
        /// <summary>
        /// Parameterless constructor
        /// </summary>
        public Message()
        {
        }

        /// <summary>
        /// Constructor for the Message class.
        /// </summary>
        /// <param name="body">The body text of the message</param>
        /// <param name="endDateTime">The end date and time that the message should be shown</param>
        /// <param name="messageID">The unique ID for the message</param>
        /// <param name="priorityID">The ID of the priority for the message</param>
        /// <param name="startDateTime">The start date and time that the message should be shown</param>
        /// <param name="statusID">The ID of the status for the message</param>
        /// <param name="subject">The subject text of the message</param>
        /// <param name="from">The from text of the message</param>
        /// <param name="createdby">The user id of the user that created the message initially</param>
        /// <param name="lastupdatedby">The user id of the user that last updated the message</param>
        /// <param name="updateddatetime">The date/time that the message was updated</param>
        /// <param name="createddatetime">The date/time that the message was created</param>
        public Message(System.String body, System.DateTime endDateTime, System.Int32 messageID, System.Int32 priorityID, System.DateTime startDateTime, System.Int32 statusID, System.String subject, System.String from, System.String createdby, System.String lastupdatedby, System.DateTime updateddatetime, System.DateTime createddatetime)
        {
            this.bodyField = body;
            this.endDateTimeField = endDateTime;
            this.messageIDField = messageID;
            this.priorityIDField = priorityID;
            this.startDateTimeField = startDateTime;
            this.statusIDField = statusID;
            this.subjectField = subject;
            this.fromField = from;
            this.createdbyField = createdby;
            this.lastupdatedbyField = lastupdatedby;
            this.updateddatetimeField = updateddatetime;
            this.createdDateTimeField = createddatetime;
        }

        private System.DateTime createdDateTimeField;
        public System.DateTime CreatedDateTime
        {
            get { return this.createdDateTimeField; }
            set { this.createdDateTimeField = value; }
        }

        private System.String fromField;

        /// <summary>
        /// A text property that holds the name of the person or entity that the message is from.
        /// </summary>
        public System.String From
        {
            get { return this.fromField; }
            set { this.fromField = value; }
        }

        private System.String createdbyField;

        public System.String CreatedBy
        {
            get { return this.createdbyField; }
            set { this.createdbyField = value; }
        }

        private System.String lastupdatedbyField;

        public System.String LastUpdatedBy
        {
            get { return this.lastupdatedbyField; }
            set { this.lastupdatedbyField = value; }
        }

        private System.DateTime updateddatetimeField;

        public System.DateTime UpdatedDateTime
        {
            get { return this.updateddatetimeField; }
            set { this.updateddatetimeField = value; }
        }

        private System.String bodyField;

        /// <summary>
        /// The body text of the message.
        /// </summary>
        public System.String Body
        {
            get { return this.bodyField; }
            set { this.bodyField = value; }
        }

        private System.DateTime endDateTimeField;

        /// <summary>
        /// The ending date that the message should be displayed.  The LCP should use the StartDateTime and EndDateTime fields to figure out when to display the message.
        /// </summary>
        public System.DateTime EndDateTime
        {
            get { return this.endDateTimeField; }
            set { this.endDateTimeField = value; }
        }

        private System.Int32 messageIDField;

        /// <summary>
        /// The internal ID of the message in the LCP system.
        /// </summary>
        public System.Int32 MessageID
        {
            get { return this.messageIDField; }
            set { this.messageIDField = value; }
        }

        private System.Int32 priorityIDField;

        /// <summary>
        /// The priority of the message.  Acceptable values are 1 (Low), 2 (Medium), 3 (High)
        /// </summary>
        public System.Int32 PriorityID
        {
            get { return this.priorityIDField; }
            set { this.priorityIDField = value; }
        }

        private System.DateTime startDateTimeField;

        /// <summary>
        /// The starting date that the message should be displayed.  The LCP should use the StartDateTime and EndDateTime fields to figure out when to display the message.
        /// </summary>
        public System.DateTime StartDateTime
        {
            get { return this.startDateTimeField; }
            set { this.startDateTimeField = value; }
        }

        private System.Int32 statusIDField;

        /// <summary>
        /// The status of the message.  Acceptable values are 1 (Unpublished), 2 (Published)
        /// </summary>
        public System.Int32 StatusID
        {
            get { return this.statusIDField; }
            set { this.statusIDField = value; }
        }

        private System.String subjectField;

        /// <summary>
        /// The subject of the message.
        /// </summary>
        public System.String Subject
        {
            get { return this.subjectField; }
            set { this.subjectField = value; }
        }

    }
}

