using System;
using System.Collections.Generic;
using System.Text;
using OHE.LCP.Admin.Messages;

namespace OHE.LCP.Admin.Messages.BusinessEntities
{
    /// <summary>
    /// The priority of the message.
    /// </summary>
    public enum MessagePriority
    {
        Normal = 1,
        High = 2
    }

    /// <summary>
    /// The status of the message.
    /// </summary>
    public enum MessageStatus
    {
        Unpublished = 1,
        Published = 2
    }

    /// <summary>
    /// The type of the target for a message
    /// </summary>
    public enum MessageTargetType
    {
        Role = 1,
        School = 2,
        Program = 3,
        Concentration = 4,
        Course = 5,
        Section = 6,
        Person = 7
    }

    public partial class Message
    {
        private List<MessageTarget> _messageTargets;
        private List<MessageRead> _messageReads;

        /// <summary>
        /// The list of message targets for the message.
        /// </summary>
        public List<MessageTarget> MessageTargets
        {
            get { return _messageTargets; }
            set { _messageTargets = value; }
        }

        /// <summary>
        /// The list of message reads for the message.
        /// </summary>
        public List<MessageRead> MessageReads
        {
            get { return _messageReads; }
            set { _messageReads = value; }
        }

        /// <summary>
        /// A read only property that returns a text version of the status.
        /// </summary>
        public string StatusText
        {
            get { return Enum.GetName(typeof(MessageStatus), this.StatusID); }
            set { string _test = value; }
        }

        /// <summary>
        /// A read only property that returns a text version of the priority.
        /// </summary>
        public string PriorityText
        {
            get { return Enum.GetName(typeof(MessagePriority), this.PriorityID); }
            set { string _test = value; }
        }

        /// <summary>
        /// A read only property that returns a text description of the recipients for this message.  This property analyzes all of the current targets for the message and creates a text description of the recipients.  This is used as a preview for the admin user so that they can check to make sure that the message will be delivered to the proper recipients.
        /// </summary>
        public string RecipientsDescription
        {
            get
            {
                string _recipients = null;

                _recipients = "This message will be sent to people who meet the following criteria: <br />";

                string _roles = "";
                string _schools = "";
                string _programs = "";
                string _concentrations = "";
                string _courses = "";
                string _sections = "";
                string _people = "";

                //Iterate through all of the message targets and build the strings for the various target types
                foreach (MessageTarget _messageTarget in this.MessageTargets)
                {
                    switch (_messageTarget.TargetTypeID)
                    {
                        case 1:
                            //Role
                            if (_roles == "")
                            { _roles = _messageTarget.Description; }
                            else { _roles = _roles + " or " + _messageTarget.Description; }
                            break;
                        case 2:
                            //School
                            if (_schools == "")
                            { _schools = "&nbsp;&nbsp;" + _messageTarget.Description; }
                            else { _schools = _schools + " or <br />&nbsp;&nbsp;" + _messageTarget.Description; }
                            break;
                        case 3:
                            //Program
                            if (_programs == "")
                            { _programs = "&nbsp;&nbsp;" + _messageTarget.Description + " program"; }
                            else { _programs = _programs + " or <br />&nbsp;&nbsp;" + _messageTarget.Description + " program"; }
                            break;
                        case 4:
                            //Concentration
                            if (_concentrations == "")
                            { _concentrations = "&nbsp;&nbsp;" + _messageTarget.Description + " concentration"; }
                            else { _concentrations = _concentrations + " or <br />&nbsp;&nbsp;" + _messageTarget.Description + " concentration"; }
                            break;
                        case 5:
                            //Course
                            if (_courses == "")
                            { _courses = "&nbsp;&nbsp;" + _messageTarget.Description + " course"; }
                            else { _courses = _courses + " or <br />&nbsp;&nbsp;" + _messageTarget.Description + " course"; }
                            break;
                        case 6:
                            //Section
                            if (_sections == "")
                            { _sections = "&nbsp;&nbsp;" + _messageTarget.Description + " section"; }
                            else { _sections = _sections + " or <br />&nbsp;&nbsp;" + _messageTarget.Description + " section"; }
                            break;
                        case 7:
                            //People
                            if (_people == "")
                            { _people = _messageTarget.Description; }
                            else { _people = _people + " or " + _messageTarget.Description; }
                            break;
                    }
                }

                //First check and see if any roles were specified
                if (_roles == "")
                //If no roles were specified then it could be anyone
                { _recipients = _recipients + "anyone"; }
                else { _recipients = _recipients + _roles; }

                //This string will hold the text for the schools, programs and concentrations
                string school_prog_conc = "";

                //If the schools string is not empty then add it to the string
                if (_schools != "")
                {
                    school_prog_conc = _schools;
                }

                //If the programs string is not empty then add it to the string
                if (_programs != "")
                {
                    //If there were some schools specified then use an OR to add the programs
                    if (school_prog_conc == "")
                    { school_prog_conc = _programs; }
                    else { school_prog_conc = school_prog_conc + " or<br />" + _programs; }
                }

                //If the concentrations string is not empty then add it to the string
                if (_concentrations != "")
                {
                    //If there were schools or programs specified then use an OR to add the concentrations
                    if (school_prog_conc == "")
                    { school_prog_conc = _concentrations; }
                    else { school_prog_conc = school_prog_conc + " or<br />" + _concentrations; }
                }

                //If the school/program/concentration string is not empty then add it to the recipients string
                if (school_prog_conc != "")
                {
                    _recipients = _recipients + " in<br />" + school_prog_conc;
                }

                //This string will be used for courses and sections
                string course_sect = "";

                //If the courses string is not empty then add it to the string
                if (_courses != "")
                {
                    course_sect = _courses;
                }

                //If the sections string is not empty then add it to the string
                if (_sections != "")
                {
                    //If there were courses specified then use an OR to add the sections
                    if (course_sect == "")
                    { course_sect = _sections; }
                    else { course_sect = course_sect + " or<br />" + _sections; }
                }

                //If the courses/section string is not empty then add it to the recipients string
                if (course_sect != "")
                {
                    //If the school/prog/conc string is not empty then use an AND to add the courses/sections
                    if (school_prog_conc == "")
                    { _recipients = _recipients + " in<br />" + course_sect; }
                    else { _recipients = _recipients + "<br />and in<br />" + course_sect; }
                }

                //If the people string is not empty then add it to the recipients string
                if (_people != "")
                {
                    _recipients = _recipients + "<br />who is " + _people;
                }

                return _recipients;
            }
        }
    }
}
