﻿using System;
using System.Collections.Generic;
using System.Text;

namespace OHE.LCP.Admin.Messages.BusinessEntities
{
    /// <summary>
    /// The MessageTarget class represents a particular target for a message.  Targets are used to allow admin users to send messages to specific sets of LCP users.
    /// </summary>
    [Serializable]
    public partial class MessageTarget
    {
        /// <summary>
        /// Parameterless constructor.
        /// </summary>
        public MessageTarget()
        {
        }
        
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="messageID">The ID of the message for this target</param>
        /// <param name="messageTargetID">The unique ID for the message target</param>
        /// <param name="targetID">The foreign ID for this target.  This is the unique identifier for the particular target (School of Education, etc.)</param>
        /// <param name="targetTypeID">The type of target (Role, Student, School, etc.)</param>
        /// <param name="description">A description of the target.  This is simply used to make the database more readable.</param>
        public MessageTarget(System.Int32 messageID, System.Int32 messageTargetID, System.String targetID, System.Int32 targetTypeID, System.String description)
        {
            this.messageIDField = messageID;
            this.messageTargetIDField = messageTargetID;
            this.targetIDField = targetID;
            this.targetTypeIDField = targetTypeID;
            this.descriptionField = description;
        }
		
		private System.Int32 messageIDField;		

		/// <summary>
		/// The ID of the message for this target.
		/// </summary>
	    public System.Int32 MessageID
	    {
	        get { return this.messageIDField; }
	        set { this.messageIDField = value; }
	    }

		private System.Int32 messageTargetIDField;		
		
        /// <summary>
        /// The internal, system assigned ID for this MessageTarget object.
        /// </summary>
	    public System.Int32 MessageTargetID
	    {
	        get { return this.messageTargetIDField; }
            set { this.messageTargetIDField = value; }
	    }

		private System.String targetIDField;		
		
        /// <summary>
        /// The ID for the target.  Since targets can be of many different types, this field holds the ID for the specific target.  For example, a message target may be for a school.  This field would then hold the ID for the school.
        /// </summary>
	    public System.String TargetID
	    {
	        get { return this.targetIDField; }
	        set { this.targetIDField = value; }
	    }

		private System.Int32 targetTypeIDField;		

		/// <summary>
		/// The ID of the target type.  There a number of different types for targets.  Acceptable values are as follows:
        /// <list type="bullet">
        /// <item><description>1 = Role</description></item>
        /// <item><description>2 = School</description></item>
        /// <item><description>3 = Program</description></item>
        /// <item><description>4 = Concentration</description></item>
        /// <item><description>5 = Course</description></item>
        /// <item><description>6 = Section</description></item>
        /// <item><description>7 = Person</description></item>
        /// </list>
		/// </summary>
	    public System.Int32 TargetTypeID
	    {
	        get { return this.targetTypeIDField; }
	        set { this.targetTypeIDField = value; }
	    }

        private System.String descriptionField;

        /// <summary>
        /// A description of the Message Target.  This is populated with the target type text.
        /// </summary>
        public System.String Description
        {
            get { return this.descriptionField; }
            set { this.descriptionField = value; }
        }
    }
}

