﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// This class is responsible for turning a selection criteria
    /// into a select statement and database query parameters.
    /// </summary>
    internal class GetMessagesForLCPSelectionFactory : ISelectionFactory<GetMessagesForLCPIdentity>
    {
        /// <summary>
        /// Creates the GetMessagesForLCP_TempSelectionFactory to build a select statement for
        /// the given GetMessagesForLCPIdentity.
        /// </summary>
        /// <param name="GetMessagesForLCPIdentity">GetMessagesForLCPIdentity to select from the database.</param>      
        public GetMessagesForLCPSelectionFactory()
        {
        }

        #region ISelectionFactory<GetMessagesForLCPIdentity> Members
        public DbCommand ConstructSelectCommand(Database db, GetMessagesForLCPIdentity getMessagesForLCPIdentity)
        {

            DbCommand command = db.GetStoredProcCommand("dbo.GetMessagesForLCP");
            db.AddInParameter(command, "Concentrations", DbType.String, getMessagesForLCPIdentity.Concentrations);
            db.AddInParameter(command, "Courses", DbType.String, getMessagesForLCPIdentity.Courses);
            db.AddInParameter(command, "PersonID", DbType.String, getMessagesForLCPIdentity.PersonID);
            db.AddInParameter(command, "Programs", DbType.String, getMessagesForLCPIdentity.Programs);
            db.AddInParameter(command, "Roles", DbType.String, getMessagesForLCPIdentity.Roles);
            db.AddInParameter(command, "Schools", DbType.String, getMessagesForLCPIdentity.Schools);
            db.AddInParameter(command, "Sections", DbType.String, getMessagesForLCPIdentity.Sections);
            return command;
        }
        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "Concentrations":
                    return "Concentrations";
                case "Courses":
                    return "Courses";
                case "PersonID":
                    return "PersonID";
                case "Programs":
                    return "Programs";
                case "Roles":
                    return "Roles";
                case "Schools":
                    return "Schools";
                case "Sections":
                    return "Sections";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }
}

