﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a Message
    /// and generating the corresponding SQL to insert that
    /// Message into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// Message object with it.
    /// </summary>
    internal class MessageInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<Message>
    {
        /// <summary>
        /// Creates the MessageInsertFactory to build an insert statement for
        /// the given Message object.
        /// </summary>
        /// <param name="Message">New Message to insert into the database.</param>
        public MessageInsertFactory()
        {
        }

		#region IInsertFactory<Message> Members
		
        public DbCommand ConstructInsertCommand(Database db, Message message)
        {            
			DbCommand command = db.GetStoredProcCommand("dbo.InsertMessage");
			
			if(message.Body != null)
			{
				db.AddInParameter(command, "body", DbType.String, message.Body);
			}
            if (message.From != null)
            {
                db.AddInParameter(command, "from", DbType.String, message.From);
            }
			if(message.EndDateTime != null)
			{
				db.AddInParameter(command, "endDateTime", DbType.DateTime, message.EndDateTime);
			}
			db.AddOutParameter(command, "messageID", DbType.Int32, 4);
				db.AddInParameter(command, "priorityID", DbType.Int32, message.PriorityID);
			if(message.StartDateTime != null)
			{
				db.AddInParameter(command, "startDateTime", DbType.DateTime, message.StartDateTime);
			}
				db.AddInParameter(command, "statusID", DbType.Int32, message.StatusID);
			if(message.Subject != null)
			{
				db.AddInParameter(command, "subject", DbType.String, message.Subject);
			}
            if (message.CreatedBy != null)
            {
                db.AddInParameter(command, "createdby", DbType.String, message.CreatedBy);
            }
			return command;
        }

        public void SetNewID(Database db, DbCommand command, Message message)
        {
            System.Int32 id1 = (System.Int32)(db.GetParameterValue(command, "messageID"));
	message.MessageID = id1;
	
        }
        
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "adminID":
					return "AdminID";
				case "body":
					return "Body";
				case "endDateTime":
					return "EndDateTime";
				case "externalMessageID":
					return "ExternalMessageID";
				case "priorityID":
					return "PriorityID";
				case "startDateTime":
					return "StartDateTime";
				case "statusID":
					return "StatusID";
				case "subject":
					return "Subject";
				case "systemID":
					return "SystemID";
                case "from":
                    return "From";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }

}

