﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// A class that converts a MessageRead
    /// into a DbCommand that will delete
    /// that MessageRead.
    /// </summary>
    internal class MessageReadDeleteFactory : IDeleteFactory<System.Int32>
    {
        /// <summary>
        /// Creates the MessageReadDeleteFactory to build a delete statement for
        /// the given System.Int32.
        /// </summary>
        /// <param name="System.Int32">System.Int32 to delete from the database.</param>    
        public MessageReadDeleteFactory()
        {
        }
       
        #region IDeleteFactory<System.Int32> Members
        public DbCommand ConstructDeleteCommand(Database db, System.Int32 messageReadID)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.DeleteMessageRead");
			db.AddInParameter(command, "messageReadID", DbType.Int32, messageReadID);
			return command;
        }
        #endregion
        
        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "messageReadID":
					return "messageReadID";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }
}

