﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a MessageRead
    /// and generating the corresponding SQL to insert that
    /// MessageRead into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// MessageRead object with it.
    /// </summary>
    internal class MessageReadInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<MessageRead>
    {
        /// <summary>
        /// Creates the MessageReadInsertFactory to build an insert statement for
        /// the given MessageRead object.
        /// </summary>
        /// <param name="MessageRead">New MessageRead to insert into the database.</param>
        public MessageReadInsertFactory()
        {
        }

		#region IInsertFactory<MessageRead> Members
		
        public DbCommand ConstructInsertCommand(Database db, MessageRead messageRead)
        {            
			DbCommand command = db.GetStoredProcCommand("dbo.InsertMessageRead");
			
			if(messageRead.LCPUserID != null)
			{
				db.AddInParameter(command, "lCPUserID", DbType.String, messageRead.LCPUserID);
			}
				db.AddInParameter(command, "messageID", DbType.Int32, messageRead.MessageID);
				db.AddInParameter(command, "readDateTime", DbType.DateTime, messageRead.ReadDateTime);
			db.AddOutParameter(command, "messageReadID", DbType.Int32, 4);
			return command;
        }

        public void SetNewID(Database db, DbCommand command, MessageRead messageRead)
        {
            System.Int32 id1 = (System.Int32)(db.GetParameterValue(command, "messageReadID"));
	messageRead.MessageReadID = id1;
	
        }
        
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "lCPUserID":
					return "LCPUserID";
				case "messageID":
					return "MessageID";
				case "readDateTime":
					return "ReadDateTime";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }

}

