﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.Messages.DataAccess.SQLServer;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// Respository that lets you find MessageRead in the
    /// CRM database.
    /// </summary>
    public class MessageReadRepository : Repository<MessageRead>
    {
        private string databaseName;

        public MessageReadRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

		public List<MessageRead> GetAllFromMessageRead()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromMessageReadSelectionFactory();
            
            try
            {
				NullableIdentity nullableIdentity = new NullableIdentity();
				return base.Find(selectionFactory, new GetAllFromMessageReadFactory(), nullableIdentity);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<MessageRead>();
        }

        public List<MessageRead> GetMessageReadByMessageID(System.Int32 messageID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetMessageReadByMessageIDSelectionFactory();
            
            try
            {
				return base.Find(selectionFactory, new GetMessageReadByMessageIDFactory(), messageID);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<MessageRead>();
        }

        public void Add(MessageRead messageRead)
        {
            MessageReadInsertFactory insertFactory = new MessageReadInsertFactory();
            try
            {
                base.Add(insertFactory, messageRead);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }

        public void Remove(List<MessageRead> _messageReads)
        {
            foreach (MessageRead _messageRead in _messageReads)
            {
                this.Remove(_messageRead.MessageReadID);
            }
        }

        public void Remove(System.Int32 messageReadID)
        {
			IDeleteFactory<System.Int32> deleteFactory = new MessageReadDeleteFactory();

            try
            {
				base.Remove(deleteFactory, messageReadID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, deleteFactory);
            }            
        }
      

		public void Save(MessageRead messageRead)
        {
            MessageReadUpdateFactory updateFactory = new MessageReadUpdateFactory();
            try
            {
                base.Save(updateFactory, messageRead);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

