﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// This factory class takes a MessageRead object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class MessageReadUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<MessageRead>
    {
        /// <summary>
        /// Creates the MessageReadUpdateFactory to build an update statement for
        /// the given MessageRead object.
        /// </summary>
        /// <param name="MessageRead">MessageRead to update into the database.</param>
        public MessageReadUpdateFactory()
        {
        }

        #region IUpdateFactory<MessageRead> Members

        public DbCommand ConstructUpdateCommand(Database db, MessageRead messageRead)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.UpdateMessageRead");

            db.AddInParameter(command, "readDateTime", DbType.DateTime, messageRead.ReadDateTime);
            db.AddInParameter(command, "messageReadID", DbType.Int32, messageRead.MessageReadID);
            db.AddInParameter(command, "messageID", DbType.Int32, messageRead.MessageID);
            if (messageRead.LCPUserID != null)
            {
                db.AddInParameter(command, "lCPUserID", DbType.String, messageRead.LCPUserID);
            }
            return command;
        }
        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "readDateTime":
                    return "ReadDateTime";
                case "messageReadID":
                    return "MessageReadID";
                case "messageID":
                    return "MessageID";
                case "lCPUserID":
                    return "LCPUserID";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }
}

