﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.Messages.DataAccess.SQLServer;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// Respository that lets you find Message in the
    /// CRM database.
    /// </summary>
    public class MessageRepository : Repository<Message>
    {
        private string databaseName;

        public MessageRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

		public List<Message> GetAllFromMessage()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetAllFromMessageSelectionFactory();
            
            try
            {
				NullableIdentity nullableIdentity = new NullableIdentity();
				return base.Find(selectionFactory, new GetAllFromMessageFactory(), nullableIdentity);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<Message>();
        }

		public Message GetMessageByMessageID(System.Int32 messageID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetMessageByMessageIDSelectionFactory();
            
            try
            {
				return base.FindOne(selectionFactory, new GetMessageByMessageIDFactory(), messageID);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
			return new Message();
        }

        public List<Message> GetMessagesForLCP(System.String concentrations, System.String courses, System.String personID, System.String programs, System.String roles, System.String schools, System.String sections)
        {
            ISelectionFactory<GetMessagesForLCPIdentity> selectionFactory = new GetMessagesForLCPSelectionFactory();
            
            try
            {

				GetMessagesForLCPIdentity getMessagesForLCPIdentity = new GetMessagesForLCPIdentity(concentrations, courses, personID, programs, roles, schools, sections);
				return base.Find(selectionFactory, new GetMessagesForLCPFactory(), getMessagesForLCPIdentity);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<Message>();
        }

        public void Add(Message message)
        {
            MessageInsertFactory insertFactory = new MessageInsertFactory();
            try
            {
                base.Add(insertFactory, message);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }

        public void Remove(System.Int32 messageID)
        {
			IDeleteFactory<System.Int32> deleteFactory = new MessageDeleteFactory();

            try
            {
				base.Remove(deleteFactory, messageID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, deleteFactory);
            }            
        }
      

		public void Save(Message message)
        {
            MessageUpdateFactory updateFactory = new MessageUpdateFactory();
            try
            {
                base.Save(updateFactory, message);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

