﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a MessageTarget
    /// and generating the corresponding SQL to insert that
    /// MessageTarget into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// MessageTarget object with it.
    /// </summary>
    internal class MessageTargetInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<MessageTarget>
    {
        /// <summary>
        /// Creates the MessageTargetInsertFactory to build an insert statement for
        /// the given MessageTarget object.
        /// </summary>
        /// <param name="MessageTarget">New MessageTarget to insert into the database.</param>
        public MessageTargetInsertFactory()
        {
        }

		#region IInsertFactory<MessageTarget> Members
		
        public DbCommand ConstructInsertCommand(Database db, MessageTarget messageTarget)
        {            
			DbCommand command = db.GetStoredProcCommand("dbo.InsertMessageTarget");
			
			db.AddInParameter(command, "messageID", DbType.Int32, messageTarget.MessageID);
			db.AddOutParameter(command, "messageTargetID", DbType.Int32, 4);
			if(messageTarget.TargetID != null)
			{
				db.AddInParameter(command, "targetID", DbType.String, messageTarget.TargetID);
			}
			db.AddInParameter(command, "targetTypeID", DbType.Int32, messageTarget.TargetTypeID);
            if (messageTarget.Description != null)
            {
                db.AddInParameter(command, "description", DbType.String, messageTarget.Description);
            }
			return command;
        }

        public void SetNewID(Database db, DbCommand command, MessageTarget messageTarget)
        {
            System.Int32 id1 = (System.Int32)(db.GetParameterValue(command, "messageTargetID"));
	messageTarget.MessageTargetID = id1;
	
        }
        
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "messageID":
					return "MessageID";
				case "targetID":
					return "TargetID";
				case "targetTypeID":
					return "TargetTypeID";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }

}

