﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using OHE.LCP.Admin.Messages.DataAccess.SQLServer;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// Respository that lets you find MessageTarget in the
    /// CRM database.
    /// </summary>
    public class MessageTargetRepository : Repository<MessageTarget>
    {
        private string databaseName;

        public MessageTargetRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

        public List<MessageTarget> GetMessageTargetByMessageID(System.Int32 messageID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetMessageTargetByMessageIDSelectionFactory();
            
            try
            {
				return base.Find(selectionFactory, new GetMessageTargetByMessageIDFactory(), messageID);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<MessageTarget>();
        }

        public void Add(MessageTarget messageTarget)
        {
            MessageTargetInsertFactory insertFactory = new MessageTargetInsertFactory();
            try
            {
                base.Add(insertFactory, messageTarget);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }

        public void Remove(System.Int32 messageTargetID)
        {
			IDeleteFactory<System.Int32> deleteFactory = new MessageTargetDeleteFactory();

            try
            {
				base.Remove(deleteFactory, messageTargetID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, deleteFactory);
            }            
        }

        public void Remove(List<MessageTarget> _messageTargets)
        {
            foreach (MessageTarget _target in _messageTargets)
            {
                this.Remove(_target.MessageTargetID);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

