﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// This factory class takes a MessageTarget object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class MessageTargetUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<MessageTarget>
    {
        /// <summary>
        /// Creates the MessageTargetUpdateFactory to build an update statement for
        /// the given MessageTarget object.
        /// </summary>
        /// <param name="MessageTarget">MessageTarget to update into the database.</param>
        public MessageTargetUpdateFactory()
        {
        }
        
        #region IUpdateFactory<MessageTarget> Members

        public DbCommand ConstructUpdateCommand(Database db, MessageTarget messageTarget)
        {
			DbCommand command = db.GetStoredProcCommand("dbo.UpdateMessageTarget");
			
			db.AddInParameter(command, "messageID", DbType.Int32, messageTarget.MessageID);
			db.AddInParameter(command, "messageTargetID", DbType.Int32, messageTarget.MessageTargetID);
			if(messageTarget.TargetID != null)
			{
				db.AddInParameter(command, "targetID", DbType.String, messageTarget.TargetID);
			}
			db.AddInParameter(command, "targetTypeID", DbType.Int32, messageTarget.TargetTypeID);
            if (messageTarget.Description != null)
            {
                db.AddInParameter(command, "description", DbType.String, messageTarget.Description);
            }
			return command;
        }
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "messageID":
					return "MessageID";
				case "messageTargetID":
					return "MessageTargetID";
				case "targetID":
					return "TargetID";
				case "targetTypeID":
					return "TargetTypeID";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }
}

