﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using OHE.LCP.Admin.Messages.BusinessEntities;
using OHE.LCP.Admin.Messages.DataAccess.Generic;
using System.Globalization;

namespace OHE.LCP.Admin.Messages.DataAccess
{
    /// <summary>
    /// This factory class takes a Message object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class MessageUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<Message>
    {
        /// <summary>
        /// Creates the MessageUpdateFactory to build an update statement for
        /// the given Message object.
        /// </summary>
        /// <param name="Message">Message to update into the database.</param>
        public MessageUpdateFactory()
        {
        }
        
        #region IUpdateFactory<Message> Members

        public DbCommand ConstructUpdateCommand(Database db, Message message)
        {
			DbCommand command = db.GetStoredProcCommand("dbo.UpdateMessage");
			
			if(message.Body != null)
			{
				db.AddInParameter(command, "body", DbType.String, message.Body);
			}
            if (message.From != null)
            {
                db.AddInParameter(command, "from", DbType.String, message.From);
            }
			if(message.EndDateTime != null)
			{
				db.AddInParameter(command, "endDateTime", DbType.DateTime, message.EndDateTime);
			}
			db.AddInParameter(command, "messageID", DbType.Int32, message.MessageID);
			db.AddInParameter(command, "priorityID", DbType.Int32, message.PriorityID);
			if(message.StartDateTime != null)
			{
				db.AddInParameter(command, "startDateTime", DbType.DateTime, message.StartDateTime);
			}
			db.AddInParameter(command, "statusID", DbType.Int32, message.StatusID);
			if(message.Subject != null)
			{
				db.AddInParameter(command, "subject", DbType.String, message.Subject);
			}
            if (message.LastUpdatedBy != null)
            {
                db.AddInParameter(command, "lastUpdatedBy", DbType.String, message.LastUpdatedBy);
            }
			return command;
        }
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "adminID":
					return "AdminID";
				case "body":
					return "Body";
				case "endDateTime":
					return "EndDateTime";
				case "externalMessageID":
					return "ExternalMessageID";
				case "messageID":
					return "MessageID";
				case "priorityID":
					return "PriorityID";
				case "startDateTime":
					return "StartDateTime";
				case "statusID":
					return "StatusID";
				case "subject":
					return "Subject";
				case "systemID":
					return "SystemID";
                case "from":
                    return "From";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }
}

