﻿
/*
IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'RethrowError')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].RethrowError AS RETURN')
END
GO

ALTER PROCEDURE RethrowError AS
    -- Return if there is no error information to retrieve. 
    IF ERROR_NUMBER() IS NULL
        RETURN;

    DECLARE
        @ErrorMessage    NVARCHAR(4000),
        @ErrorNumber     INT,
        @ErrorSeverity   INT,
        @ErrorState      INT,
        @ErrorLine       INT,
        @ErrorProcedure  NVARCHAR(200); 

    -- Assign variables to error-handling functions that
      -- capture information for RAISERROR. 

    SELECT
        @ErrorNumber = ERROR_NUMBER(),
        @ErrorSeverity = ERROR_SEVERITY(),
        @ErrorState = ERROR_STATE(),
        @ErrorLine = ERROR_LINE(),
        @ErrorProcedure = ISNULL(ERROR_PROCEDURE(), '-'); 

    -- Building the message string that will contain original
    --   error information. 

    SELECT @ErrorMessage = 
        N'Error %d, Level %d, State %d, Procedure %s, Line %d, ' + 
         'Message: '+ ERROR_MESSAGE(); 

    -- Raise an error: msg_str parameter of RAISERROR will contain
	  -- the original error information. 

    RAISERROR(@ErrorMessage, @ErrorSeverity, 1,
        @ErrorNumber,    -- parameter: original error number. 
        @ErrorSeverity,  -- parameter: original error severity. 
        @ErrorState,     -- parameter: original error state. 
        @ErrorProcedure, -- parameter: original error procedure name. 
        @ErrorLine       -- parameter: original error line number. 
        );

GO
*/
----------------------------------------------------------------
-- [dbo].[MessageTarget] Table
--
IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'InsertMessageTarget')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[InsertMessageTarget] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[InsertMessageTarget]
    @messageID int,
	@messageTargetID int OUT,
	@targetID nvarchar(50),
	@targetTypeID int
AS
BEGIN
	SET NOCOUNT ON
	
	--BEGIN TRY
    INSERT INTO [dbo].[MessageTarget] ([MessageID], [TargetID], [TargetTypeID])
	VALUES (@messageID, @targetID, @targetTypeID)
    SET @messageTargetID = SCOPE_IDENTITY()
    --END TRY

    --BEGIN CATCH
	--	EXEC RethrowError;
	--END CATCH
    
    SET NOCOUNT OFF
END    

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'UpdateMessageTarget')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[UpdateMessageTarget] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[UpdateMessageTarget]
    @messageID int,
	@messageTargetID int,
	@targetID nvarchar(50),
	@targetTypeID int
AS
BEGIN

	--The [dbo].[MessageTarget] table doesn't have a timestamp column. Optimistic concurrency logic cannot be generated
	SET NOCOUNT ON

	--BEGIN TRY
	UPDATE [dbo].[MessageTarget] 
	SET [MessageID] = @messageID, [TargetID] = @targetID, [TargetTypeID] = @targetTypeID
	WHERE [MessageTargetID]=@messageTargetID

	IF @@ROWCOUNT = 0
	BEGIN
		RAISERROR('Concurrent update error. Updated aborted.', 16, 2)
	END
    --END TRY

    --BEGIN CATCH
	--	EXEC RethrowError;
	--END CATCH	

	SET NOCOUNT OFF
END

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'DeleteMessageTarget')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[DeleteMessageTarget] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[DeleteMessageTarget]
	 @messageTargetID int
AS
BEGIN
	SET NOCOUNT ON
	
    DELETE FROM [dbo].[MessageTarget]
	WHERE [MessageTargetID]=@messageTargetID
    
    SET NOCOUNT OFF
END

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'GetAllFromMessageTarget')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[GetAllFromMessageTarget] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[GetAllFromMessageTarget]    
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT
	[messageTarget].[MessageID] AS 'MessageID',
	[messageTarget].[MessageTargetID] AS 'MessageTargetID',
	[messageTarget].[TargetID] AS 'TargetID',
	[messageTarget].[TargetTypeID] AS 'TargetTypeID'
FROM [dbo].[MessageTarget] [messageTarget]

	SET NOCOUNT OFF
END

GO

IF NOT EXISTS (SELECT NAME FROM sys.objects WHERE TYPE = 'P' AND NAME = 'GetMessageTargetByMessageTargetID')
BEGIN
	EXEC('CREATE PROCEDURE [dbo].[GetMessageTargetByMessageTargetID] AS RETURN')
END

GO

ALTER PROCEDURE [dbo].[GetMessageTargetByMessageTargetID] 
	@messageTargetID int
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT
	[messageTarget].[MessageID] AS 'MessageID',
	[messageTarget].[MessageTargetID] AS 'MessageTargetID',
	[messageTarget].[TargetID] AS 'TargetID',
	[messageTarget].[TargetTypeID] AS 'TargetTypeID'
	FROM [dbo].[MessageTarget] [messageTarget]
	WHERE [MessageTargetID]=@messageTargetID

	SET NOCOUNT OFF
END

GO


