﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Microsoft.Practices.ObjectBuilder;
using OHE.LCP.Admin.CourseCopying.Views;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using System.Collections.Generic;
using OHE.LCP.Admin.CourseCopying.Resources;
using OHE.LCP.Admin.CourseCopying;
using Telerik.Web.UI;

public partial class CourseCopying_CourseCopy : System.Web.UI.Page, ICourseCopy
{
    private CourseCopyPresenter _presenter;

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            this._presenter.OnViewInitialized();
        }

        this._presenter.OnViewLoaded();
        //this should only get set to true on the copy button click event
        this.cvCourseGrid.Enabled = false;
    }

    [CreateNew]
    public CourseCopyPresenter Presenter
    {
        set
        {
            this._presenter = value;
            this._presenter.View = this;
        }
    }

    public IList<Course> Courses
    {
        set
        {
            Session[Constants.SessionKeys.Courses] = value;
            CoursesDataSource.DataSource = value;
        }
    }

    public IList<Term> Terms
    {
        set { TermsDataSource.DataSource = value; }
    }

    public IList<SubTerm> SubTerms
    {
        set { SubTermsDataSource.DataSource = value; }
    }

    public IList<ECollegeTerm> ECollegeTerms
    {
        set { ECollegeTermsDataSource.DataSource = value; }
    }

    public IList<School> Schools
    {
        set { SchoolsDataSource.DataSource = value; }
    }

    public IList<CopyHistory> History
    {
        set { HistoryDataSource.DataSource = value; }
    }

    protected void btnUpdate_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            _presenter.OnUpdateClick(ddlTerm.SelectedValue, ddlSubTerm.SelectedValue, ddlSchool.SelectedValue);
        }
    }
    protected void ddlTerm_SelectedIndexChanged(object sender, EventArgs e)
    {
        _presenter.OnTermSelected(ddlTerm.SelectedValue);
    }
    protected void ddlTerm_DataBound(object sender, EventArgs e)
    {
        //Add a Select One item
        ddlTerm.Items.Insert(0, new ListItem("Select One", ""));
        if (Session[Constants.SessionKeys.SelectedTerm] != null)
        {
            //Set the selected value if necessary
            ddlTerm.SelectedValue = Session[Constants.SessionKeys.SelectedTerm].ToString();
        }
    }
    protected void ddlSubTerm_DataBound(object sender, EventArgs e)
    {
        //Ad a Select On item
        ddlSubTerm.Items.Insert(0, new ListItem("Select One", ""));
        if (Session[Constants.SessionKeys.SelectedSubTerm] != null)
        {
            //Set the selected value if necessary
            ddlSubTerm.SelectedValue = Session[Constants.SessionKeys.SelectedSubTerm].ToString();
        }
    }

    protected void CoursesGrid_ItemDataBound(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {

        if (e.Item.ItemType == Telerik.Web.UI.GridItemType.Item ||
            e.Item.ItemType == Telerik.Web.UI.GridItemType.AlternatingItem)
        {
            if (e.Item.DataItem.GetType() == typeof(Course))
            {

                //Get the course object
                Course _course = (Course)e.Item.DataItem;

                if (_course.CopyHistoryItems.Count > 0)

                    e.Item.Cells[5].Text = "Yes";
                else
                    e.Item.Cells[5].Text = "No";

                e.Item.Cells[2].Style.Add("border-right", "solid 2px #666666");
                e.Item.Cells[3].Style.Add("border-right", "solid 2px #666666");

                //Get references to the controls
                CheckBox _chkSelect = (CheckBox)e.Item.FindControl("chkSelectSelectCheckbox"); 
                Label _lblCourseDisplayCode = (Label)e.Item.FindControl("lblCourseDisplayCode");
                Label _lblCourseTitle = (Label)e.Item.FindControl("lblCourseTitle");
                Label _lblECollegeID = (Label)e.Item.FindControl("lblECollegeID");
                HyperLink _hypInfo = (HyperLink)e.Item.FindControl("hypInfo");
                HyperLink _hypChange = (HyperLink)e.Item.FindControl("hypChange");

                //Set the navigate url for the Change link
                _hypChange.NavigateUrl = "javascript:doSelectMaster('" + _course.CourseCode + "');";
                //Check to see if there is a selected Master Course for this item
                if (_course.SelectedMasterCourse == null)
                {
                    //Set the labels to blank and hide the Info link and the Check box
                    _lblCourseDisplayCode.Text = "";
                    _lblCourseTitle.Text = "";
                    _lblECollegeID.Text = "";
                    _chkSelect.Visible = false;
                }
                else
                {
                    //There is a selected Master Course
                    //Set the labels
                    _lblCourseDisplayCode.Text = _course.SelectedMasterCourse.CourseDisplayCode;
                    _lblCourseTitle.Text = _course.SelectedMasterCourse.CourseTitle;
                    _lblECollegeID.Text = _course.SelectedMasterCourse.eCollegeID;
                    _chkSelect.Checked = _course.ToBeCopied;
                    e.Item.Selected = _course.ToBeCopied;
                }
            }
        }
    }

    protected void ddlSchool_DataBound(object sender, EventArgs e)
    {
        //Add in a Select All item
        ddlSchool.Items.Insert(0, new ListItem("All", ""));
        if (Session[Constants.SessionKeys.SelectedSchool] != null)
        {
            //Set the selected value if necessary
            ddlSchool.SelectedValue = Session[Constants.SessionKeys.SelectedSchool].ToString();
        }

    }

    public Telerik.Web.UI.RadGrid CourseGrid
    {
        get { return this.CoursesGrid; }
    }

    protected void btnCopy_Click(object sender, EventArgs e)
    {
        //enable validator for checking if courses were selected
        this.cvCourseGrid.Enabled = true;
        this.Page.Validate();
        if (this.Page.IsValid)
        {
            _presenter.OnCopyClick(ddlECollegeTerms.SelectedValue, ddlECollegeTerms.SelectedItem.Text);
        }
    }

    protected void ValidateCourseGrid(object sender, ServerValidateEventArgs e)
    {
        e.IsValid = false;
        foreach (Telerik.Web.UI.GridDataItem _row in this.CourseGrid.Items)
        {
            if (_row.Selected)
            {
                e.IsValid = true;
                break;
            }
        }
    }

    protected void DetailTableDataBind(object source, Telerik.Web.UI.GridDetailTableDataBindEventArgs e)
    {
        if (e.DetailTableView.Name == "HistoryDetail")
        {
            GridDataItem dataItem = (GridDataItem)e.DetailTableView.ParentItem;
            string courseCode = dataItem.GetDataKeyValue("CourseCode").ToString();
            _presenter.GetBannerCourseCopyHistory(courseCode, ddlSubTerm.SelectedValue, ddlTerm.SelectedValue);
        }
    }
}


