﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Microsoft.Practices.ObjectBuilder;
using OHE.LCP.Admin.CourseCopying.Views;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying;
using System.Collections.Generic;

public partial class CourseCopying_MasterCourses : System.Web.UI.Page, IMasterCourses
{
    private MasterCoursesPresenter _presenter;
    private string _mode;
    protected void Page_Load(object sender, EventArgs e)
    {
        //Check the session for a warning about inactive master courses with banner courses
        if (Session[Constants.SessionKeys.MasterCourseWarningDisplayCode] != null && Session[Constants.SessionKeys.MasterCourseWarningTitle] != null)
        {
            lblWarning.Text = "WARNING: You have disabled a Master Course that was mapped to at least one Banner Course.  Please make sure that you set up a new Default Mapping for the affected Banner Courses and that you map them to an active Master Course.  The Master Course that you deactivated was " + Session[Constants.SessionKeys.MasterCourseWarningDisplayCode] + " " + Session[Constants.SessionKeys.MasterCourseWarningTitle];
            WarningDiv.Visible = true;
            Session[Constants.SessionKeys.MasterCourseWarningDisplayCode] = null;
            Session[Constants.SessionKeys.MasterCourseWarningTitle] = null;
        }
        else { WarningDiv.Visible = false; }

        //Get the mode from the query string
        _mode = Page.Request.QueryString["mode"];
        switch (_mode)
        {
            case "insert":
                //Set the mode for the details view
                MasterCourseDetailsView.DefaultMode = DetailsViewMode.Insert;
                MasterCourseDetailsView.Visible = true;
                break;
            case "edit":
                //Set the mode for the details view
                MasterCourseDetailsView.DefaultMode = DetailsViewMode.Edit;
                MasterCourseDetailsView.Visible = true;
                //Get the actual message from the presenter
                this._presenter.OnGetMasterCourse(Convert.ToInt32(Page.Request.QueryString["id"]));
                break;
            default:
                MasterCourseDetailsView.DefaultMode = DetailsViewMode.Insert;
                break;
        }

        if (!this.IsPostBack)
        {
            this._presenter.OnViewInitialized();
        }
        this._presenter.OnViewLoaded();
    }

    [CreateNew]
    public MasterCoursesPresenter Presenter
    {
        set
        {
            this._presenter = value;
            this._presenter.View = this;
        }
    }

    public IList<MasterCourse> MasterCourses
    {
        set { MasterCoursesDataSource.DataSource = value; }
    }

    public MasterCourse masterCourse
    {
        set { MasterCourseDataSource.DataSource = value; }
    }

    protected void MasterCourseGrid_ItemDataBound(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {
        if (e.Item.ItemType == Telerik.Web.UI.GridItemType.Item ||
            e.Item.ItemType == Telerik.Web.UI.GridItemType.AlternatingItem)
        {
            //Find the status label
            Label _label = (Label)e.Item.FindControl("lblStatus");
            //Get the master course for this row
            MasterCourse _dataItem = (MasterCourse)e.Item.DataItem;
            //Set the status label appropriately
            if (_dataItem.IsActive)
            { _label.Text = "Active"; }
            else 
            { 
                _label.Text = "Inactive";
                _label.ForeColor = System.Drawing.Color.Red;
            }
        }
    }

    protected void MasterCourseDetailsView_ModeChanging(object sender, DetailsViewModeEventArgs e)
    {
        if (e.CancelingEdit == true)
        {
            MasterCourseDetailsView.Visible = false;
        }
    }

    protected void MasterCourseDataSource_Updated(object sender, Microsoft.Practices.Web.UI.WebControls.ObjectContainerDataSourceStatusEventArgs e)
    {
        MasterCourse _masterCourse = (MasterCourse)e.Instance;
        //Update the course
        _presenter.OnUpdateMasterCourse(_masterCourse);
    }
    protected void MasterCourseDataSource_Inserted(object sender, Microsoft.Practices.Web.UI.WebControls.ObjectContainerDataSourceStatusEventArgs e)
    {
        MasterCourse _masterCourse = (MasterCourse)e.Instance;
        _presenter.OnInsertMasterCourse(_masterCourse);
    }
    protected void MasterCourseDataSource_Updating(object sender, Microsoft.Practices.Web.UI.WebControls.ObjectContainerDataSourceUpdatingEventArgs e)
    {
        //During the update, get the IsActive property from the "old values".  This will be used to see
        //if the warning needs to be displayed
        Session[Constants.SessionKeys.OldIsActive] = Convert.ToBoolean(e.OldValues["IsActive"]);
    }
    protected void MasterCourseDetailsView_ItemUpdated(object sender, DetailsViewUpdatedEventArgs e)
    {
        //This runs after the data source has updated the item 
        MasterCourseDetailsView.Visible =
        false;
        //Check to see if the master course display code has a value. If so, then set the warning values 
        if (Session[Constants.SessionKeys.MasterCourseDisplayCode] != null && Session[Constants.SessionKeys.MasterCourseDisplayCode].ToString() != "")
        {
            Session[
            Constants.SessionKeys.MasterCourseWarningDisplayCode] = Session[Constants.SessionKeys.MasterCourseDisplayCode];
            Session[
            Constants.SessionKeys.MasterCourseWarningTitle] = Session[Constants.SessionKeys.MasterCourseTitle];
            Session[
            Constants.SessionKeys.MasterCourseDisplayCode] = string.Empty;
            Session[
            Constants.SessionKeys.MasterCourseTitle] = string.Empty;
        }
        Response.Redirect(
        Constants.URLs.MasterCourses);
    }
    protected void MasterCourseDetailsView_ItemInserted(object sender, DetailsViewInsertedEventArgs e)
    {
        //This runs after the data source has inserted the item 
        MasterCourseDetailsView.Visible =
        false;
        Response.Redirect(
        Constants.URLs.MasterCourses);
    }
}


