﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Microsoft.Practices.ObjectBuilder;
using OHE.LCP.Admin.CourseCopying.Views;
using System.Collections.Generic;
using OHE.LCP.Admin.CourseCopying.BusinessEntities;
using OHE.LCP.Admin.CourseCopying.Resources;
using System.Text;
using System.IO;
using OHE.LCP.Admin.CourseCopying;

public partial class CourseCopying_NewSections : System.Web.UI.Page, INewSections
{
    private NewSectionsPresenter _presenter;

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            this._presenter.OnViewInitialized();
        }
        this._presenter.OnViewLoaded();
    }

    [CreateNew]
    public NewSectionsPresenter Presenter
    {
        set
        {
            this._presenter = value;
            this._presenter.View = this;
        }
    }

    public Telerik.Web.UI.RadGrid SectionsGrid
    {
        get { return this.NewSectionsGrid; }
    }

    public List<Course> NewSections
    {
        set 
        {
            //This method actually receives a list of Course objects.  However the grid actually
            //requires a list of Section objects.  This was built this way so that we did not have
            //to change a lot of stuff in the way that data is retrieved from the database.  This is the
            //only place in the application where we look at the data at the section level instead of the
            //course level.
            List<Course> _newCourses = value;
            //Put the courses in to session
            Session[Constants.SessionKeys.NewCourses] = _newCourses;
            //Create a new list of sections
            List<Section> _newSections = new List<Section>();
            //Iterate through the courses
            foreach (Course _course in _newCourses)
            {
                //Iterate through the sections in each course
                foreach (Section _section in _course.Sections)
                {
                    //Set the sections course to the current course
                    //This is because the grid needs to display some of the course info
                    if (_section.SectionCourse == null)
                    { _section.SectionCourse = _course; }
                    //Add this section to the list
                    _newSections.Add(_section);
                }
            }
            //If there is at least one section then display the copy button
            if (_newSections.Count < 1)
            {
                btnCopy.Visible = false;
            }


            _newSections.Sort(delegate(Section t1, Section t2) { return String.Format("{0}-{1}", t1.SectionCourse.CourseCode, t1.SectionCode).CompareTo(String.Format("{0}-{1}", t2.SectionCourse.CourseCode, t2.SectionCode)); });

            //Set the grid data source to the list of sections
            NewSectionsDataSource.DataSource = _newSections; 
        }
    }

    protected void NewSectionsGrid_ItemDataBound(object sender, Telerik.Web.UI.GridItemEventArgs e)
    {
        if (e.Item.ItemType == Telerik.Web.UI.GridItemType.Item ||
            e.Item.ItemType == Telerik.Web.UI.GridItemType.AlternatingItem)
        {
            //Get the underlying section object
            Section _section = (Section)e.Item.DataItem;

            //Find the labels
            Label _lblSectionName = (Label)e.Item.FindControl("lblSectionName");
            Label _lblTerm = (Label)e.Item.FindControl("lblTerm");

            //Set the label text
            _lblSectionName.Text = _section.SectionCourse.CourseCode + "-" + _section.SectionCode;
            _lblTerm.Text = _section.SectionTerm.TermDescription;
        }
    }

    protected void btnCopy_Click(object sender, EventArgs e)
    {
        this._presenter.OnCopyNewSections();
    }
}


