﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Microsoft.Practices.ObjectBuilder;
using OHE.LCP.Admin.Messages.Views;
using OHE.LCP.Admin.Messages.BusinessEntities;
using LCPCustomControls;
using Microsoft.Practices.Web.UI.WebControls;
using System.Collections.Generic;
using OHE.FoundationModules.Base.Business;
using OHE.FoundationModules.ProviderServices.BusinessEntities;
using OHE.LCP.Admin.Messages;


public partial class Messages_MessageDetail : System.Web.UI.Page, IMessageDetail
{
    private MessageDetailPresenter _presenter;
    private string _mode;   //The mode can be either Insert or Edit
    private Dictionary<String, List<IBusinessEntity>> _schoolsProgramsConcentrations;
    private Enums.BusinessEntityStructureType _structureType = Enums.BusinessEntityStructureType.SimpleOnly;
   
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            //Get the mode from the query string
            _mode = Page.Request.QueryString["mode"];
            switch (_mode)
            {
                case "insert":
                    //Set the mode for the details view
                    MessageDetailsView.DefaultMode = DetailsViewMode.Insert;
                    break;
                case "edit":
                    //Set the mode for the details view
                    MessageDetailsView.DefaultMode = DetailsViewMode.Edit;

                    //Get the actual message from the presenter
                    this._presenter.OnGetMessage(Convert.ToInt32(Page.Request.QueryString["id"]));
                    break;
                default:
                    MessageDetailsView.DefaultMode = DetailsViewMode.Insert;
                    break;
            }
            this._presenter.OnViewInitialized();

            //Set the source for the roles drop down list
            ListBox _rolesListBox = (ListBox)MessageDetailsView.FindControl("RolesTargetsListBox");
            _rolesListBox.DataSource = System.Web.Security.Roles.Providers["OHELCPRoleProvider"].GetAllRoles();
            _rolesListBox.DataBind();
        }
        this._presenter.OnViewLoaded();

        ClientScriptManager cs = Page.ClientScript;
        if (!cs.IsClientScriptBlockRegistered("openDialogs"))
        {
            //Add in code for javascript for searching for Courses, Sections and People
            //Hidden fields are used to hold the data that comes back from the pop-ups for searching courses, sections and people
            TextBox txtCourses = (TextBox)MessageDetailsView.FindControl("CoursesSearch");
            HiddenField txtCoursesHidden = (HiddenField)MessageDetailsView.FindControl("CoursesHidden");

            TextBox txtSections = (TextBox)MessageDetailsView.FindControl("SectionsSearch");
            HiddenField txtSectionsHidden = (HiddenField)MessageDetailsView.FindControl("SectionsHidden");

            TextBox txtPeople = (TextBox)MessageDetailsView.FindControl("PeopleSearch");
            HiddenField txtPeopleHidden = (HiddenField)MessageDetailsView.FindControl("PeopleHidden");

            //Build the script string for the javascript for searching using modal dialog boxes
            string script = "<script language='javascript' type='text/javascript'>var rc = new Array(0); ";
            script += "function openCourseDialog(){";
            script += "var searchField = document.getElementById('" + txtCourses.ClientID + "');";
            script += "if(searchField.value != '') {";
            script += "var searchVal = searchField.value;";
            script += "searchField.value = '';";
            script += "rc=window.showModalDialog('CourseSearch.aspx?search='+searchVal,'','dialogHeight:500px;dialogWidth:800px;');";
            script += "var hiddenField = document.getElementById('" + txtCoursesHidden.ClientID + "');";
            script += "hiddenField.value = rc[0];}}";
            script += "function openSectionDialog(){";
            script += "var searchField = document.getElementById('" + txtSections.ClientID + "');";
            script += "if(searchField.value != '') {";
            script += "var searchVal = searchField.value;";
            script += "searchField.value = '';";
            script += "rc=window.showModalDialog('SectionSearch.aspx?search='+searchVal,'','dialogHeight:500px;dialogWidth:800px;');";
            script += "var hiddenField = document.getElementById('" + txtSectionsHidden.ClientID + "');";
            script += "hiddenField.value = rc[0];}}";
            script += "function openPeopleDialog(){";
            script += "var searchField = document.getElementById('" + txtPeople.ClientID + "');";
            script += "if(searchField.value != '') {";
            script += "var searchVal = searchField.value;";
            script += "searchField.value = '';";
            script += "rc=window.showModalDialog('PeopleSearch.aspx?search='+searchVal,'','dialogHeight:500px;dialogWidth:800px;');";
            script += "var hiddenField = document.getElementById('" + txtPeopleHidden.ClientID + "');";
            script += "hiddenField.value = rc[0];}}</script>";
            cs.RegisterClientScriptBlock(this.GetType(), "openDialogs", script);
        }

        #region Client Script for Date Validation
        if (!cs.IsClientScriptBlockRegistered("StartEndDateCompare"))
        {
            LCPCustomControls.DropDownDateTime startDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime1");
            LCPCustomControls.DropDownDateTime endDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime2");
            //Build the script string for the javascript for searching using modal dialog boxes
            string script  = "<script language='javascript' type='text/javascript'>";
                   script += "function StartEndDateCompare(sender,args){";
                   script += "args.IsValid = CompareDates('" + startDateTime.ClientID + "','" + endDateTime.ClientID + "');";
                   script += "}</script>";

                   cs.RegisterClientScriptBlock(this.GetType(), "StartEndDateCompare", script);
        }

        if (this.MessageDetailsView.DefaultMode == DetailsViewMode.Insert)
        {
            if (!cs.IsClientScriptBlockRegistered("StartEndCurrentDateCompare"))
            {
                LCPCustomControls.DropDownDateTime startDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime1");
                LCPCustomControls.DropDownDateTime endDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime2");
                //Build the script string for the javascript for searching using modal dialog boxes
                string script = "<script language='javascript' type='text/javascript'>";
                script += "function StartEndCurrentDateCompare(sender,args){";
                // If being inserted then start date must occur after the current date.
                string isValidCondition = "(";
                if (MessageDetailsView.DefaultMode == DetailsViewMode.Insert)
                {
                    isValidCondition += "CompareDates(null,'" + startDateTime.ClientID + "') && ";
                }

                isValidCondition += " CompareDates(null,'" + endDateTime.ClientID + "')";
                isValidCondition += ");";
                script += "args.IsValid = " + isValidCondition;
                script += "}</script>";
                cs.RegisterClientScriptBlock(this.GetType(), "StartEndCurrentDateCompare", script);
            }
        }
        else
        {
            //only validate these dates on the insert
            CustomValidator cvDateCheck2 = (CustomValidator)MessageDetailsView.FindControl("cvDateCheck2");
            cvDateCheck2.Enabled = false;
        }
        #endregion
    }

    /// <summary>
    /// The structure type for the business entities for Schools/Programs/Concentrations
    /// </summary>
    public Enums.BusinessEntityStructureType StructureType
    {
        get 
        {
            return _structureType;
        }
    }

    /// <summary>
    /// A dictionary of business entity lists that holds the Schools/Programs/Concentrations for the drop down lists
    /// </summary>
    public Dictionary<String, List<IBusinessEntity>> SchoolsProgramsConcentrations
    {
        set 
        {
            _schoolsProgramsConcentrations = value;

            //Get references to the list boxes and databind them
            ListBox _schools = (ListBox)MessageDetailsView.FindControl("SchoolsTargetsListBox");
            _schools.DataSource = _schoolsProgramsConcentrations[OHE.FoundationModules.ProviderServices.BusinessEntities.Constants.BusinessEntityNames.Schools];
            _schools.DataBind();

            ListBox _programs = (ListBox)MessageDetailsView.FindControl("ProgramsTargetsListBox");
            _programs.DataSource = _schoolsProgramsConcentrations[OHE.FoundationModules.ProviderServices.BusinessEntities.Constants.BusinessEntityNames.SchoolDegreePrograms];
            _programs.DataBind();

            ListBox _concentrations = (ListBox)MessageDetailsView.FindControl("ConcentrationsTargetsListBox");
            _concentrations.DataSource = _schoolsProgramsConcentrations[OHE.FoundationModules.ProviderServices.BusinessEntities.Constants.BusinessEntityNames.Concentrations];
            _concentrations.DataBind();
        }
    }

    /// <summary>
    /// The underlying message object for the page
    /// </summary>
    public Message message
    {
        set { MessageDataSource.DataSource = value; }
    }

    /// <summary>
    /// The presenter for the page
    /// </summary>
    [CreateNew]
    public MessageDetailPresenter Presenter
    {
        set
        {
            this._presenter = value;
            this._presenter.View = this;
        }
    }

    //Redirect back to the list after an item is inserted
    protected void MessageDetailsView_ItemInserted(object sender, DetailsViewInsertedEventArgs e)
    {
        if (((DetailsView)sender).CurrentMode == DetailsViewMode.Edit)
        {
            Page.Response.Redirect(OHE.LCP.Admin.Messages.Constants.URLs.MessageList + "?messageid=" + ((DetailsView)sender).DataKey.Value.ToString());
        }
    }

    //Redirect back to the list after an item is updated
    protected void MessageDetailsView_ItemUpdated(object sender, DetailsViewUpdatedEventArgs e)
    {
        if (((DetailsView)sender).CurrentMode == DetailsViewMode.Edit)
        {
            Page.Response.Redirect(OHE.LCP.Admin.Messages.Constants.URLs.MessageList + "?messageid=" + ((DetailsView)sender).DataKey.Value.ToString());
        }
    }

    //Redirect back to the list if the user clicked Cancel
    protected void MessageDetailsView_ModeChanging(object sender, DetailsViewModeEventArgs e)
    {
        if (e.CancelingEdit == true)
        {
            Page.Response.Redirect(OHE.LCP.Admin.Messages.Constants.URLs.MessageList + "?messageid=" + ((DetailsView)sender).DataKey.Value.ToString());
        }
    }

    private void AddTargets(Message message)
    {
        //This function adds all of the message targets to the message object
        //This is used to go through the various listboxes and get all of the targets and add them to the message
        MessageTarget _messageTarget;
        ListBox _listBox;

        //Add the roles
        //Get a reference to the roles list box
        _listBox = (ListBox)MessageDetailsView.FindControl("RolesTargetsListBox");
        foreach (ListItem _listItem in _listBox.Items)
        {
            //If it is selected then add it to the targets list
            if (_listItem.Selected)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                _messageTarget = new MessageTarget(0, 0, _listItem.Value, (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), MessageTargetType.Role)),_listItem.Text);
                message.MessageTargets.Add(_messageTarget);
            }
        }

        //Add the schools
        //Get a reference to the schools list box
        _listBox = (ListBox)MessageDetailsView.FindControl("SchoolsTargetsListBox");
        foreach (ListItem _listItem in _listBox.Items)
        {
            //If it is selected then add it to the targets list
            if (_listItem.Selected)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                _messageTarget = new MessageTarget(0, 0, _listItem.Value, (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), MessageTargetType.School)), _listItem.Text);
                message.MessageTargets.Add(_messageTarget);
            }
        }

        //Add the programs
        //Get a reference to the programs list box
        _listBox = (ListBox)MessageDetailsView.FindControl("ProgramsTargetsListBox");
        foreach (ListItem _listItem in _listBox.Items)
        {
            //If it is selected then add it to the targets list
            if (_listItem.Selected)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                _messageTarget = new MessageTarget(0, 0, _listItem.Value, (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), MessageTargetType.Program)), _listItem.Text);
                message.MessageTargets.Add(_messageTarget);
            }
        }

        //Add the concentrations
        //Get a reference to the concentrations list box
        _listBox = (ListBox)MessageDetailsView.FindControl("ConcentrationsTargetsListBox");
        foreach (ListItem _listItem in _listBox.Items)
        {
            //If it is selected then add it to the targets list
            if (_listItem.Selected)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                _messageTarget = new MessageTarget(0, 0, _listItem.Value, (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), MessageTargetType.Concentration)), _listItem.Text);
                message.MessageTargets.Add(_messageTarget);
            }
        } 

        //Add the courses
        //The courses list is held in a hashtable in viewstate
        if (ViewState["CoursesHashTable"] != null)
        {
            Hashtable coursesHashTable = (Hashtable)ViewState["CoursesHashTable"];
            if (coursesHashTable.Count > 0)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                foreach(object key in coursesHashTable.Keys)
                {
                    _messageTarget = new MessageTarget(0,0,key.ToString(),(int)Enum.Parse(typeof(MessageTargetType),Enum.GetName(typeof(MessageTargetType), MessageTargetType.Course)), coursesHashTable[key].ToString());
                    message.MessageTargets.Add(_messageTarget);
                }
            }
        }

        //Add the sections
        //The sections list is held in a hashtable in viewstate
        if (ViewState["SectionsHashTable"] != null)
        {
            Hashtable sectionsHashTable = (Hashtable)ViewState["SectionsHashTable"];
            if (sectionsHashTable.Count > 0)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                foreach (object key in sectionsHashTable.Keys)
                {
                    _messageTarget = new MessageTarget(0, 0, key.ToString(), (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), MessageTargetType.Section)), sectionsHashTable[key].ToString());
                    message.MessageTargets.Add(_messageTarget);
                }
            }
        }

        //Add the people
        //The person list is held in a hashtable in viewstate
        if (ViewState["PeopleHashTable"] != null)
        {
            Hashtable peopleHashTable = (Hashtable)ViewState["PeopleHashTable"];
            if (peopleHashTable.Count > 0)
            {
                if (message.MessageTargets == null)
                {
                    message.MessageTargets = new List<MessageTarget>();
                }
                foreach (object key in peopleHashTable.Keys)
                {
                    _messageTarget = new MessageTarget(0, 0, key.ToString(), (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), MessageTargetType.Person)), peopleHashTable[key].ToString());
                    message.MessageTargets.Add(_messageTarget);
                }
            }
        }
    }

    protected void MessageDataSource_Inserted(object sender, Microsoft.Practices.Web.UI.WebControls.ObjectContainerDataSourceStatusEventArgs e)
    {
        //Before we pass the message object to the presenter we need to add the necessary target objects
        Message _message = (Message)e.Instance;

        //Add the messages
        AddTargets(_message);

        //Set the admin id for the message
        //TODO: this needs to be changed to use the id of the logged in user
        //_message.AdminID = 1;
        _message.CreatedBy = Page.User.Identity.Name;

        //Set the system for the message.  By default all messages generated from this UI are from the LCP
        //TODO: need to put this in a config file or something
        //_message.SystemID = 1;

        _presenter.OnMessageInserted(_message);
    }
    protected void MessageDataSource_Updated(object sender, Microsoft.Practices.Web.UI.WebControls.ObjectContainerDataSourceStatusEventArgs e)
    {
        //Before we pass the message to the presenter we need to add in all of the targets.
        Message _message = (Message)e.Instance;

        //Add all of the targets for the message
        AddTargets(_message);

        _message.LastUpdatedBy = Page.User.Identity.Name;

        _presenter.OnMessageUpdated(_message);
    }

    protected void TargetsListBox_DataBound(object sender, EventArgs e)
    {
        //This is used for all listboxes
        //Get a reference to the listbox
        TargetsListBox _listBox = (TargetsListBox)sender;
        //Get a reference to the underlying message object
        Message _message = (Message)MessageDetailsView.DataItem;

        if (_message != null)
        {
            //Get the targetTypeID for this particular listbox
            int _targetTypeId = (int)Enum.Parse(typeof(MessageTargetType), Enum.GetName(typeof(MessageTargetType), _listBox.TargetType));
            //Convert it to a string
            string _targetType = _targetTypeId.ToString();
            //Create a new MessageTargetIDFilter and MessageTarget object
            MessageTargetIDFilter _filter;
            MessageTarget _target;

            //Iterate through all of the items in the listbox
            foreach (ListItem _listItem in _listBox.Items)
            {
                //The basic idea for this loop is that we are trying to figure out for each
                //item in the list if it is in the list of targets for this type in the message
                //Create the new MessageTargetIDFilter object
                _filter = new MessageTargetIDFilter(_listItem.Value, _targetType);
                //Create the predicate for the filter
                Predicate<MessageTarget> idSearch = new Predicate<MessageTarget>(_filter.Filter);
                //Get a target that matches the filter
                _target = _message.MessageTargets.Find(idSearch);
                //If this is not null then there is a match and we need to make it selected
                if (_target != null)
                {
                    _listItem.Selected = true;
                }
            }
        }
    }

    private class MessageTargetIDFilter
    {
        //This is an internal class used to help filter
        private string _id;
        private string _targetTypeID;

        public MessageTargetIDFilter(string id, string targetTypeID)
        {
            this._id = id;
            this._targetTypeID = targetTypeID;
        }

        public bool Filter(MessageTarget _target)
        {
            if (_target.TargetID == this._id && _target.TargetTypeID.ToString() == this._targetTypeID)
            { return true; }
            else { return false; }
        }
    }

    protected void CoursesButton_Click(object sender, EventArgs e)
    {
        SearchButton_Click("CoursesHidden", "CoursesHashTable");
    }

    protected void SectionsButton_Click(object sender, EventArgs e)
    {
        SearchButton_Click("SectionsHidden", "SectionsHashTable");
    }

    protected void PeopleButton_Click(object sender, EventArgs e)
    {
        SearchButton_Click("PeopleHidden", "PeopleHashTable");
    }

    protected void SearchButton_Click(string HiddenField, string HashTableName)
    {
        //This function looks in the hidden field to see what data came back from the modal dialog
        //If there is anything in here then it should add the data in to view state.
        HiddenField hiddenText = (HiddenField)MessageDetailsView.FindControl(HiddenField);
        if (hiddenText.Value != "")
        {
            //There is some data that came back from the modal dialog
            //The data should be a pipe delimited list of target ids and descriptions for courses
            string delimitedValues = hiddenText.Value;
            hiddenText.Value = "";
            string[] values = delimitedValues.Split(new Char[] { '|' });

            //Get a reference to the hash table
            Hashtable targetsHashTable;
            if (ViewState[HashTableName] == null)
            {
                targetsHashTable = new Hashtable();
            }
            else
            {
                targetsHashTable = (Hashtable)ViewState[HashTableName];
            }

            bool flag = true;
            string key = "";
            string value = "";

            //Move through the array and add the items to the hash table
            for (int i = 0; i < values.Length; i++)
            {
                if (flag)
                {
                    //This item is an id
                    key = values[i];
                    flag = false;
                }
                else
                {
                    //This item is a description
                    value = values[i];

                    //Only add if the item hasn't already been added
                    if (targetsHashTable[key] == null)
                    {
                        //Add both to the hash table
                        targetsHashTable.Add(key, value);
                    }
                    flag = true;
                }
            }
            //Save the hash table to the view state
            ViewState[HashTableName] = targetsHashTable;

            //Do the data binding
            BindGrids();
        }
    }

    protected void BindGrids()
    {
        //This function is in charge of binding the grids for the three lists of targets that are too large for list boxes
        // Courses - Sections - People

        //Get references to the grids
        GridView coursesGrid = (GridView)MessageDetailsView.FindControl("CoursesGrid");
        GridView sectionsGrid = (GridView)MessageDetailsView.FindControl("SectionsGrid");
        GridView peopleGrid = (GridView)MessageDetailsView.FindControl("PeopleGrid");

        // Get the hash tables out of view state
        if (ViewState["CoursesHashTable"] != null)
        {
            Hashtable CoursesHashTable = (Hashtable)ViewState["CoursesHashTable"];
            coursesGrid.DataSource = CoursesHashTable;
            coursesGrid.DataBind();
        }

        if (ViewState["SectionsHashTable"] != null)
        {
            Hashtable SectionsHashTable = (Hashtable)ViewState["SectionsHashTable"];
            sectionsGrid.DataSource = SectionsHashTable;
            sectionsGrid.DataBind();
        }

        if (ViewState["PeopleHashTable"] != null)
        {
            Hashtable PeopleHashTable = (Hashtable)ViewState["PeopleHashTable"];
            peopleGrid.DataSource = PeopleHashTable;
            peopleGrid.DataBind();
        }
    }

    protected void Courses_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Remove")
        {
            //Get a reference to the hash table
            Hashtable coursesHashTable = (Hashtable)ViewState["CoursesHashTable"];

            //Get a reference to the grid
            GridView coursesGrid = (GridView)sender;

            int index = Convert.ToInt32(e.CommandArgument);
            string deleteKey = coursesGrid.DataKeys[index].Value.ToString();

            //Remove the item from the hash table
            coursesHashTable.Remove(deleteKey);

            //Save the hash table back to the view state
            ViewState["CoursesHashTable"] = coursesHashTable;

            //Bind the grids
            BindGrids(); 
        }
    }

    protected void Sections_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Remove")
        {
            //Get a reference to the hash table
            Hashtable sectionsHashTable = (Hashtable)ViewState["SectionsHashTable"];

            //Get a reference to the grid
            GridView sectionsGrid = (GridView)sender;

            int index = Convert.ToInt32(e.CommandArgument);
            string deleteKey = sectionsGrid.DataKeys[index].Value.ToString();

            //Remove the item from the hash table
            sectionsHashTable.Remove(deleteKey);

            //Save the hash table back to the view state
            ViewState["SectionsHashTable"] = sectionsHashTable;

            //Bind the grids
            BindGrids();
        }
    }

    protected void People_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Remove")
        {
            //Get a reference to the hash table
            Hashtable peopleHashTable = (Hashtable)ViewState["PeopleHashTable"];

            //Get a reference to the grid
            GridView peopleGrid = (GridView)sender;

            int index = Convert.ToInt32(e.CommandArgument);
            string deleteKey = peopleGrid.DataKeys[index].Value.ToString();

            //Remove the item from the hash table
            peopleHashTable.Remove(deleteKey);

            //Save the hash table back to the view state
            ViewState["PeopleHashTable"] = peopleHashTable;

            //Bind the grids
            BindGrids();
        }
    }

    protected void LoadHashTables()
    {
        //Get a reference to the underlying message object
        Message _message = (Message)MessageDetailsView.DataItem;

        Hashtable coursesHashTable = null;
        Hashtable sectionsHashTable = null;
        Hashtable peopleHashTable = null;

        if (_message != null)
        {
            if (_message.MessageTargets != null)
            {
                //We need to build three hashtables - one for each type of target - and put them in view state
                foreach (MessageTarget _messageTarget in _message.MessageTargets)
                {
                    switch (_messageTarget.TargetTypeID)
                    {
                        case 1:
                            //Role
                            break;
                        case 2:
                            //School
                            break;
                        case 3:
                            //Program
                            break;
                        case 4:
                            //Concentration
                            break;
                        case 5:
                            //Course
                            if (coursesHashTable == null)
                            { coursesHashTable = new Hashtable(); }
                            coursesHashTable.Add(_messageTarget.TargetID.ToString(), _messageTarget.Description.ToString());
                            break;
                        case 6:
                            //Section
                            if (sectionsHashTable == null)
                            { sectionsHashTable = new Hashtable(); }
                            sectionsHashTable.Add(_messageTarget.TargetID.ToString(), _messageTarget.Description.ToString());
                            break;
                        case 7:
                            //Person
                            if (peopleHashTable == null)
                            { peopleHashTable = new Hashtable(); }
                            peopleHashTable.Add(_messageTarget.TargetID.ToString(), _messageTarget.Description.ToString());
                            break;
                        default:
                            break;
                    }
                }
            }
        }

        if(coursesHashTable != null)
        {ViewState["CoursesHashTable"] = coursesHashTable;}

        if(sectionsHashTable != null)
        {ViewState["SectionsHashTable"] = sectionsHashTable;}

        if(peopleHashTable != null)
        { ViewState["PeopleHashTable"] = peopleHashTable; }
    }

    protected void MessageDetailsView_DataBound(object sender, EventArgs e)
    {
        //Load the hash tables with data for Courses, Sections and People
        LoadHashTables();
        //Bind the grids for Courses, Sections and People
        BindGrids();
    }

    protected void PreviewButton_Click(object sender, EventArgs e)
    {
        //Create a quick message object, populate it with the targets and then get it's recipientsdescription property
        Message _message = new Message();
        this.AddTargets(_message);
        Label _preview = (Label)MessageDetailsView.FindControl("RecipientsLabel");
        _preview.Text = _message.RecipientsDescription;
        Panel _panel = (Panel)MessageDetailsView.FindControl("RecipientsPanel");
        _panel.Visible = true;
    }

    /// <summary>
    /// Compare the Start Date to the End date.  Start date must occur before End Date
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="args"></param>
    protected void StartEndDateCompare(Object sender, ServerValidateEventArgs args)
    {
        args.IsValid = false;
        LCPCustomControls.DropDownDateTime startDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime1");
        LCPCustomControls.DropDownDateTime endDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime2");
        if (DateTime.Parse(startDateTime.Value) < DateTime.Parse(endDateTime.Value))
            args.IsValid = true;
    }

    /// <summary>
    /// Compare the Start/End date to the current date.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="args"></param>
    protected void StartEndCurrentDateCompare(Object sender, ServerValidateEventArgs args)
    {
        args.IsValid = false;
        LCPCustomControls.DropDownDateTime startDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime1");
        LCPCustomControls.DropDownDateTime endDateTime = (LCPCustomControls.DropDownDateTime)MessageDetailsView.FindControl("DropDownDateTime2");
        // If being inserted then both dates must occur after the current date.
        if (MessageDetailsView.DefaultMode == DetailsViewMode.Insert)
        {
            if (DateTime.Parse(startDateTime.Value) >= DateTime.Now && DateTime.Parse(endDateTime.Value) >= DateTime.Now)
                args.IsValid = true;
        }
        // If being updated then only the end date must occur after the current date.
        else
        {
            if (DateTime.Parse(endDateTime.Value) >= DateTime.Now)
                args.IsValid = true;
        }
    }
}


