using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using OHE.Admin.Security.Module.Views;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.Web.UI.WebControls;
using Telerik.Web.UI;
using OHE.Security.DataTypes;
using Microsoft.Practices.CompositeWeb.Interfaces;
using Microsoft.Practices.EnterpriseLibrary.Security;
using OHE.Admin.Security.Module;
    public partial class Security_UserSearch : System.Web.UI.Page, IUserSearchView
    {

        private RadPanelItem _searchPanelItem;
        private RadPanelItem _resultsPanelItem;
        private RadPanelItem _resultsExpanderPanelItem;
        private RadGrid _resultsGrid;
        private TextBox _firstNameTextBox;
        private TextBox _lastNameTextBox;
        private TextBox _emailTextBox;
        private TextBox _userIDTextBox;
        private TextBox _altIDTextBox;
        private UserSearchPresenter _presenter;
        private Label _errorMessage;
        private RadPanelItem _createPanelItem;
        private ValidationSummary _processValidator;
        private String updateOperator;
        


        protected void Page_Load(object sender, EventArgs e)
        {

            _searchPanelItem = this.MainPanelBar.FindItemByValue("Search");
            _resultsPanelItem = this.MainPanelBar.FindItemByValue("Results");
            _resultsExpanderPanelItem = this.MainPanelBar.FindItemByValue("ResultExpander");
            _resultsGrid = (RadGrid)_resultsPanelItem.FindControl("Results");
            _firstNameTextBox = (TextBox)_searchPanelItem.FindControl("FirstNameTextBox");
            _lastNameTextBox = (TextBox)_searchPanelItem.FindControl("LastNameTextBox");
            _altIDTextBox = (TextBox)_searchPanelItem.FindControl("AltIDTextBox");
            _userIDTextBox = (TextBox)_searchPanelItem.FindControl("UserIDTextBox");
            _emailTextBox = (TextBox)_searchPanelItem.FindControl("EmailTextBox");
            _errorMessage = this.ErrorMessage;
            _createPanelItem = this.MainPanelBar.FindItemByValue("Create");
            _processValidator = this.ProcessValidator;
            //updateOperator;
            
            if (!this.IsPostBack)
            {

                //Call the presenter's initialized
                this._presenter.OnViewInitialized();
            }
            //Call the presenter's loaded
            this._presenter.OnViewLoaded();
            
            //Get current username
            updateOperator = Membership.GetUser().UserName.ToString();

            this.UserDataSource.TotalRowCount = 0;
            this.InstitutionsDataSource.DataSource = _presenter.GetAllInstitutions();
            this.SecurityContextDataSource.DataSource = _presenter.GetAllSecurityContexts();            
            
        }

        [CreateNew]
        public UserSearchPresenter Presenter
        {
            get
            {
                return _presenter;
            }
            set
            {
                _presenter = value;
                _presenter.View = this;
            }
        }

        
        protected void SearchButton_Click(object sender, EventArgs e)
        {
            this.OnSelecting(0, _resultsGrid.PageSize);
            _resultsExpanderPanelItem.Expanded = true;
        }

        protected void ResultsDataSource_Selecting(object sender, ObjectContainerDataSourceSelectingEventArgs e)
        {
            if (Page.IsPostBack && Page.Request.Form["ctl00$DefaultContent$SearchButton"] == null)
            {
                this.OnSelecting(e.Arguments.StartRowIndex, e.Arguments.MaximumRows);
            }
        }

        private void OnSelecting(int startRowIndex, int maximumRows)
        {
            OHE.Security.DataTypes.UserSearchCriteria criteria = new OHE.Security.DataTypes.UserSearchCriteria();

            criteria.FirstName = _firstNameTextBox.Text;
            criteria.LastName = _lastNameTextBox.Text;
            criteria.Email = _emailTextBox.Text;
            criteria.ID = _altIDTextBox.Text;

            // validation code

            if (!String.IsNullOrEmpty(_userIDTextBox.Text))
            {
                criteria.UserID = System.Convert.ToInt32(_userIDTextBox.Text);
            }
            _presenter.OnSelecting(criteria, startRowIndex, maximumRows);
        }

        protected void DetailTableDataBind(object source, Telerik.Web.UI.GridDetailTableDataBindEventArgs e)
        {
            if (e.DetailTableView.Name == "SecurityIdentityDetail")
            {
                                
                GridDataItem dataItem = (GridDataItem)e.DetailTableView.ParentItem;
                int userId = Convert.ToInt32(dataItem.GetDataKeyValue("ID").ToString());
                _presenter.OnDetailSelecting(userId);
                
                Cache.Add("institutions", this._presenter.GetAllInstitutions(), null, System.Web.Caching.Cache.NoAbsoluteExpiration, System.Web.Caching.Cache.NoSlidingExpiration, System.Web.Caching.CacheItemPriority.Normal, null);
                Cache.Add("securityContexts", this._presenter.GetAllSecurityContexts(), null, System.Web.Caching.Cache.NoAbsoluteExpiration, System.Web.Caching.Cache.NoSlidingExpiration, System.Web.Caching.CacheItemPriority.Normal, null);
                                
            }
        }

        protected void Results_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "ResetPassword")
            {
                string userName = e.Item.OwnerTableView.Items[0].GetDataKeyValue("UserName").ToString();
                Institution institution = (Institution)e.Item.OwnerTableView.Items[0].GetDataKeyValue("Institution");
                int contextid = Convert.ToInt32(e.Item.OwnerTableView.Items[0].GetDataKeyValue("SecurityContextId"));
                bool result = _presenter.ResetPassword(institution, userName, contextid);

                if (!result)
                {
                    UpdateErrorMessage("Password failed to reset.", true, true);
                    e.Canceled = true;
                    return;
                }
                else
                    UpdateErrorMessage("Password reset.", true, false);
            }
            else if (e.CommandName == "Cancel")
            {
                UpdateErrorMessage(string.Empty, false, false);                
            }
            else if (e.CommandName == "Update" && ((System.Web.UI.WebControls.Button)(e.CommandSource)).CommandArgument.Equals("Move"))                
            {
                MoveCredential(e);                   
            }
        }

        private void UpdateErrorMessage(string message, bool visible, bool failed)
        {
            _errorMessage.Visible = visible;
            _errorMessage.Text = "<br />&nbsp;" + message + "<br /><br />";

            if (failed)
            {
                _errorMessage.ForeColor = System.Drawing.Color.Red;
            }
            else
            {
                _errorMessage.ForeColor = System.Drawing.Color.Green;
            }
        }

        protected void InsertUserAndIdentity(object sender, GridCommandEventArgs e)
        {
            GridEditableItem editedItem = e.Item as GridEditableItem;
            GridDataItem parentItem = (GridDataItem)e.Item.OwnerTableView.ParentItem;
            UserControl userControl = (UserControl)e.Item.FindControl(GridEditFormItem.EditFormUserControlID);
            List<string> results = new List<string>();
            String editionType = (userControl.FindControl("EditionType") as Label).Text;

            if (editionType.Equals("Identity"))
            {
                OHE.Security.DataTypes.User user = new OHE.Security.DataTypes.User();
                OHE.Security.DataTypes.Identity identity = new OHE.Security.DataTypes.Identity();

                user.Email = (string)parentItem.OwnerTableView.DataKeyValues[parentItem.ItemIndex]["Email"];
                user.FirstName = (string)parentItem.OwnerTableView.DataKeyValues[parentItem.ItemIndex]["FirstName"];
                user.LastName = (string)parentItem.OwnerTableView.DataKeyValues[parentItem.ItemIndex]["LastName"];
                user.CreateOperator = updateOperator;
                user.UpdateOperator = updateOperator;

                results = InsertIdentity(user, userControl);
                

            }
            if (editionType.Equals("User"))
            {
                OHE.Security.DataTypes.User user = new OHE.Security.DataTypes.User();
                OHE.Security.DataTypes.Identity olaIdentity = new OHE.Security.DataTypes.Identity();

                user.FirstName = ((userControl).FindControl("FirstName") as TextBox).Text;
                user.LastName = ((userControl).FindControl("LastName") as TextBox).Text;
                user.Email = ((userControl).FindControl("Email") as TextBox).Text;
                user.CreateOperator = updateOperator;
                user.UpdateOperator = updateOperator;

                olaIdentity.Password = ((userControl).FindControl("Password") as TextBox).Text;
                olaIdentity.SecurityContextId = Convert.ToInt32(ConfigurationManager.AppSettings["OLASystemId"]);
                olaIdentity.CreateOperator = updateOperator;
                olaIdentity.UpdateOperator = updateOperator;

                if (!String.IsNullOrEmpty(((userControl).FindControl("SystemUserId") as TextBox).Text))
                {
                    olaIdentity.SystemUserID = Convert.ToInt32(((userControl).FindControl("SystemUserId") as TextBox).Text);
                }
                if (!String.IsNullOrEmpty(((userControl).FindControl("OLAId") as TextBox).Text))
                {
                    olaIdentity.UserName = ((userControl).FindControl("OLAId") as TextBox).Text;
                }
                else
                {
                    olaIdentity.UserName = ((userControl).FindControl("Email") as TextBox).Text;
                }

                user.Identities.Add(olaIdentity);

                results = _presenter.CreateUser(user);

            }

            _errorMessage.Visible = true;

            if (results.Count > 0 && results[0].Equals("Success"))
            {
                UpdateErrorMessage("Record Created.", true, false);
            }
            else
            {
                UpdateErrorMessage("Error Creating Record: " + results[0], true, true);
            }

        }

        private void MoveCredential(GridCommandEventArgs e)
        {
            UserControl userControl = (UserControl)e.Item.FindControl(GridEditFormItem.EditFormUserControlID);
            if(!String.IsNullOrEmpty(((userControl).FindControl("MoveUserId") as TextBox).Text))
            {
                OHE.Security.DataTypes.UserSearchCriteria criteria = new OHE.Security.DataTypes.UserSearchCriteria();
                criteria.UserID = Convert.ToInt32(((userControl).FindControl("MoveUserId") as TextBox).Text);
                List<User> users = _presenter.FindUsersBySearchCriteria(criteria, 0, 2);

                if(users.Count ==1)
                {
                    User user = users[0];
                    user.Identities.Clear();
                    List<string> results = InsertIdentity(user, userControl);

                    _errorMessage.Visible = true;

                    if (results[0].Equals("Success"))
                    {
                        UpdateErrorMessage("Record Moved.", true, false);
                    }
                    else
                    {
                        UpdateErrorMessage("Failed to move.", true, true);
                        e.Canceled = true;
                        return;
                    }                    
                }               
            }
        }

        private List<string> InsertIdentity(User user, UserControl userControl)
        {
            List<string> results = new List<string>();
            OHE.Security.DataTypes.Identity identity = new Identity();
            int systemUserID;
            bool result = Int32.TryParse((userControl.FindControl("SystemUserID") as TextBox).Text, out systemUserID);
            if (!result) { systemUserID = 0; }

            identity.Password = ((userControl).FindControl("Password") as TextBox).Text;
            identity.SecurityContextId = Convert.ToInt32((userControl.FindControl("SecurityContext") as DropDownList).SelectedValue);
            identity.UserName = ((userControl).FindControl("UserName") as TextBox).Text;
            identity.SystemUserID = systemUserID;
            identity.Hint = ((userControl).FindControl("Hint") as TextBox).Text;
            identity.HintAnswer = ((userControl).FindControl("HintAnswer") as TextBox).Text;
            identity.ActiveStatus = (userControl.FindControl("ActiveFlag") as CheckBox).Checked;
            identity.IsLockedOut = (userControl.FindControl("IsLockedOut") as CheckBox).Checked;
            identity.CreateOperator = updateOperator;
            identity.UpdateOperator = updateOperator;
            Institution institution = new Institution();
            institution.ID = Convert.ToInt32((userControl.FindControl("Institution") as DropDownList).SelectedValue);
            identity.Institution = institution;
            user.Identities.Add(identity);
            return _presenter.CreateUser(user);

        }


        protected void UpdateSecurityIdentity(object sender, GridCommandEventArgs e)
        {
            if (!((System.Web.UI.WebControls.Button)(e.CommandSource)).CommandArgument.Equals("Move"))
            {
                GridEditableItem editedItem = e.Item as GridEditableItem;
                GridDataItem parentItem = (GridDataItem)e.Item.OwnerTableView.ParentItem;
                UserControl userControl = (UserControl)e.Item.FindControl(GridEditFormItem.EditFormUserControlID);
                // (userControl.FindControl("asdf") as ObjectContainerDataSource).DataSource;
                int systemUserID;
                bool result = Int32.TryParse((userControl.FindControl("SystemUserID") as TextBox).Text, out systemUserID);

                if (result)
                {

                    bool active = (userControl.FindControl("ActiveFlag") as CheckBox).Checked;
                    //int securityConextID = Convert.ToInt32((userControl.FindControl("SecurityContextID") as TextBox).Text);
                    int securityConextID = Convert.ToInt32((userControl.FindControl("SecurityContext") as DropDownList).SelectedValue); //(int)editedItem.OwnerTableView.DataKeyValues[editedItem.ItemIndex]["SecurityContextId"];
                    Institution institution = new Institution();// (Institution)editedItem.OwnerTableView.DataKeyValues[editedItem.ItemIndex]["Institution"];
                    institution.ID = Convert.ToInt32((userControl.FindControl("Institution") as DropDownList).SelectedValue);
                    string username = ((userControl).FindControl("UserName") as TextBox).Text; //(string)editedItem.OwnerTableView.DataKeyValues[editedItem.ItemIndex]["UserName"];
                    int userid = (int)parentItem.OwnerTableView.DataKeyValues[parentItem.ItemIndex]["ID"];
                    string hint = ((userControl).FindControl("Hint") as TextBox).Text;
                    //string hint = (string)editedItem.OwnerTableView.DataKeyValues[editedItem.ItemIndex]["Hint"];
                    string hintAnswer = ((userControl).FindControl("HintAnswer") as TextBox).Text; //(string)editedItem.OwnerTableView.DataKeyValues[editedItem.ItemIndex]["HintAnswer"];
                    bool isLockedOut = (userControl.FindControl("IsLockedOut") as CheckBox).Checked; //(bool)editedItem.OwnerTableView.DataKeyValues[editedItem.ItemIndex]["IsLockedOut"];
                    string password = ((userControl).FindControl("Password") as TextBox).Text;
                    string oldPassword = (string)e.Item.OwnerTableView.DataKeyValues[e.Item.ItemIndex]["Password"];
                    bool update = _presenter.UpdatePassword(username, password, oldPassword, securityConextID);
                    result = _presenter.UpdateSecurityIdentity(institution, userid, systemUserID, securityConextID, active, hint, hintAnswer, isLockedOut, username, updateOperator);

                }

                _errorMessage.Visible = true;

                if (!result)
                {
                    UpdateErrorMessage("Failed to update.", true, true);
                    e.Canceled = true;
                    return;
                }
                else
                {
                    UpdateErrorMessage("Record Updated.", true, false);
                }
            }
        }

        public bool CanEdit
        {
            get
            {
                //            IAuthorizationRulesService authorizationRuleService = services.Get<IAuthorizationRulesService>();
                //            AuthorizationRuleProvider rulesProvider = new AuthorizationRuleProvider(authorizationRuleService.GetAuthorizationRules);
                //            return rulesProvider.Authorize(this.User, Constants.Permissions.CanEditSecurityIdentity);
                
                return false;
            }
        }


        #region IUserSearchView Members

        public System.Collections.Generic.IList<OHE.Security.DataTypes.User> Users
        {
            set { this.UserDataSource.DataSource = value; }
        }

        public System.Collections.Generic.IList<OHE.Security.DataTypes.Identity> Identities
        {
            set { this.IdentitiesDataSource.DataSource = value; }
        }

        public int TotalUsersCount
        {
            set { this.UserDataSource.TotalRowCount = value; }
        }

        #endregion        
        

        public List<Institution> GetAllInstitutions()
        {
            return _presenter.GetAllInstitutions();
        }
        public List<SecurityContext> GetAllSecurityContexts()
        {
            return _presenter.GetAllSecurityContexts();
        }
    }


